@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,16 +52,16 @@
 
 package org.mariadb.jdbc;
 
-import org.mariadb.jdbc.internal.*;
-import org.mariadb.jdbc.internal.com.read.resultset.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-
-import java.io.*;
-import java.math.*;
-import java.net.*;
-import java.sql.Date;
+import java.io.InputStream;
+import java.io.Reader;
+import java.math.BigDecimal;
+import java.net.URL;
 import java.sql.*;
-import java.util.*;
+import java.util.Calendar;
+import java.util.Map;
+import org.mariadb.jdbc.internal.ColumnType;
+import org.mariadb.jdbc.internal.com.read.resultset.SelectResultSet;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
 
 public abstract class CallableFunctionStatement extends ClientSidePreparedStatement
     implements CallableStatement {
@@ -79,12 +79,23 @@ public abstract class CallableFunctionStatement extends ClientSidePreparedStatem
    *     ResultSet.TYPE_SCROLL_INSENSITIVE</code>, or <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
    * @param resultSetConcurrency a concurrency type; one of <code>ResultSet.CONCUR_READ_ONLY</code>
    *     or <code>ResultSet.CONCUR_UPDATABLE</code>
+   * @param exceptionFactory Exception factory
    * @throws SQLException if clientPrepareStatement creation throw an exception
    */
   public CallableFunctionStatement(
-      MariaDbConnection connection, String sql, int resultSetType, final int resultSetConcurrency)
+      MariaDbConnection connection,
+      String sql,
+      int resultSetType,
+      final int resultSetConcurrency,
+      ExceptionFactory exceptionFactory)
       throws SQLException {
-    super(connection, sql, resultSetType, resultSetConcurrency, Statement.NO_GENERATED_KEYS);
+    super(
+        connection,
+        sql,
+        resultSetType,
+        resultSetConcurrency,
+        Statement.NO_GENERATED_KEYS,
+        exceptionFactory);
   }
 
   /**
@@ -143,7 +154,7 @@ public abstract class CallableFunctionStatement extends ClientSidePreparedStatem
         return i;
       }
     }
-    throw new SQLException("there is no parameter with the name " + parameterName);
+    throw exceptionFactory.create("there is no parameter with the name " + parameterName);
   }
 
   /**
@@ -160,7 +171,7 @@ public abstract class CallableFunctionStatement extends ClientSidePreparedStatem
         return i;
       }
     }
-    throw new SQLException("there is no parameter with the name " + parameterName);
+    throw exceptionFactory.create("there is no parameter with the name " + parameterName);
   }
 
   /**
@@ -437,12 +448,12 @@ public abstract class CallableFunctionStatement extends ClientSidePreparedStatem
 
   @Override
   public RowId getRowId(int parameterIndex) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("RowIDs not supported");
+    throw exceptionFactory.notSupported("RowIDs not supported");
   }
 
   @Override
   public RowId getRowId(String parameterName) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("RowIDs not supported");
+    throw exceptionFactory.notSupported("RowIDs not supported");
   }
 
   @Override
@@ -457,12 +468,12 @@ public abstract class CallableFunctionStatement extends ClientSidePreparedStatem
 
   @Override
   public SQLXML getSQLXML(int parameterIndex) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("SQLXML not supported");
+    throw exceptionFactory.notSupported("SQLXML not supported");
   }
 
   @Override
   public SQLXML getSQLXML(String parameterName) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("SQLXML not supported");
+    throw exceptionFactory.notSupported("SQLXML not supported");
   }
 
   @Override
@@ -616,19 +627,19 @@ public abstract class CallableFunctionStatement extends ClientSidePreparedStatem
 
   private CallParameter getParameter(int index) throws SQLException {
     if (index > params.length || index <= 0) {
-      throw new SQLException("No parameter with index " + (index));
+      throw exceptionFactory.create("No parameter with index " + (index));
     }
     return params[index - 1];
   }
 
   @Override
   public void setSQLXML(String parameterName, SQLXML xmlObject) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("SQLXML not supported");
+    throw exceptionFactory.notSupported("SQLXML not supported");
   }
 
   @Override
   public void setRowId(String parameterName, RowId rowid) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("RowIDs not supported");
+    throw exceptionFactory.notSupported("RowIDs not supported");
   }
 
   @Override
