@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,15 +52,16 @@
 
 package org.mariadb.jdbc;
 
-import org.mariadb.jdbc.internal.com.read.dao.*;
-import org.mariadb.jdbc.internal.com.read.resultset.*;
-import org.mariadb.jdbc.internal.com.send.parameters.*;
-import org.mariadb.jdbc.internal.logging.*;
-import org.mariadb.jdbc.internal.util.dao.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-
 import java.sql.*;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.List;
+import org.mariadb.jdbc.internal.com.read.dao.Results;
+import org.mariadb.jdbc.internal.com.read.resultset.SelectResultSet;
+import org.mariadb.jdbc.internal.com.send.parameters.ParameterHolder;
+import org.mariadb.jdbc.internal.logging.Logger;
+import org.mariadb.jdbc.internal.logging.LoggerFactory;
+import org.mariadb.jdbc.internal.util.dao.ClientPrepareResult;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
 
 public class ClientSidePreparedStatement extends BasePrepareStatement {
 
@@ -84,6 +85,7 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
    *     or <code>ResultSet.CONCUR_UPDATABLE</code>
    * @param autoGeneratedKeys a flag indicating whether auto-generated keys should be returned; one
    *     of <code>Statement.RETURN_GENERATED_KEYS</code> or <code>Statement.NO_GENERATED_KEYS</code>
+   * @param exceptionFactory exception factory
    * @throws SQLException exception
    */
   public ClientSidePreparedStatement(
@@ -91,9 +93,11 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
       String sql,
       int resultSetScrollType,
       int resultSetConcurrency,
-      int autoGeneratedKeys)
+      int autoGeneratedKeys,
+      ExceptionFactory exceptionFactory)
       throws SQLException {
-    super(connection, resultSetScrollType, resultSetConcurrency, autoGeneratedKeys);
+    super(
+        connection, resultSetScrollType, resultSetConcurrency, autoGeneratedKeys, exceptionFactory);
     sqlQuery = sql;
 
     if (options.rewriteBatchedStatements) {
@@ -166,11 +170,12 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
    * Executes the SQL statement in this <code>PreparedStatement</code> object, which must be an SQL
    * Data Manipulation Language (DML) statement, such as <code>INSERT</code>, <code>UPDATE</code> or
    * <code>DELETE</code>; or an SQL statement that returns nothing, such as a DDL statement.
+   * Result-set are permitted for historical reason, even if spec indicate to throw exception.
    *
    * @return either (1) the row count for SQL Data Manipulation Language (DML) statements or (2) 0
    *     for SQL statements that return nothing
    * @throws SQLException if a database access error occurs; this method is called on a closed
-   *     <code>PreparedStatement</code> or the SQL statement returns a <code>ResultSet</code> object
+   *     <code>PreparedStatement</code>
    */
   public int executeUpdate() throws SQLException {
     if (execute()) {
@@ -185,10 +190,9 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
     for (int i = 0; i < prepareResult.getParamCount(); i++) {
       if (parameters[i] == null) {
         logger.error("Parameter at position {} is not set", (i + 1));
-        ExceptionMapper.throwException(
-            new SQLException("Parameter at position " + (i + 1) + " is not set", "07004"),
-            connection,
-            this);
+        throw exceptionFactory
+            .raiseStatementError(connection, this)
+            .create("Parameter at position " + (i + 1) + " is not set", "07004");
       }
     }
 
@@ -247,10 +251,12 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
             "You need to set exactly "
                 + prepareResult.getParamCount()
                 + " parameters on the prepared statement");
-        throw ExceptionMapper.getSqlException(
-            "You need to set exactly "
-                + prepareResult.getParamCount()
-                + " parameters on the prepared statement");
+        throw exceptionFactory
+            .raiseStatementError(connection, this)
+            .create(
+                "You need to set exactly "
+                    + prepareResult.getParamCount()
+                    + " parameters on the prepared statement");
       }
     }
     parameterList.add(holder);
@@ -264,14 +270,15 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
    */
   @Override
   public void addBatch(final String sql) throws SQLException {
-    throw new SQLException("Cannot do addBatch(String) on preparedStatement");
+    throw exceptionFactory
+        .raiseStatementError(connection, this)
+        .create("Cannot do addBatch(String) on preparedStatement");
   }
 
   /** Clear batch. */
   @Override
   public void clearBatch() {
     parameterList.clear();
-    hasLongData = false;
     this.parameters = new ParameterHolder[prepareResult.getParamCount()];
   }
 
@@ -360,7 +367,7 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
             null,
             null);
     if (protocol.executeBatchClient(
-        protocol.isMasterConnection(), results, prepareResult, parameterList, hasLongData)) {
+        protocol.isMasterConnection(), results, prepareResult, parameterList)) {
       return;
     }
 
@@ -471,7 +478,7 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
       }
 
       logger.error(error);
-      throw ExceptionMapper.getSqlException(error);
+      throw exceptionFactory.raiseStatementError(connection, this).create(error);
     }
   }
 
@@ -502,14 +509,12 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
             sqlQuery,
             ResultSet.TYPE_SCROLL_INSENSITIVE,
             ResultSet.CONCUR_READ_ONLY,
-            Statement.NO_GENERATED_KEYS)) {
+            Statement.NO_GENERATED_KEYS,
+            exceptionFactory)) {
       resultSetMetaData = ssps.getMetaData();
       parameterMetaData = ssps.getParameterMetaData();
-    } catch (SQLSyntaxErrorException sqlSyntaxErrorException) {
-      // if error is due to wrong SQL syntax, better to throw exception immediately
-      throw sqlSyntaxErrorException;
-    } catch (SQLException sqle) {
-      parameterMetaData = new MariaDbParameterMetaData(null);
+    } catch (Exception exception) {
+      parameterMetaData = new SimpleParameterMetaData(prepareResult.getParamCount());
     }
   }
 
@@ -529,13 +534,6 @@ public class ClientSidePreparedStatement extends BasePrepareStatement {
   @Override
   public void close() throws SQLException {
     super.close();
-    if (connection == null
-        || connection.pooledConnection == null
-        || connection.pooledConnection.noStmtEventListeners()) {
-      return;
-    }
-    connection.pooledConnection.fireStatementClosed(this);
-    connection = null;
   }
 
   protected int getParameterCount() {
