@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,22 +52,26 @@
 
 package org.mariadb.jdbc;
 
-import org.mariadb.jdbc.internal.com.read.dao.*;
-import org.mariadb.jdbc.internal.com.read.resultset.*;
-import org.mariadb.jdbc.internal.logging.*;
-import org.mariadb.jdbc.internal.protocol.*;
-import org.mariadb.jdbc.internal.util.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-import org.mariadb.jdbc.internal.util.scheduler.*;
-import org.mariadb.jdbc.util.*;
-
-import java.io.*;
-import java.nio.charset.*;
+import java.io.InputStream;
+import java.nio.charset.Charset;
 import java.sql.*;
 import java.util.*;
-import java.util.concurrent.*;
-import java.util.concurrent.locks.*;
-import java.util.regex.*;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.Future;
+import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.locks.ReentrantLock;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+import org.mariadb.jdbc.internal.com.read.dao.Results;
+import org.mariadb.jdbc.internal.com.read.resultset.SelectResultSet;
+import org.mariadb.jdbc.internal.logging.Logger;
+import org.mariadb.jdbc.internal.logging.LoggerFactory;
+import org.mariadb.jdbc.internal.protocol.Protocol;
+import org.mariadb.jdbc.internal.util.Utils;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
+import org.mariadb.jdbc.internal.util.scheduler.SchedulerServiceProviderHolder;
+import org.mariadb.jdbc.util.Options;
 
 public class MariaDbStatement implements Statement, Cloneable {
 
@@ -106,6 +110,7 @@ public class MariaDbStatement implements Statement, Cloneable {
   protected Results results;
   protected int fetchSize;
   protected volatile boolean executing;
+  protected ExceptionFactory exceptionFactory;
   private ScheduledExecutorService timeoutScheduler;
   // are warnings cleared?
   private boolean warningsCleared;
@@ -114,6 +119,7 @@ public class MariaDbStatement implements Statement, Cloneable {
   private Future<?> timerTaskFuture;
   private boolean isTimedout;
   private int maxFieldSize;
+  private boolean escape = true;
 
   /**
    * Creates a new Statement.
@@ -124,9 +130,13 @@ public class MariaDbStatement implements Statement, Cloneable {
    *     <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
    * @param resultSetConcurrency a concurrency type; one of <code>ResultSet.CONCUR_READ_ONLY</code>
    *     or <code>ResultSet.CONCUR_UPDATABLE</code>
+   * @param exceptionFactory exception factory
    */
   public MariaDbStatement(
-      MariaDbConnection connection, int resultSetScrollType, int resultSetConcurrency) {
+      MariaDbConnection connection,
+      int resultSetScrollType,
+      int resultSetConcurrency,
+      ExceptionFactory exceptionFactory) {
     this.protocol = connection.getProtocol();
     this.connection = connection;
     this.canUseServerTimeout = connection.canUseServerTimeout();
@@ -134,6 +144,7 @@ public class MariaDbStatement implements Statement, Cloneable {
     this.resultSetConcurrency = resultSetConcurrency;
     this.lock = this.connection.lock;
     this.options = this.protocol.getOptions();
+    this.exceptionFactory = exceptionFactory;
     this.fetchSize = this.options.defaultFetchSize;
   }
 
@@ -154,7 +165,9 @@ public class MariaDbStatement implements Statement, Cloneable {
     clone.warningsCleared = true;
     clone.maxRows = 0;
     clone.fetchSize = this.options.defaultFetchSize;
-
+    clone.exceptionFactory =
+        ExceptionFactory.of(
+            this.exceptionFactory.getThreadId(), this.exceptionFactory.getOptions());
     return clone;
   }
 
@@ -195,7 +208,9 @@ public class MariaDbStatement implements Statement, Cloneable {
   protected void executeQueryPrologue(boolean isBatch) throws SQLException {
     executing = true;
     if (closed) {
-      throw new SQLException("execute() is called on closed statement");
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("execute() is called on closed statement");
     }
     protocol.prolog(maxRows, protocol.getProxy() != null, connection, this);
     if (queryTimeout != 0 && (!canUseServerTimeout || isBatch)) {
@@ -240,22 +255,23 @@ public class MariaDbStatement implements Statement, Cloneable {
     }
 
     if (sqle.getErrorCode() == 1148 && !options.allowLocalInfile) {
-      return new SQLFeatureNotSupportedException(
-          "(conn:"
-              + getServerThreadId()
-              + ") Usage of LOCAL INFILE is disabled. "
-              + "To use it enable it via the connection property allowLocalInfile=true",
-          "42000",
-          1148,
-          sqle);
+      return exceptionFactory
+          .raiseStatementError(connection, this)
+          .create(
+              "Usage of LOCAL INFILE is disabled. "
+                  + "To use it enable it via the connection property allowLocalInfile=true",
+              "42000",
+              1148,
+              sqle);
     }
 
     if (isTimedout) {
-      return new SQLTimeoutException(
-          "(conn:" + getServerThreadId() + ") Query timed out", "JZ0002", 1317, sqle);
+      return exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("Query timed out", "70100", 1317, sqle);
     }
-    SQLException sqlException =
-        ExceptionMapper.getException(sqle, connection, this, queryTimeout != 0);
+
+    SQLException sqlException = exceptionFactory.raiseStatementError(connection, this).create(sqle);
     logger.error("error executing query", sqlException);
     return sqlException;
   }
@@ -285,9 +301,9 @@ public class MariaDbStatement implements Statement, Cloneable {
     }
 
     if (isTimedout) {
-      sqle =
-          new SQLTimeoutException(
-              "(conn:" + getServerThreadId() + ") Query timed out", "JZ0002", 1317, sqle);
+      return exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("Query timed out", "70100", 1317, sqle);
     }
     return sqle;
   }
@@ -301,8 +317,7 @@ public class MariaDbStatement implements Statement, Cloneable {
     } else {
       ret = results.getCmdInformation().getUpdateCounts();
     }
-
-    sqle = ExceptionMapper.getException(sqle, connection, this, queryTimeout != 0);
+    sqle = exceptionFactory.raiseStatementError(connection, this).create(sqle);
     logger.error("error executing query", sqle);
 
     return new BatchUpdateException(
@@ -339,9 +354,8 @@ public class MariaDbStatement implements Statement, Cloneable {
               protocol.getAutoIncrementIncrement(),
               sql,
               null);
-
       protocol.executeQuery(
-          protocol.isMasterConnection(), results, getTimeoutSql(Utils.nativeSql(sql, protocol)));
+          protocol.isMasterConnection(), results, getTimeoutSql(nativeSql(sql, protocol)));
       results.commandEnd();
       return results.getResultSet() != null;
 
@@ -387,7 +401,9 @@ public class MariaDbStatement implements Statement, Cloneable {
       return alwaysQuote ? "`" + identifier + "`" : identifier;
     } else {
       if (identifier.contains("\u0000")) {
-        throw new SQLException("Invalid name - containing u0000 character");
+        throw exceptionFactory
+            .raiseStatementError(connection, this)
+            .create("Invalid name - containing u0000 character", "42000");
       }
 
       if (identifier.matches("^`.+`$")) {
@@ -431,6 +447,10 @@ public class MariaDbStatement implements Statement, Cloneable {
     return sql;
   }
 
+  private String nativeSql(String sql, Protocol protocol) throws SQLException {
+    return escape ? Utils.nativeSql(sql, protocol) : sql;
+  }
+
   /**
    * ! This method is for test only ! This permit sending query using specific charset.
    *
@@ -458,10 +478,7 @@ public class MariaDbStatement implements Statement, Cloneable {
               sql,
               null);
       protocol.executeQuery(
-          protocol.isMasterConnection(),
-          results,
-          getTimeoutSql(Utils.nativeSql(sql, protocol)),
-          charset);
+          protocol.isMasterConnection(), results, getTimeoutSql(nativeSql(sql, protocol)), charset);
       results.commandEnd();
       return results.getResultSet() != null;
 
@@ -599,7 +616,8 @@ public class MariaDbStatement implements Statement, Cloneable {
   }
 
   /**
-   * Executes an update.
+   * Executes an update. Result-set are permitted for historical reason, even if spec indicate to
+   * throw exception.
    *
    * @param sql the update query.
    * @return update count
@@ -617,7 +635,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * auto-generated keys produced by this <code>Statement</code> object should be made available for
    * retrieval. The driver will ignore the flag if the SQL statement is not an <code>INSERT</code>
    * statement, or an SQL statement able to return auto-generated keys (the list of such statements
-   * is vendor-specific).
+   * is vendor-specific). Result-set are permitted for historical reason, even if spec indicate to
+   * throw exception.
    *
    * @param sql an SQL Data Manipulation Language (DML) statement, such as <code>INSERT</code>,
    *     <code>UPDATE</code> or <code>DELETE</code>; or an SQL statement that returns nothing, such
@@ -628,8 +647,7 @@ public class MariaDbStatement implements Statement, Cloneable {
    * @return either (1) the row count for SQL Data Manipulation Language (DML) statements or (2) 0
    *     for SQL statements that return nothing
    * @throws SQLException if a database access error occurs, this method is called on a closed
-   *     <code>Statement</code>, the given SQL statement returns a <code>ResultSet</code> object, or
-   *     the given constant is not one of those allowed
+   *     <code>Statement</code> or the given constant is not one of those allowed
    */
   public int executeUpdate(final String sql, final int autoGeneratedKeys) throws SQLException {
     if (executeInternal(sql, fetchSize, autoGeneratedKeys)) {
@@ -644,7 +662,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * the columns in the target table that contain the auto-generated keys that should be made
    * available. The driver will ignore the array if the SQL statement is not an <code>INSERT</code>
    * statement, or an SQL statement able to return auto-generated keys (the list of such statements
-   * is vendor-specific).
+   * is vendor-specific). Result-set are permitted for historical reason, even if spec indicate to
+   * throw exception.
    *
    * @param sql an SQL Data Manipulation Language (DML) statement, such as <code>INSERT</code>,
    *     <code>UPDATE</code> or <code>DELETE</code>; or an SQL statement that returns nothing, such
@@ -654,9 +673,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * @return either (1) the row count for SQL Data Manipulation Language (DML) statements or (2) 0
    *     for SQL statements that return nothing
    * @throws SQLException if a database access error occurs, this method is called on a closed
-   *     <code>Statement</code>, the SQL statement returns a <code>ResultSet</code> object, or the
-   *     second argument supplied to this method is not an <code>int</code> array whose elements are
-   *     valid column indexes
+   *     <code>Statement</code> or the second argument supplied to this method is not an <code>int
+   *     </code> array whose elements are valid column indexes
    */
   public int executeUpdate(final String sql, final int[] columnIndexes) throws SQLException {
     return executeUpdate(sql, Statement.RETURN_GENERATED_KEYS);
@@ -668,7 +686,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * columns in the target table that contain the auto-generated keys that should be made available.
    * The driver will ignore the array if the SQL statement is not an <code>INSERT</code> statement,
    * or an SQL statement able to return auto-generated keys (the list of such statements is
-   * vendor-specific).
+   * vendor-specific). Result-set are permitted for historical reason, even if spec indicate to
+   * throw exception.
    *
    * @param sql an SQL Data Manipulation Language (DML) statement, such as <code>INSERT</code>,
    *     <code>UPDATE</code> or <code>DELETE</code>; or an SQL statement that returns nothing, such
@@ -678,9 +697,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * @return either the row count for <code>INSERT</code>, <code>UPDATE</code>, or <code>DELETE
    *     </code> statements, or 0 for SQL statements that return nothing
    * @throws SQLException if a database access error occurs, this method is called on a closed
-   *     <code>Statement</code>, the SQL statement returns a <code>ResultSet</code> object, or the
-   *     second argument supplied to this method is not a <code>String</code> array whose elements
-   *     are valid column names
+   *     <code>Statement</code> or the second argument supplied to this method is not a <code>String
+   *     </code> array whose elements are valid column names
    */
   public int executeUpdate(final String sql, final String[] columnNames) throws SQLException {
     return executeUpdate(sql, Statement.RETURN_GENERATED_KEYS);
@@ -773,7 +791,6 @@ public class MariaDbStatement implements Statement, Cloneable {
 
         results.close();
       }
-      protocol = null;
 
       if (connection == null
           || connection.pooledConnection == null
@@ -782,6 +799,8 @@ public class MariaDbStatement implements Statement, Cloneable {
       }
       connection.pooledConnection.fireStatementClosed(this);
     } finally {
+      protocol = null;
+      connection = null;
       lock.unlock();
     }
   }
@@ -842,7 +861,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    */
   public void setMaxRows(final int max) throws SQLException {
     if (max < 0) {
-      throw new SQLException("max rows cannot be negative : asked for " + max);
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("max rows cannot be negative : asked for " + max, "42000");
     }
     maxRows = max;
   }
@@ -870,7 +891,9 @@ public class MariaDbStatement implements Statement, Cloneable {
   @Override
   public void setLargeMaxRows(long max) throws SQLException {
     if (max < 0) {
-      throw new SQLException("max rows cannot be negative : setLargeMaxRows value is " + max);
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("max rows cannot be negative : setLargeMaxRows value is " + max, "42000");
     }
     maxRows = max;
   }
@@ -884,7 +907,7 @@ public class MariaDbStatement implements Statement, Cloneable {
    * @param enable <code>true</code> to enable escape processing; <code>false</code> to disable it
    */
   public void setEscapeProcessing(final boolean enable) {
-    // not handled
+    escape = enable;
   }
 
   /**
@@ -911,7 +934,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    */
   public void setQueryTimeout(final int seconds) throws SQLException {
     if (seconds < 0) {
-      throw new SQLException("Query timeout rows cannot be negative : asked for " + seconds);
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("Query timeout cannot be negative : asked for " + seconds, "42000");
     }
     this.queryTimeout = seconds;
   }
@@ -959,7 +984,7 @@ public class MariaDbStatement implements Statement, Cloneable {
 
     } catch (SQLException e) {
       logger.error("error cancelling query", e);
-      ExceptionMapper.throwException(e, connection, this);
+      throw exceptionFactory.raiseStatementError(connection, this).create(e);
     } finally {
       if (locked) {
         lock.unlock();
@@ -1020,7 +1045,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    *     <code>Statement</code>
    */
   public void setCursorName(final String name) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("Cursors are not supported");
+    throw exceptionFactory
+        .raiseStatementError(connection, this)
+        .notSupported("Cursors are not supported");
   }
 
   /**
@@ -1091,7 +1118,7 @@ public class MariaDbStatement implements Statement, Cloneable {
 
   /**
    * Requests that a <code>Statement</code> be pooled or not pooled. The value specified is a hint
-   * to the statement pool implementation indicating whether the applicaiton wants the statement to
+   * to the statement pool implementation indicating whether the application wants the statement to
    * be pooled. It is up to the statement pool manager as to whether the hint is used.
    *
    * <p>The poolable value of a statement is applicable to both internal statement caches
@@ -1159,7 +1186,7 @@ public class MariaDbStatement implements Statement, Cloneable {
       connection.reenableWarnings();
     } catch (SQLException e) {
       logger.debug("error skipMoreResults", e);
-      ExceptionMapper.throwException(e, connection, this);
+      throw exceptionFactory.raiseStatementError(connection, this).create(e);
     }
   }
 
@@ -1268,7 +1295,7 @@ public class MariaDbStatement implements Statement, Cloneable {
    */
   public void setFetchSize(final int rows) throws SQLException {
     if (rows < 0 && rows != Integer.MIN_VALUE) {
-      throw new SQLException("invalid fetch size");
+      throw exceptionFactory.raiseStatementError(connection, this).create("invalid fetch size");
     } else if (rows == Integer.MIN_VALUE) {
       // for compatibility Integer.MIN_VALUE is transform to 0 => streaming
       this.fetchSize = 1;
@@ -1316,7 +1343,9 @@ public class MariaDbStatement implements Statement, Cloneable {
       batchQueries = new ArrayList<>();
     }
     if (sql == null) {
-      throw ExceptionMapper.getSqlException("null cannot be set to addBatch( String sql)");
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("null cannot be set to addBatch( String sql)");
     }
     batchQueries.add(sql);
   }
@@ -1450,11 +1479,14 @@ public class MariaDbStatement implements Statement, Cloneable {
       if (isWrapperFor(iface)) {
         return (T) this;
       } else {
-        throw new SQLException(
-            "The receiver is not a wrapper and does not implement the interface");
+        throw exceptionFactory
+            .raiseStatementError(connection, this)
+            .create("The receiver is not a wrapper and does not implement the interface", "42000");
       }
     } catch (Exception e) {
-      throw new SQLException("The receiver is not a wrapper and does not implement the interface");
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("The receiver is not a wrapper and does not implement the interface", "42000");
     }
   }
 
@@ -1509,16 +1541,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    */
   protected void checkClose() throws SQLException {
     if (closed) {
-      throw new SQLException("Cannot do an operation on a closed statement");
+      throw exceptionFactory
+          .raiseStatementError(connection, this)
+          .create("Cannot do an operation on a closed statement");
     }
   }
-
-  /**
-   * Permit to retrieve current connection thread id, or -1 if unknown.
-   *
-   * @return current connection thread id.
-   */
-  public long getServerThreadId() {
-    return (protocol != null) ? protocol.getServerThreadId() : -1;
-  }
 }
