@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,11 +52,13 @@
 
 package org.mariadb.jdbc;
 
-import org.mariadb.jdbc.internal.util.*;
-
-import javax.transaction.xa.*;
-import java.sql.*;
-import java.util.*;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import javax.transaction.xa.XAException;
+import javax.transaction.xa.XAResource;
+import javax.transaction.xa.Xid;
+import org.mariadb.jdbc.internal.util.Utils;
 
 public class MariaXaResource implements XAResource {
 
@@ -72,7 +74,7 @@ public class MariaXaResource implements XAResource {
         + ",0x"
         + Utils.byteArrayToHexString(xid.getBranchQualifier())
         + ",0x"
-        + Utils.intToHexString(xid.getFormatId());
+        + Integer.toHexString(xid.getFormatId());
   }
 
   private static String flagsToString(int flags) {
@@ -215,8 +217,13 @@ public class MariaXaResource implements XAResource {
    */
   @Override
   public boolean isSameRM(XAResource xaResource) {
-    // Typically used by transaction manager to "join" transactions. We do not support joins,
-    // so always return false;
+    if (xaResource instanceof MariaXaResource) {
+      MariaXaResource other = (MariaXaResource) xaResource;
+      return connection
+          .getProtocol()
+          .getUrlParser()
+          .equals(other.connection.getProtocol().getUrlParser());
+    }
     return false;
   }
 
