@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,14 +52,20 @@
 
 package org.mariadb.jdbc;
 
-import org.mariadb.jdbc.credential.*;
-import org.mariadb.jdbc.internal.logging.*;
-import org.mariadb.jdbc.internal.util.constant.*;
-import org.mariadb.jdbc.util.*;
-
-import java.sql.*;
-import java.util.*;
-import java.util.regex.*;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Locale;
+import java.util.Properties;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+import org.mariadb.jdbc.credential.CredentialPlugin;
+import org.mariadb.jdbc.credential.CredentialPluginLoader;
+import org.mariadb.jdbc.internal.logging.LoggerFactory;
+import org.mariadb.jdbc.internal.util.constant.HaMode;
+import org.mariadb.jdbc.internal.util.constant.ParameterConstant;
+import org.mariadb.jdbc.util.DefaultOptions;
+import org.mariadb.jdbc.util.Options;
 
 /**
  * parse and verification of URL.
@@ -75,7 +81,7 @@ import java.util.regex.*;
  * (for example localhost:3306)<br>
  * <br>
  * - complex :<br>
- * {@code address=[(type=(master|slave))][(port=<portnumber>)](host=<host>)}<br>
+ * {@code address=[(type=(master|replica))][(port=<portnumber>)](host=<host>)}<br>
  * <br>
  * <br>
  * type is by default master<br>
@@ -83,12 +89,12 @@ import java.util.regex.*;
  *
  * <p>host can be dns name, ipv4 or ipv6.<br>
  * in case of ipv6 and simple host description, the ip must be written inside bracket.<br>
- * exemple : {@code jdbc:mariadb://[2001:0660:7401:0200:0000:0000:0edf:bdd7]:3306}<br>
+ * example : {@code jdbc:mariadb://[2001:0660:7401:0200:0000:0000:0edf:bdd7]:3306}<br>
  *
  * <p>Some examples :<br>
  * {@code jdbc:mariadb://localhost:3306/database?user=greg&password=pass}<br>
  * {@code
- * jdbc:mariadb://address=(type=master)(host=master1),address=(port=3307)(type=slave)(host=slave1)/database?user=greg&password=pass}
+ * jdbc:mariadb://address=(type=master)(host=master1),address=(port=3307)(type=replica)(host=replica1)/database?user=greg&password=pass}
  * <br>
  */
 public class UrlParser implements Cloneable {
@@ -251,25 +257,14 @@ public class UrlParser implements Cloneable {
     if (additionalParameters != null) {
       //noinspection Annotator
       Matcher matcher = URL_PARAMETER.matcher(additionalParameters);
-
-      if (matcher.find()) {
-
-        urlParser.database = matcher.group(2);
-        urlParser.options =
-            DefaultOptions.parse(urlParser.haMode, matcher.group(4), properties, urlParser.options);
-        if (urlParser.database != null && urlParser.database.isEmpty()) {
-          urlParser.database = null;
-        }
-
-      } else {
-
+      matcher.find();
+      urlParser.database = matcher.group(2);
+      urlParser.options =
+          DefaultOptions.parse(urlParser.haMode, matcher.group(4), properties, urlParser.options);
+      if (urlParser.database != null && urlParser.database.isEmpty()) {
         urlParser.database = null;
-        urlParser.options =
-            DefaultOptions.parse(urlParser.haMode, "", properties, urlParser.options);
       }
-
     } else {
-
       urlParser.database = null;
       urlParser.options = DefaultOptions.parse(urlParser.haMode, "", properties, urlParser.options);
     }
