@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,10 +52,13 @@
 
 package org.mariadb.jdbc.internal;
 
-import org.mariadb.jdbc.util.*;
-
-import java.math.*;
-import java.sql.*;
+import java.math.BigDecimal;
+import java.math.BigInteger;
+import java.sql.Date;
+import java.sql.Time;
+import java.sql.Timestamp;
+import java.sql.Types;
+import org.mariadb.jdbc.util.Options;
 
 public enum ColumnType {
   OLDDECIMAL(0, Types.DECIMAL, "Types.DECIMAL", BigDecimal.class.getName()),
@@ -217,12 +220,13 @@ public enum ColumnType {
    *
    * @param type type
    * @param len len
+   * @param charLen character length
    * @param signed signed
    * @param binary binary
    * @return type
    */
   public static String getColumnTypeName(
-      ColumnType type, long len, boolean signed, boolean binary) {
+      ColumnType type, long len, long charLen, boolean signed, boolean binary) {
     switch (type) {
       case SMALLINT:
       case MEDIUMINT:
@@ -254,7 +258,17 @@ public enum ColumnType {
         if (binary) {
           return "VARBINARY";
         }
-        return "VARCHAR";
+        if (len < 0) {
+          return "LONGTEXT";
+        } else if (charLen <= 65532) {
+          return "VARCHAR";
+        } else if (charLen <= 65535) {
+          return "TEXT";
+        } else if (charLen <= 16777215) {
+          return "MEDIUMTEXT";
+        } else {
+          return "LONGTEXT";
+        }
       case STRING:
         if (binary) {
           return "BINARY";
