@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,20 +52,24 @@
 
 package org.mariadb.jdbc.internal.com.read.resultset;
 
-import org.mariadb.jdbc.*;
-import org.mariadb.jdbc.internal.*;
-import org.mariadb.jdbc.internal.com.read.dao.*;
-import org.mariadb.jdbc.internal.com.send.parameters.*;
-import org.mariadb.jdbc.internal.io.input.*;
-import org.mariadb.jdbc.internal.protocol.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-
-import java.io.*;
-import java.math.*;
-import java.sql.Date;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.Reader;
+import java.math.BigDecimal;
 import java.sql.*;
 import java.time.*;
-import java.util.*;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.TimeZone;
+import org.mariadb.jdbc.*;
+import org.mariadb.jdbc.internal.ColumnType;
+import org.mariadb.jdbc.internal.com.read.dao.Results;
+import org.mariadb.jdbc.internal.com.read.resultset.rowprotocol.BinaryRowProtocol;
+import org.mariadb.jdbc.internal.com.send.parameters.*;
+import org.mariadb.jdbc.internal.io.input.PacketInputStream;
+import org.mariadb.jdbc.internal.protocol.Protocol;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
 
 public class UpdatableResultSet extends SelectResultSet {
 
@@ -87,7 +91,6 @@ public class UpdatableResultSet extends SelectResultSet {
   private ParameterHolder[] parameterHolders;
   private MariaDbConnection connection;
   private PreparedStatement refreshPreparedStatement = null;
-  private ClientSidePreparedStatement insertPreparedStatement = null;
   private ClientSidePreparedStatement deletePreparedStatement = null;
 
   /**
@@ -103,7 +106,7 @@ public class UpdatableResultSet extends SelectResultSet {
    * @throws SQLException if any connection error occur
    */
   public UpdatableResultSet(
-      ColumnInformation[] columnsInformation,
+      ColumnDefinition[] columnsInformation,
       Results results,
       Protocol protocol,
       PacketInputStream reader,
@@ -129,9 +132,9 @@ public class UpdatableResultSet extends SelectResultSet {
     canBeRefresh = false;
 
     // check that resultSet concern one table and database exactly
-    for (ColumnInformation columnInformation : columnsInformation) {
+    for (ColumnDefinition columnDefinition : columnsInformation) {
 
-      if (columnInformation.getDatabase() == null || columnInformation.getDatabase().isEmpty()) {
+      if (columnDefinition.getDatabase() == null || columnDefinition.getDatabase().isEmpty()) {
 
         cannotUpdateInsertRow(
             "The result-set contains fields without without any database information");
@@ -139,16 +142,16 @@ public class UpdatableResultSet extends SelectResultSet {
 
       } else {
 
-        if (database != null && !database.equals(columnInformation.getDatabase())) {
+        if (database != null && !database.equals(columnDefinition.getDatabase())) {
           cannotUpdateInsertRow("The result-set contains more than one database");
           return;
         }
 
-        database = columnInformation.getDatabase();
+        database = columnDefinition.getDatabase();
       }
 
-      if (columnInformation.getOriginalTable() == null
-          || columnInformation.getOriginalTable().isEmpty()) {
+      if (columnDefinition.getOriginalTable() == null
+          || columnDefinition.getOriginalTable().isEmpty()) {
 
         cannotUpdateInsertRow(
             "The result-set contains fields without without any table information");
@@ -156,12 +159,12 @@ public class UpdatableResultSet extends SelectResultSet {
 
       } else {
 
-        if (table != null && !table.equals(columnInformation.getOriginalTable())) {
+        if (table != null && !table.equals(columnDefinition.getOriginalTable())) {
           cannotUpdateInsertRow("The result-set contains fields on different tables");
           return;
         }
 
-        table = columnInformation.getOriginalTable();
+        table = columnDefinition.getOriginalTable();
       }
     }
 
@@ -185,15 +188,15 @@ public class UpdatableResultSet extends SelectResultSet {
                 ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
         ResultSet rs = stmt.executeQuery("SHOW COLUMNS FROM `" + database + "`.`" + table + "`");
 
-        UpdatableColumnInformation[] updatableColumns =
-            new UpdatableColumnInformation[columnInformationLength];
+        UpdatableColumnDefinition[] updatableColumns =
+            new UpdatableColumnDefinition[columnInformationLength];
 
         boolean primaryFound = false;
         while (rs.next()) {
           // read SHOW COLUMNS informations
           String fieldName = rs.getString("Field");
           boolean canBeNull = "YES".equals(rs.getString("Null"));
-          boolean hasDefault = rs.getString("Default") == null;
+          boolean hasDefault = rs.getString("Default") != null;
           String extra = rs.getString("Extra");
           boolean generated = extra != null && !extra.isEmpty();
           boolean autoIncrement = extra != null && "auto_increment".equals(extra);
@@ -203,11 +206,11 @@ public class UpdatableResultSet extends SelectResultSet {
 
           // update column information with SHOW COLUMNS additional informations
           for (int index = 0; index < columnInformationLength; index++) {
-            ColumnInformation columnInformation = columnsInformation[index];
-            if (fieldName.equals(columnInformation.getOriginalName())) {
+            ColumnDefinition columnDefinition = columnsInformation[index];
+            if (fieldName.equals(columnDefinition.getOriginalName())) {
               updatableColumns[index] =
-                  new UpdatableColumnInformation(
-                      columnInformation, canBeNull, hasDefault, generated, primary, autoIncrement);
+                  new UpdatableColumnDefinition(
+                      columnDefinition, canBeNull, hasDefault, generated, primary, autoIncrement);
               found = true;
             }
           }
@@ -272,8 +275,8 @@ public class UpdatableResultSet extends SelectResultSet {
     }
   }
 
-  private UpdatableColumnInformation[] getUpdatableColumns() {
-    return (UpdatableColumnInformation[]) columnsInformation;
+  private UpdatableColumnDefinition[] getUpdatableColumns() {
+    return (UpdatableColumnDefinition[]) columnsInformation;
   }
 
   private void cannotUpdateInsertRow(String reason) {
@@ -621,7 +624,7 @@ public class UpdatableResultSet extends SelectResultSet {
       case Types.ROWID:
       case Types.SQLXML:
       case Types.STRUCT:
-        throw ExceptionMapper.getFeatureNotSupportedException("Type not supported");
+        throw ExceptionFactory.INSTANCE.notSupported("Type not supported");
       default:
         break;
     }
@@ -630,7 +633,7 @@ public class UpdatableResultSet extends SelectResultSet {
       updateNull(parameterIndex);
     } else if (obj instanceof String) {
       if (targetSqlType == Types.BLOB) {
-        throw ExceptionMapper.getSqlException("Cannot convert a String to a Blob");
+        throw ExceptionFactory.INSTANCE.create("Cannot convert a String to a Blob");
       }
       String str = (String) obj;
       try {
@@ -685,25 +688,22 @@ public class UpdatableResultSet extends SelectResultSet {
           case Types.TIME_WITH_TIMEZONE:
             parameterHolders[parameterIndex - 1] =
                 new OffsetTimeParameter(
-                    OffsetTime.parse(str),
-                    timeZone.toZoneId(),
-                    options.useFractionalSeconds,
-                    options);
+                    OffsetTime.parse(str), timeZone, options.useFractionalSeconds, options);
             break;
           case Types.TIMESTAMP_WITH_TIMEZONE:
             parameterHolders[parameterIndex - 1] =
                 new ZonedDateTimeParameter(
                     ZonedDateTime.parse(str, BasePrepareStatement.SPEC_ISO_ZONED_DATE_TIME),
-                    timeZone.toZoneId(),
+                    timeZone,
                     options.useFractionalSeconds,
                     options);
             break;
           default:
-            throw ExceptionMapper.getSqlException(
+            throw ExceptionFactory.INSTANCE.create(
                 "Could not convert [" + str + "] to " + targetSqlType);
         }
       } catch (IllegalArgumentException e) {
-        throw ExceptionMapper.getSqlException(
+        throw ExceptionFactory.INSTANCE.create(
             "Could not convert [" + str + "] to " + targetSqlType, e);
       }
     } else if (obj instanceof Number) {
@@ -746,7 +746,7 @@ public class UpdatableResultSet extends SelectResultSet {
           updateString(parameterIndex, bd.toString());
           break;
         default:
-          throw ExceptionMapper.getSqlException(
+          throw ExceptionFactory.INSTANCE.create(
               "Could not convert [" + bd + "] to " + targetSqlType);
       }
     } else if (obj instanceof byte[]) {
@@ -755,7 +755,7 @@ public class UpdatableResultSet extends SelectResultSet {
           || targetSqlType == Types.LONGVARBINARY) {
         updateBytes(parameterIndex, (byte[]) obj);
       } else {
-        throw ExceptionMapper.getSqlException(
+        throw ExceptionFactory.INSTANCE.create(
             "Can only convert a byte[] to BINARY, VARBINARY or LONGVARBINARY");
       }
 
@@ -794,21 +794,21 @@ public class UpdatableResultSet extends SelectResultSet {
       parameterHolders[parameterIndex - 1] =
           new ZonedDateTimeParameter(
               ((OffsetDateTime) obj).toZonedDateTime(),
-              timeZone.toZoneId(),
+              timeZone,
               options.useFractionalSeconds,
               options);
     } else if (obj instanceof OffsetTime) {
       parameterHolders[parameterIndex - 1] =
           new OffsetTimeParameter(
-              (OffsetTime) obj, timeZone.toZoneId(), options.useFractionalSeconds, options);
+              (OffsetTime) obj, timeZone, options.useFractionalSeconds, options);
     } else if (obj instanceof ZonedDateTime) {
       parameterHolders[parameterIndex - 1] =
           new ZonedDateTimeParameter(
-              (ZonedDateTime) obj, timeZone.toZoneId(), options.useFractionalSeconds, options);
+              (ZonedDateTime) obj, timeZone, options.useFractionalSeconds, options);
     } else if (obj instanceof LocalTime) {
       updateTime(parameterIndex, Time.valueOf((LocalTime) obj));
     } else {
-      throw ExceptionMapper.getSqlException(
+      throw ExceptionFactory.INSTANCE.create(
           "Could not set parameter in setObject, could not convert: "
               + obj.getClass()
               + " to "
@@ -860,12 +860,12 @@ public class UpdatableResultSet extends SelectResultSet {
 
   /** {inheritDoc}. */
   public void updateRef(int columnIndex, Ref ref) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("REF not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("REF not supported");
   }
 
   /** {inheritDoc}. */
   public void updateRef(String columnLabel, Ref ref) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("REF not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("REF not supported");
   }
 
   /** {inheritDoc}. */
@@ -950,22 +950,22 @@ public class UpdatableResultSet extends SelectResultSet {
 
   /** {inheritDoc}. */
   public void updateArray(int columnIndex, Array array) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("Arrays not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("Arrays not supported");
   }
 
   /** {inheritDoc}. */
   public void updateArray(String columnLabel, Array array) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("Arrays not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("Arrays not supported");
   }
 
   /** {inheritDoc}. */
   public void updateRowId(int columnIndex, RowId rowId) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("RowIDs not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("RowIDs not supported");
   }
 
   /** {inheritDoc}. */
   public void updateRowId(String columnLabel, RowId rowId) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("RowIDs not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("RowIDs not supported");
   }
 
   /** {inheritDoc}. */
@@ -1011,13 +1011,13 @@ public class UpdatableResultSet extends SelectResultSet {
   /** {inheritDoc}. */
   @Override
   public void updateSQLXML(int columnIndex, SQLXML xmlObject) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("SQlXML not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("SQlXML not supported");
   }
 
   /** {inheritDoc}. */
   @Override
   public void updateSQLXML(String columnLabel, SQLXML xmlObject) throws SQLException {
-    throw ExceptionMapper.getFeatureNotSupportedException("SQLXML not supported");
+    throw ExceptionFactory.INSTANCE.notSupported("SQLXML not supported");
   }
 
   /** {inheritDoc}. */
@@ -1045,67 +1045,113 @@ public class UpdatableResultSet extends SelectResultSet {
   /** {inheritDoc}. */
   public void insertRow() throws SQLException {
     if (state == STATE_INSERT) {
-      if (insertPreparedStatement == null) {
-        // Create query will all field with WHERE clause contain primary field.
-        // if field are not updated, value DEFAULT will be set
-        // (if field has no default, then insert will throw an exception that will be return to
-        // user)
-        StringBuilder insertSql = new StringBuilder("INSERT `" + database + "`.`" + table + "` ( ");
-        StringBuilder valueClause = new StringBuilder();
 
-        for (int pos = 0; pos < columnInformationLength; pos++) {
-          UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+      boolean hasGeneratedPrimaryFields = false;
+      int generatedSqlType = 0;
+      HashMap<Integer, ParameterHolder> paramMap = new HashMap<>();
 
-          if (pos != 0) {
+      // Create query will all field with WHERE clause contain primary field.
+      // if field are not updated, value DEFAULT will be set
+      // (if field has no default, then insert will throw an exception that will be return to
+      // user)
+      StringBuilder insertSql = new StringBuilder("INSERT `" + database + "`.`" + table + "` ( ");
+      StringBuilder valueClause = new StringBuilder();
+      StringBuilder returningClause = new StringBuilder();
+      int fieldsIndex = 0;
+      boolean firstParam = true;
+
+      for (int pos = 0; pos < columnInformationLength; pos++) {
+        UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
+
+        if (pos != 0) {
+          returningClause.append(", ");
+        }
+        returningClause.append("`").append(colInfo.getOriginalName()).append("`");
+
+        ParameterHolder value = parameterHolders[pos];
+        if (value != null) {
+          if (!firstParam) {
             insertSql.append(",");
             valueClause.append(", ");
           }
-
           insertSql.append("`").append(colInfo.getOriginalName()).append("`");
           valueClause.append("?");
-        }
-        insertSql.append(") VALUES (").append(valueClause).append(")");
-        insertPreparedStatement = connection.clientPrepareStatement(insertSql.toString());
-      }
-
-      int fieldsIndex = 0;
-      boolean hasGeneratedPrimaryFields = false;
-      int generatedSqlType = 0;
-      for (int pos = 0; pos < columnInformationLength; pos++) {
-        ParameterHolder value = parameterHolders[pos];
-        if (value != null) {
-          insertPreparedStatement.setParameter((fieldsIndex++) + 1, value);
+          paramMap.put((fieldsIndex++) + 1, value);
+          firstParam = false;
         } else {
-          UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
-          if (colInfo.isPrimary() && colInfo.isAutoIncrement()) {
-            hasGeneratedPrimaryFields = true;
-            generatedSqlType = colInfo.getColumnType().getSqlType();
-          }
-          insertPreparedStatement.setParameter((fieldsIndex++) + 1, new DefaultParameter());
-        }
-      }
+          if (colInfo.isPrimary()) {
+            if (colInfo.isAutoIncrement() || colInfo.hasDefault()) {
+              if (colInfo.isAutoIncrement()) {
+                hasGeneratedPrimaryFields = true;
+                generatedSqlType = colInfo.getColumnType().getSqlType();
+              } else if (!connection.isServerMariaDb()
+                  || !connection.versionGreaterOrEqual(10, 5, 1)) {
+                // driver cannot know generated default value like uuid().
+                // but for server 10.5+, will use RETURNING to know primary key
+                throw new SQLException(
+                    String.format(
+                        "Cannot call insertRow() not setting value for primary key %s "
+                            + "with default value before server 10.5",
+                        colInfo.getOriginalName()));
+              }
+            } else {
+              throw new SQLException(
+                  String.format(
+                      "Cannot call insertRow() not setting value for primary key %s",
+                      colInfo.getOriginalName()));
+            }
+            // paramMap.put((fieldsIndex++) + 1, new DefaultParameter());
+          } else if (!colInfo.hasDefault()) {
+            if (!firstParam) {
+              insertSql.append(",");
+              valueClause.append(", ");
+            }
+            firstParam = false;
+            insertSql.append("`").append(colInfo.getOriginalName()).append("`");
+            valueClause.append("?");
 
-      insertPreparedStatement.execute();
-
-      if (hasGeneratedPrimaryFields) {
-        // primary is auto_increment (only one field)
-        ResultSet rsKey = insertPreparedStatement.getGeneratedKeys();
-        if (rsKey.next()) {
-
-          prepareRefreshStmt();
-          refreshPreparedStatement.setObject(1, rsKey.getObject(1), generatedSqlType);
-          SelectResultSet rs = (SelectResultSet) refreshPreparedStatement.executeQuery();
-
-          // update row data only if not deleted externally
-          if (rs.next()) {
-            addRowData(rs.getCurrentRowData());
+            paramMap.put((fieldsIndex++) + 1, new NullParameter());
           }
         }
-
-      } else {
-        addRowData(refreshRawData());
+      }
+      insertSql.append(") VALUES (").append(valueClause).append(")");
+      if (connection.isServerMariaDb() && connection.versionGreaterOrEqual(10, 5, 1)) {
+        insertSql.append(" RETURNING ").append(returningClause);
       }
 
+      try (BasePrepareStatement insertPreparedStatement =
+          (row instanceof BinaryRowProtocol)
+              ? connection.serverPrepareStatement(insertSql.toString())
+              : connection.clientPrepareStatement(insertSql.toString())) {
+
+        for (Map.Entry<Integer, ParameterHolder> entry : paramMap.entrySet()) {
+          insertPreparedStatement.setParameter(entry.getKey(), entry.getValue());
+        }
+
+        ResultSet insertRs = insertPreparedStatement.executeQuery();
+        if (connection.isServerMariaDb() && connection.versionGreaterOrEqual(10, 5, 1)) {
+          if (insertRs.next()) {
+            byte[] rowByte = ((SelectResultSet) insertRs).getCurrentRowData();
+            addRowData(rowByte);
+          }
+        } else if (hasGeneratedPrimaryFields) {
+          // primary is auto_increment (only one field)
+          ResultSet rsKey = insertPreparedStatement.getGeneratedKeys();
+          if (rsKey.next()) {
+
+            prepareRefreshStmt();
+            refreshPreparedStatement.setObject(1, rsKey.getObject(1), generatedSqlType);
+            SelectResultSet rs = (SelectResultSet) refreshPreparedStatement.executeQuery();
+
+            // update row data only if not deleted externally
+            if (rs.next()) {
+              addRowData(rs.getCurrentRowData());
+            }
+          }
+        } else {
+          addRowData(refreshRawData());
+        }
+      }
       Arrays.fill(parameterHolders, null);
     }
   }
@@ -1129,7 +1175,7 @@ public class UpdatableResultSet extends SelectResultSet {
       boolean firstPrimary = true;
       int fieldsToUpdate = 0;
       for (int pos = 0; pos < columnInformationLength; pos++) {
-        UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+        UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
 
         ParameterHolder value = parameterHolders[pos];
         if (colInfo.isPrimary()) {
@@ -1156,7 +1202,7 @@ public class UpdatableResultSet extends SelectResultSet {
       int fieldsIndex = 0;
       int fieldsPrimaryIndex = 0;
       for (int pos = 0; pos < columnInformationLength; pos++) {
-        UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+        UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
         ParameterHolder value = parameterHolders[pos];
 
         if (value != null) {
@@ -1206,7 +1252,7 @@ public class UpdatableResultSet extends SelectResultSet {
           new StringBuilder("DELETE FROM `" + database + "`.`" + table + "` WHERE ");
       boolean firstPrimary = true;
       for (int pos = 0; pos < columnInformationLength; pos++) {
-        UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+        UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
 
         if (colInfo.isPrimary()) {
           if (!firstPrimary) {
@@ -1222,7 +1268,7 @@ public class UpdatableResultSet extends SelectResultSet {
     int fieldsPrimaryIndex = 1;
 
     for (int pos = 0; pos < columnInformationLength; pos++) {
-      UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+      UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
       if (colInfo.isPrimary()) {
         deletePreparedStatement.setObject(
             fieldsPrimaryIndex++, getObject(pos + 1), colInfo.getColumnType().getSqlType());
@@ -1243,7 +1289,7 @@ public class UpdatableResultSet extends SelectResultSet {
 
       boolean firstPrimary = true;
       for (int pos = 0; pos < columnInformationLength; pos++) {
-        UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+        UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
         if (pos != 0) {
           selectSql.append(",");
         }
@@ -1279,7 +1325,7 @@ public class UpdatableResultSet extends SelectResultSet {
     prepareRefreshStmt();
     int fieldsPrimaryIndex = 1;
     for (int pos = 0; pos < columnInformationLength; pos++) {
-      UpdatableColumnInformation colInfo = getUpdatableColumns()[pos];
+      UpdatableColumnDefinition colInfo = getUpdatableColumns()[pos];
       if (colInfo.isPrimary()) {
         ParameterHolder value = parameterHolders[pos];
 
@@ -1422,4 +1468,9 @@ public class UpdatableResultSet extends SelectResultSet {
     }
     return super.previous();
   }
+
+  public void close() throws SQLException {
+    if (refreshPreparedStatement != null) refreshPreparedStatement.close();
+    super.close();
+  }
 }
