@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,38 +52,40 @@
 
 package org.mariadb.jdbc.internal.com.read.resultset.rowprotocol;
 
-import org.mariadb.jdbc.internal.com.read.resultset.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-import org.mariadb.jdbc.util.*;
-
-import java.math.*;
-import java.nio.charset.*;
-import java.sql.Date;
+import java.math.BigDecimal;
+import java.math.BigInteger;
+import java.nio.charset.StandardCharsets;
 import java.sql.*;
 import java.time.*;
-import java.time.format.*;
-import java.util.*;
+import java.time.format.DateTimeFormatter;
+import java.time.format.DateTimeParseException;
+import java.util.Calendar;
+import java.util.TimeZone;
+import org.mariadb.jdbc.internal.ColumnType;
+import org.mariadb.jdbc.internal.com.read.resultset.ColumnDefinition;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
+import org.mariadb.jdbc.util.Options;
 
 public class BinaryRowProtocol extends RowProtocol {
 
-  private final ColumnInformation[] columnInformation;
+  private final ColumnDefinition[] columnDefinition;
   private final int columnInformationLength;
 
   /**
    * Constructor.
    *
-   * @param columnInformation column information.
+   * @param columnDefinition column information.
    * @param columnInformationLength number of columns
    * @param maxFieldSize max field size
    * @param options connection options
    */
   public BinaryRowProtocol(
-      ColumnInformation[] columnInformation,
+      ColumnDefinition[] columnDefinition,
       int columnInformationLength,
       int maxFieldSize,
       Options options) {
     super(maxFieldSize, options);
-    this.columnInformation = columnInformation;
+    this.columnDefinition = columnDefinition;
     this.columnInformationLength = columnInformationLength;
   }
 
@@ -120,7 +122,7 @@ public class BinaryRowProtocol extends RowProtocol {
         if ((buf[1 + (index + 2) / 8] & (1 << ((index + 2) % 8))) == 0) {
           if (index != newIndex) {
             // skip bytes
-            switch (columnInformation[index].getColumnType()) {
+            switch (columnDefinition[index].getColumnType()) {
               case BIGINT:
               case DOUBLE:
                 internalPos += 8;
@@ -185,7 +187,7 @@ public class BinaryRowProtocol extends RowProtocol {
             }
           } else {
             // read asked field position and length
-            switch (columnInformation[index].getColumnType()) {
+            switch (columnDefinition[index].getColumnType()) {
               case BIGINT:
               case DOUBLE:
                 this.pos = internalPos;
@@ -284,7 +286,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return String value of raw bytes
    * @throws SQLException if conversion failed
    */
-  public String getInternalString(ColumnInformation columnInfo, Calendar cal, TimeZone timeZone)
+  public String getInternalString(ColumnDefinition columnInfo, Calendar cal, TimeZone timeZone)
       throws SQLException {
     if ((lastValueNull & BIT_LAST_FIELD_NULL) != 0) {
       return null;
@@ -363,7 +365,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return int value
    * @throws SQLException if column is not numeric or is not in Integer bounds.
    */
-  public int getInternalInt(ColumnInformation columnInfo) throws SQLException {
+  public int getInternalInt(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -429,7 +431,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @throws SQLException if column is not numeric or is not in Long bounds (for big unsigned
    *     values)
    */
-  public long getInternalLong(ColumnInformation columnInfo) throws SQLException {
+  public long getInternalLong(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -537,7 +539,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return float value
    * @throws SQLException if column is not numeric or is not in Float bounds.
    */
-  public float getInternalFloat(ColumnInformation columnInfo) throws SQLException {
+  public float getInternalFloat(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -636,7 +638,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return double value
    * @throws SQLException if column is not numeric or is not in Double bounds (unsigned columns).
    */
-  public double getInternalDouble(ColumnInformation columnInfo) throws SQLException {
+  public double getInternalDouble(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -723,7 +725,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return BigDecimal value
    * @throws SQLException if column is not numeric
    */
-  public BigDecimal getInternalBigDecimal(ColumnInformation columnInfo) throws SQLException {
+  public BigDecimal getInternalBigDecimal(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
@@ -796,16 +798,84 @@ public class BinaryRowProtocol extends RowProtocol {
    * @throws SQLException if column is not compatible to Date
    */
   @SuppressWarnings("deprecation")
-  public Date getInternalDate(ColumnInformation columnInfo, Calendar cal, TimeZone timeZone)
+  public Date getInternalDate(ColumnDefinition columnInfo, Calendar cal, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
+    if (length == 0) {
+      lastValueNull |= BIT_LAST_FIELD_NULL;
+      return null;
+    }
+    int year;
+    int month = 1;
+    int day = 1;
+    Calendar calendar;
     switch (columnInfo.getColumnType()) {
       case TIMESTAMP:
       case DATETIME:
-        Timestamp timestamp = getInternalTimestamp(columnInfo, cal, timeZone);
-        return (timestamp == null) ? null : new Date(timestamp.getTime());
+        year = ((buf[pos] & 0xff) | (buf[pos + 1] & 0xff) << 8);
+        month = buf[pos + 2];
+        day = buf[pos + 3];
+        int hour = 0;
+        int minutes = 0;
+        int seconds = 0;
+        int microseconds = 0;
+
+        if (length > 4) {
+          hour = buf[pos + 4];
+          minutes = buf[pos + 5];
+          seconds = buf[pos + 6];
+
+          if (length > 7) {
+            microseconds =
+                ((buf[pos + 7] & 0xff)
+                    + ((buf[pos + 8] & 0xff) << 8)
+                    + ((buf[pos + 9] & 0xff) << 16)
+                    + ((buf[pos + 10] & 0xff) << 24));
+          }
+        }
+
+        if (year == 0 && month == 0 && day == 0) {
+          lastValueNull |= BIT_LAST_FIELD_NULL;
+          return null;
+        }
+
+        if (timeZone == null) {
+          // legacy is to send timestamps with current driver timezone. So display, is immediate
+          calendar = (cal != null) ? cal : Calendar.getInstance();
+          synchronized (calendar) {
+            calendar.clear();
+            calendar.set(Calendar.YEAR, year);
+            calendar.set(Calendar.MONTH, month - 1);
+            calendar.set(Calendar.DAY_OF_MONTH, day);
+            calendar.set(Calendar.HOUR_OF_DAY, 0);
+            calendar.set(Calendar.MINUTE, 0);
+            calendar.set(Calendar.SECOND, 0);
+            calendar.set(Calendar.MILLISECOND, 0);
+            return new Date(calendar.getTimeInMillis());
+          }
+        }
+
+        // timestamp is saved in server timezone,
+        LocalDateTime ldt =
+            LocalDateTime.of(year, month, day, hour, minutes, seconds, microseconds * 1000);
+        ZonedDateTime zdt =
+            ldt.atZone(timeZone.toZoneId()).withZoneSameInstant(TimeZone.getDefault().toZoneId());
+
+        calendar = cal != null ? cal : Calendar.getInstance();
+        synchronized (calendar) {
+          calendar.clear();
+          calendar.set(Calendar.YEAR, zdt.getYear());
+          calendar.set(Calendar.MONTH, zdt.getMonthValue() - 1);
+          calendar.set(Calendar.DAY_OF_MONTH, zdt.getDayOfMonth());
+          calendar.set(Calendar.HOUR_OF_DAY, 0);
+          calendar.set(Calendar.MINUTE, 0);
+          calendar.set(Calendar.SECOND, 0);
+          calendar.set(Calendar.MILLISECOND, 0);
+          return new Date(calendar.getTimeInMillis());
+        }
+
       case TIME:
         throw new SQLException("Cannot read Date using a Types.TIME field");
       case STRING:
@@ -820,12 +890,7 @@ public class BinaryRowProtocol extends RowProtocol {
             Integer.parseInt(rawValue.substring(5, 7)) - 1,
             Integer.parseInt(rawValue.substring(8, 10)));
       default:
-        if (length == 0) {
-          lastValueNull |= BIT_LAST_FIELD_NULL;
-          return null;
-        }
-
-        int year = ((buf[pos] & 0xff) | (buf[pos + 1] & 0xff) << 8);
+        year = ((buf[pos] & 0xff) | (buf[pos + 1] & 0xff) << 8);
 
         if (length == 2 && columnInfo.getLength() == 2) {
           // YEAR(2) - deprecated
@@ -836,25 +901,12 @@ public class BinaryRowProtocol extends RowProtocol {
           }
         }
 
-        int month = 1;
-        int day = 1;
-
         if (length >= 4) {
           month = buf[pos + 2];
           day = buf[pos + 3];
         }
 
-        Calendar calendar = Calendar.getInstance();
-        calendar.clear();
-        calendar.set(Calendar.YEAR, year);
-        calendar.set(Calendar.MONTH, month - 1);
-        calendar.set(Calendar.DAY_OF_MONTH, day);
-        calendar.set(Calendar.HOUR_OF_DAY, 0);
-        calendar.set(Calendar.MINUTE, 0);
-        calendar.set(Calendar.SECOND, 0);
-        calendar.set(Calendar.MILLISECOND, 0);
-        Date dt = new Date(calendar.getTimeInMillis());
-        return dt;
+        return new Date(year - 1900, month - 1, day);
     }
   }
 
@@ -867,25 +919,83 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return Time value
    * @throws SQLException if column cannot be converted to Time
    */
-  public Time getInternalTime(ColumnInformation columnInfo, Calendar cal, TimeZone timeZone)
+  public Time getInternalTime(ColumnDefinition columnInfo, Calendar cal, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
+
+    Calendar calendar;
+    int day = 0;
+    int hour = 0;
+    int minutes = 0;
+    int seconds = 0;
+    int microseconds = 0;
+
     switch (columnInfo.getColumnType()) {
       case TIMESTAMP:
       case DATETIME:
-        Timestamp ts = getInternalTimestamp(columnInfo, cal, timeZone);
-        return (ts == null) ? null : new Time(ts.getTime());
+        if (length == 0) {
+          lastValueNull |= BIT_LAST_FIELD_NULL;
+          return null;
+        }
+        int year = ((buf[pos] & 0xff) | (buf[pos + 1] & 0xff) << 8);
+        int month = buf[pos + 2];
+        day = buf[pos + 3];
+        if (length > 4) {
+          hour = buf[pos + 4];
+          minutes = buf[pos + 5];
+          seconds = buf[pos + 6];
+
+          if (length > 7) {
+            microseconds =
+                ((buf[pos + 7] & 0xff)
+                    + ((buf[pos + 8] & 0xff) << 8)
+                    + ((buf[pos + 9] & 0xff) << 16)
+                    + ((buf[pos + 10] & 0xff) << 24));
+          }
+        }
+
+        if (timeZone == null) {
+          calendar = cal != null ? cal : Calendar.getInstance();
+          synchronized (calendar) {
+            calendar.clear();
+            calendar.set(Calendar.YEAR, 1970);
+            calendar.set(Calendar.MONTH, 0);
+            calendar.set(Calendar.DAY_OF_MONTH, 1);
+            calendar.set(Calendar.HOUR_OF_DAY, hour);
+            calendar.set(Calendar.MINUTE, minutes);
+            calendar.set(Calendar.SECOND, seconds);
+            calendar.set(Calendar.MILLISECOND, microseconds / 1_000);
+            return new Time(calendar.getTimeInMillis());
+          }
+        }
+
+        LocalDateTime ldt =
+            LocalDateTime.of(year, month, day, hour, minutes, seconds, microseconds * 1000);
+        ZonedDateTime zdt =
+            ldt.atZone(timeZone.toZoneId()).withZoneSameInstant(TimeZone.getDefault().toZoneId());
+
+        calendar = cal != null ? cal : Calendar.getInstance();
+        synchronized (calendar) {
+          calendar.clear();
+          calendar.set(Calendar.YEAR, 1970);
+          calendar.set(Calendar.MONTH, 0);
+          calendar.set(Calendar.DAY_OF_MONTH, 1);
+          calendar.set(Calendar.HOUR_OF_DAY, zdt.getHour());
+          calendar.set(Calendar.MINUTE, zdt.getMinute());
+          calendar.set(Calendar.SECOND, zdt.getSecond());
+          calendar.set(Calendar.MILLISECOND, zdt.getNano() / 1_000_000);
+          return new Time(calendar.getTimeInMillis());
+        }
+
       case DATE:
         throw new SQLException("Cannot read Time using a Types.DATE field");
+
       default:
-        Calendar calendar = cal != null ? cal : Calendar.getInstance();
+        calendar = cal != null ? cal : Calendar.getInstance();
         calendar.clear();
-        int day = 0;
-        int hour = 0;
-        int minutes = 0;
-        int seconds = 0;
+
         boolean negate = false;
         if (length > 0) {
           negate = (buf[pos] & 0xff) == 0x01;
@@ -935,7 +1045,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @throws SQLException if column type is not compatible
    */
   public Timestamp getInternalTimestamp(
-      ColumnInformation columnInfo, Calendar userCalendar, TimeZone timeZone) throws SQLException {
+      ColumnDefinition columnInfo, Calendar userCalendar, TimeZone timeZone) throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
@@ -952,9 +1062,10 @@ public class BinaryRowProtocol extends RowProtocol {
     int seconds = 0;
     int microseconds = 0;
 
+    Calendar calendar;
     switch (columnInfo.getColumnType()) {
       case TIME:
-        Calendar calendar = userCalendar != null ? userCalendar : Calendar.getInstance();
+        calendar = userCalendar != null ? userCalendar : Calendar.getInstance();
 
         boolean negate = false;
         if (length > 0) {
@@ -1009,6 +1120,15 @@ public class BinaryRowProtocol extends RowProtocol {
             }
           }
         }
+
+        if (userCalendar != null) {
+          calendar = userCalendar;
+        } else if (timeZone != null) {
+          calendar = Calendar.getInstance(timeZone);
+        } else {
+          calendar = Calendar.getInstance();
+        }
+
         break;
 
       default:
@@ -1028,15 +1148,14 @@ public class BinaryRowProtocol extends RowProtocol {
                     + ((buf[pos + 10] & 0xff) << 24));
           }
         }
-    }
 
-    Calendar calendar;
-    if (userCalendar != null) {
-      calendar = userCalendar;
-    } else if (columnInfo.getColumnType().getSqlType() == Types.TIMESTAMP) {
-      calendar = Calendar.getInstance(timeZone);
-    } else {
-      calendar = Calendar.getInstance();
+        if (userCalendar != null) {
+          calendar = userCalendar;
+        } else if (timeZone != null && columnInfo.getColumnType() != ColumnType.DATE) {
+          calendar = Calendar.getInstance(timeZone);
+        } else {
+          calendar = Calendar.getInstance();
+        }
     }
 
     Timestamp tt;
@@ -1057,7 +1176,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return Object value
    * @throws SQLException if column type is not compatible
    */
-  public Object getInternalObject(ColumnInformation columnInfo, TimeZone timeZone)
+  public Object getInternalObject(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -1141,8 +1260,8 @@ public class BinaryRowProtocol extends RowProtocol {
       default:
         break;
     }
-    throw ExceptionMapper.getFeatureNotSupportedException(
-        "Type '" + columnInfo.getColumnType().getTypeName() + "' is not supported");
+    throw ExceptionFactory.INSTANCE.notSupported(
+        String.format("Type '%s' is not supported", columnInfo.getColumnType().getTypeName()));
   }
 
   /**
@@ -1152,7 +1271,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return boolean value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public boolean getInternalBoolean(ColumnInformation columnInfo) throws SQLException {
+  public boolean getInternalBoolean(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return false;
     }
@@ -1189,7 +1308,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return byte value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public byte getInternalByte(ColumnInformation columnInfo) throws SQLException {
+  public byte getInternalByte(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -1244,7 +1363,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return short value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public short getInternalShort(ColumnInformation columnInfo) throws SQLException {
+  public short getInternalShort(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -1303,7 +1422,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @param columnInfo column information
    * @return time value
    */
-  public String getInternalTimeString(ColumnInformation columnInfo) {
+  public String getInternalTimeString(ColumnDefinition columnInfo) {
     if (lastValueWasNull()) {
       return null;
     }
@@ -1382,7 +1501,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @throws SQLException if column type doesn't permit conversion or value is not in BigInteger
    *     range
    */
-  public BigInteger getInternalBigInteger(ColumnInformation columnInfo) throws SQLException {
+  public BigInteger getInternalBigInteger(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
@@ -1454,7 +1573,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @throws SQLException if column type doesn't permit conversion
    */
   public ZonedDateTime getInternalZonedDateTime(
-      ColumnInformation columnInfo, Class clazz, TimeZone timeZone) throws SQLException {
+      ColumnDefinition columnInfo, Class clazz, TimeZone timeZone) throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
@@ -1488,7 +1607,14 @@ public class BinaryRowProtocol extends RowProtocol {
         }
 
         return ZonedDateTime.of(
-            year, month, day, hour, minutes, seconds, microseconds * 1000, timeZone.toZoneId());
+            year,
+            month,
+            day,
+            hour,
+            minutes,
+            seconds,
+            microseconds * 1000,
+            timeZone == null ? ZoneId.systemDefault() : timeZone.toZoneId());
 
       case Types.VARCHAR:
       case Types.LONGVARCHAR:
@@ -1526,7 +1652,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return OffsetTime value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public OffsetTime getInternalOffsetTime(ColumnInformation columnInfo, TimeZone timeZone)
+  public OffsetTime getInternalOffsetTime(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -1536,7 +1662,8 @@ public class BinaryRowProtocol extends RowProtocol {
       return null;
     }
 
-    ZoneId zoneId = timeZone.toZoneId().normalized();
+    ZoneId zoneId =
+        timeZone == null ? ZoneId.systemDefault().normalized() : timeZone.toZoneId().normalized();
     if (zoneId instanceof ZoneOffset) {
       ZoneOffset zoneOffset = (ZoneOffset) zoneId;
 
@@ -1650,7 +1777,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return LocalTime value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public LocalTime getInternalLocalTime(ColumnInformation columnInfo, TimeZone timeZone)
+  public LocalTime getInternalLocalTime(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -1701,6 +1828,9 @@ public class BinaryRowProtocol extends RowProtocol {
         // string conversion
         String raw = new String(buf, pos, length, StandardCharsets.UTF_8);
         try {
+          if (timeZone == null) {
+            return LocalTime.parse(raw, DateTimeFormatter.ISO_LOCAL_TIME);
+          }
           return LocalTime.parse(
               raw, DateTimeFormatter.ISO_LOCAL_TIME.withZone(timeZone.toZoneId()));
         } catch (DateTimeParseException dateParserEx) {
@@ -1734,7 +1864,7 @@ public class BinaryRowProtocol extends RowProtocol {
    * @return LocalDate value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public LocalDate getInternalLocalDate(ColumnInformation columnInfo, TimeZone timeZone)
+  public LocalDate getInternalLocalDate(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -1767,6 +1897,9 @@ public class BinaryRowProtocol extends RowProtocol {
           return null;
         }
         try {
+          if (timeZone == null) {
+            return LocalDate.parse(raw, DateTimeFormatter.ISO_LOCAL_DATE);
+          }
           return LocalDate.parse(
               raw, DateTimeFormatter.ISO_LOCAL_DATE.withZone(timeZone.toZoneId()));
         } catch (DateTimeParseException dateParserEx) {
