@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,19 +52,22 @@
 
 package org.mariadb.jdbc.internal.com.read.resultset.rowprotocol;
 
-import org.mariadb.jdbc.internal.*;
-import org.mariadb.jdbc.internal.com.read.resultset.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-import org.mariadb.jdbc.util.*;
-
-import java.math.*;
-import java.nio.charset.*;
-import java.sql.Date;
+import java.math.BigDecimal;
+import java.math.BigInteger;
+import java.nio.charset.StandardCharsets;
 import java.sql.*;
-import java.text.*;
+import java.text.DateFormat;
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
 import java.time.*;
-import java.time.format.*;
-import java.util.*;
+import java.time.format.DateTimeFormatter;
+import java.time.format.DateTimeParseException;
+import java.util.Calendar;
+import java.util.TimeZone;
+import org.mariadb.jdbc.internal.ColumnType;
+import org.mariadb.jdbc.internal.com.read.resultset.ColumnDefinition;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
+import org.mariadb.jdbc.util.Options;
 
 public class TextRowProtocol extends RowProtocol {
 
@@ -178,7 +181,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return String value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public String getInternalString(ColumnInformation columnInfo, Calendar cal, TimeZone timeZone)
+  public String getInternalString(ColumnDefinition columnInfo, Calendar cal, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -231,8 +234,7 @@ public class TextRowProtocol extends RowProtocol {
     }
 
     if (maxFieldSize > 0) {
-      return new String(buf, pos, Math.min(maxFieldSize * 3, length), StandardCharsets.UTF_8)
-          .substring(0, Math.min(maxFieldSize, length));
+      return new String(buf, pos, Math.min(maxFieldSize, length), StandardCharsets.UTF_8);
     }
 
     return new String(buf, pos, length, StandardCharsets.UTF_8);
@@ -245,7 +247,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return int value
    * @throws SQLException if column type doesn't permit conversion or not in Integer range
    */
-  public int getInternalInt(ColumnInformation columnInfo) throws SQLException {
+  public int getInternalInt(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -261,7 +263,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return long value
    * @throws SQLException if column type doesn't permit conversion or not in Long range (unsigned)
    */
-  public long getInternalLong(ColumnInformation columnInfo) throws SQLException {
+  public long getInternalLong(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -356,7 +358,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return float value
    * @throws SQLException if column type doesn't permit conversion or not in Float range
    */
-  public float getInternalFloat(ColumnInformation columnInfo) throws SQLException {
+  public float getInternalFloat(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -406,7 +408,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return double value
    * @throws SQLException if column type doesn't permit conversion or not in Double range (unsigned)
    */
-  public double getInternalDouble(ColumnInformation columnInfo) throws SQLException {
+  public double getInternalDouble(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -454,7 +456,7 @@ public class TextRowProtocol extends RowProtocol {
    * @param columnInfo column information
    * @return BigDecimal value
    */
-  public BigDecimal getInternalBigDecimal(ColumnInformation columnInfo) {
+  public BigDecimal getInternalBigDecimal(ColumnDefinition columnInfo) {
     if (lastValueWasNull()) {
       return null;
     }
@@ -475,16 +477,16 @@ public class TextRowProtocol extends RowProtocol {
    * @throws SQLException if column type doesn't permit conversion
    */
   @SuppressWarnings("deprecation")
-  public Date getInternalDate(ColumnInformation columnInfo, Calendar cal, TimeZone timeZone)
+  public Date getInternalDate(ColumnDefinition columnInfo, Calendar cal, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
 
+    int partIdx = 0;
     switch (columnInfo.getColumnType()) {
       case DATE:
         int[] datePart = new int[] {0, 0, 0};
-        int partIdx = 0;
         for (int begin = pos; begin < pos + length; begin++) {
           byte b = buf[begin];
           if (b == '-') {
@@ -509,11 +511,87 @@ public class TextRowProtocol extends RowProtocol {
 
       case TIMESTAMP:
       case DATETIME:
-        Timestamp timestamp = getInternalTimestamp(columnInfo, cal, timeZone);
-        if (timestamp == null) {
+        int nanoBegin = -1;
+        int[] timestampsPart = new int[] {0, 0, 0, 0, 0, 0, 0};
+        for (int begin = pos; begin < pos + length; begin++) {
+          byte b = buf[begin];
+          if (b == '-' || b == ' ' || b == ':') {
+            partIdx++;
+            continue;
+          }
+          if (b == '.') {
+            partIdx++;
+            nanoBegin = begin;
+            continue;
+          }
+          if (b < '0' || b > '9') {
+            throw new SQLException(
+                "cannot parse data in timestamp string '"
+                    + new String(buf, pos, length, StandardCharsets.UTF_8)
+                    + "'");
+          }
+
+          timestampsPart[partIdx] = timestampsPart[partIdx] * 10 + b - 48;
+        }
+        if (timestampsPart[0] == 0
+            && timestampsPart[1] == 0
+            && timestampsPart[2] == 0
+            && timestampsPart[3] == 0
+            && timestampsPart[4] == 0
+            && timestampsPart[5] == 0
+            && timestampsPart[6] == 0) {
+          lastValueNull |= BIT_LAST_ZERO_DATE;
           return null;
         }
-        return new Date(timestamp.getTime());
+
+        // fix non leading tray for nanoseconds
+        if (nanoBegin > 0) {
+          for (int begin = 0; begin < 6 - (pos + length - nanoBegin - 1); begin++) {
+            timestampsPart[6] = timestampsPart[6] * 10;
+          }
+        }
+
+        if (timeZone == null) {
+          // legacy is to send timestamps with current driver timezone. So display, is immediate
+          Calendar calendar = (cal != null) ? cal : Calendar.getInstance();
+          synchronized (calendar) {
+            calendar.clear();
+            calendar.set(Calendar.YEAR, timestampsPart[0]);
+            calendar.set(Calendar.MONTH, timestampsPart[1] - 1);
+            calendar.set(Calendar.DAY_OF_MONTH, timestampsPart[2]);
+            calendar.set(Calendar.HOUR_OF_DAY, 0);
+            calendar.set(Calendar.MINUTE, 0);
+            calendar.set(Calendar.SECOND, 0);
+            calendar.set(Calendar.MILLISECOND, 0);
+            return new Date(calendar.getTimeInMillis());
+          }
+        }
+
+        // timestamp is saved in server timezone,
+        LocalDateTime ldt =
+            LocalDateTime.of(
+                timestampsPart[0],
+                timestampsPart[1],
+                timestampsPart[2],
+                timestampsPart[3],
+                timestampsPart[4],
+                timestampsPart[5],
+                timestampsPart[6] * 1000);
+        ZonedDateTime zdt =
+            ldt.atZone(timeZone.toZoneId()).withZoneSameInstant(TimeZone.getDefault().toZoneId());
+
+        Calendar calendar = cal != null ? cal : Calendar.getInstance();
+        synchronized (calendar) {
+          calendar.clear();
+          calendar.set(Calendar.YEAR, zdt.getYear());
+          calendar.set(Calendar.MONTH, zdt.getMonthValue() - 1);
+          calendar.set(Calendar.DAY_OF_MONTH, zdt.getDayOfMonth());
+          calendar.set(Calendar.HOUR_OF_DAY, 0);
+          calendar.set(Calendar.MINUTE, 0);
+          calendar.set(Calendar.SECOND, 0);
+          calendar.set(Calendar.MILLISECOND, 0);
+          return new Date(calendar.getTimeInMillis());
+        }
 
       case TIME:
         throw new SQLException("Cannot read DATE using a Types.TIME field");
@@ -535,12 +613,14 @@ public class TextRowProtocol extends RowProtocol {
       default:
         try {
           DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
-          sdf.setTimeZone(timeZone);
+          if (timeZone != null) {
+            sdf.setTimeZone(timeZone);
+          }
           java.util.Date utilDate = sdf.parse(new String(buf, pos, length, StandardCharsets.UTF_8));
           return new Date(utilDate.getTime());
 
         } catch (ParseException e) {
-          throw ExceptionMapper.getSqlException(
+          throw ExceptionFactory.INSTANCE.create(
               "Could not get object as Date : " + e.getMessage(), "S1009", e);
         }
     }
@@ -555,7 +635,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return time value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public Time getInternalTime(ColumnInformation columnInfo, Calendar cal, TimeZone timeZone)
+  public Time getInternalTime(ColumnDefinition columnInfo, Calendar cal, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -563,8 +643,86 @@ public class TextRowProtocol extends RowProtocol {
 
     if (columnInfo.getColumnType() == ColumnType.TIMESTAMP
         || columnInfo.getColumnType() == ColumnType.DATETIME) {
-      Timestamp timestamp = getInternalTimestamp(columnInfo, cal, timeZone);
-      return (timestamp == null) ? null : new Time(timestamp.getTime());
+      int nanoBegin = -1;
+      int[] timestampsPart = new int[] {0, 0, 0, 0, 0, 0, 0};
+      int partIdx = 0;
+      for (int begin = pos; begin < pos + length; begin++) {
+        byte b = buf[begin];
+        if (b == '-' || b == ' ' || b == ':') {
+          partIdx++;
+          continue;
+        }
+        if (b == '.') {
+          partIdx++;
+          nanoBegin = begin;
+          continue;
+        }
+        if (b < '0' || b > '9') {
+          throw new SQLException(
+              "cannot parse data in timestamp string '"
+                  + new String(buf, pos, length, StandardCharsets.UTF_8)
+                  + "'");
+        }
+
+        timestampsPart[partIdx] = timestampsPart[partIdx] * 10 + b - 48;
+      }
+      if (timestampsPart[0] == 0
+          && timestampsPart[1] == 0
+          && timestampsPart[2] == 0
+          && timestampsPart[3] == 0
+          && timestampsPart[4] == 0
+          && timestampsPart[5] == 0
+          && timestampsPart[6] == 0) {
+        lastValueNull |= BIT_LAST_ZERO_DATE;
+        return null;
+      }
+
+      // fix non leading tray for nanoseconds
+      if (nanoBegin > 0) {
+        for (int begin = 0; begin < 6 - (pos + length - nanoBegin - 1); begin++) {
+          timestampsPart[6] = timestampsPart[6] * 10;
+        }
+      }
+
+      if (timeZone == null) {
+        Calendar calendar = cal != null ? cal : Calendar.getInstance();
+        synchronized (calendar) {
+          calendar.clear();
+          calendar.set(Calendar.YEAR, 1970);
+          calendar.set(Calendar.MONTH, 0);
+          calendar.set(Calendar.DAY_OF_MONTH, 1);
+          calendar.set(Calendar.HOUR_OF_DAY, timestampsPart[3]);
+          calendar.set(Calendar.MINUTE, timestampsPart[4]);
+          calendar.set(Calendar.SECOND, timestampsPart[5]);
+          calendar.set(Calendar.MILLISECOND, timestampsPart[6] / 1_000);
+          return new Time(calendar.getTimeInMillis());
+        }
+      }
+
+      LocalDateTime ldt =
+          LocalDateTime.of(
+              timestampsPart[0],
+              timestampsPart[1],
+              timestampsPart[2],
+              timestampsPart[3],
+              timestampsPart[4],
+              timestampsPart[5],
+              timestampsPart[6] * 1000);
+      ZonedDateTime zdt =
+          ldt.atZone(timeZone.toZoneId()).withZoneSameInstant(TimeZone.getDefault().toZoneId());
+
+      Calendar calendar = cal != null ? cal : Calendar.getInstance();
+      synchronized (calendar) {
+        calendar.clear();
+        calendar.set(Calendar.YEAR, 1970);
+        calendar.set(Calendar.MONTH, 0);
+        calendar.set(Calendar.DAY_OF_MONTH, 1);
+        calendar.set(Calendar.HOUR_OF_DAY, zdt.getHour());
+        calendar.set(Calendar.MINUTE, zdt.getMinute());
+        calendar.set(Calendar.SECOND, zdt.getSecond());
+        calendar.set(Calendar.MILLISECOND, zdt.getNano() / 1_000_000);
+        return new Time(calendar.getTimeInMillis());
+      }
 
     } else if (columnInfo.getColumnType() == ColumnType.DATE) {
 
@@ -612,7 +770,7 @@ public class TextRowProtocol extends RowProtocol {
    * @throws SQLException if column type doesn't permit conversion
    */
   public Timestamp getInternalTimestamp(
-      ColumnInformation columnInfo, Calendar userCalendar, TimeZone timeZone) throws SQLException {
+      ColumnDefinition columnInfo, Calendar userCalendar, TimeZone timeZone) throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
@@ -670,7 +828,7 @@ public class TextRowProtocol extends RowProtocol {
         Calendar calendar;
         if (userCalendar != null) {
           calendar = userCalendar;
-        } else if (columnInfo.getColumnType().getSqlType() == Types.TIMESTAMP) {
+        } else if (timeZone != null && columnInfo.getColumnType() != ColumnType.DATE) {
           calendar = Calendar.getInstance(timeZone);
         } else {
           calendar = Calendar.getInstance();
@@ -716,7 +874,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return Object value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public Object getInternalObject(ColumnInformation columnInfo, TimeZone timeZone)
+  public Object getInternalObject(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -800,7 +958,7 @@ public class TextRowProtocol extends RowProtocol {
       default:
         break;
     }
-    throw ExceptionMapper.getFeatureNotSupportedException(
+    throw ExceptionFactory.INSTANCE.notSupported(
         "Type '" + columnInfo.getColumnType().getTypeName() + "' is not supported");
   }
 
@@ -810,7 +968,7 @@ public class TextRowProtocol extends RowProtocol {
    * @param columnInfo column information
    * @return boolean value
    */
-  public boolean getInternalBoolean(ColumnInformation columnInfo) {
+  public boolean getInternalBoolean(ColumnDefinition columnInfo) {
     if (lastValueWasNull()) {
       return false;
     }
@@ -829,7 +987,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return byte value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public byte getInternalByte(ColumnInformation columnInfo) throws SQLException {
+  public byte getInternalByte(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -845,7 +1003,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return short value
    * @throws SQLException if column type doesn't permit conversion or value is not in Short range
    */
-  public short getInternalShort(ColumnInformation columnInfo) throws SQLException {
+  public short getInternalShort(ColumnDefinition columnInfo) throws SQLException {
     if (lastValueWasNull()) {
       return 0;
     }
@@ -860,7 +1018,7 @@ public class TextRowProtocol extends RowProtocol {
    * @param columnInfo column information
    * @return String representation of time
    */
-  public String getInternalTimeString(ColumnInformation columnInfo) {
+  public String getInternalTimeString(ColumnDefinition columnInfo) {
     if (lastValueWasNull()) {
       return null;
     }
@@ -884,7 +1042,7 @@ public class TextRowProtocol extends RowProtocol {
    * @param columnInfo column information
    * @return BigInteger value
    */
-  public BigInteger getInternalBigInteger(ColumnInformation columnInfo) {
+  public BigInteger getInternalBigInteger(ColumnDefinition columnInfo) {
     if (lastValueWasNull()) {
       return null;
     }
@@ -901,7 +1059,7 @@ public class TextRowProtocol extends RowProtocol {
    * @throws SQLException if column type doesn't permit conversion
    */
   public ZonedDateTime getInternalZonedDateTime(
-      ColumnInformation columnInfo, Class clazz, TimeZone timeZone) throws SQLException {
+      ColumnDefinition columnInfo, Class clazz, TimeZone timeZone) throws SQLException {
     if (lastValueWasNull()) {
       return null;
     }
@@ -918,6 +1076,11 @@ public class TextRowProtocol extends RowProtocol {
           return null;
         }
         try {
+          if (timeZone == null) {
+            LocalDateTime localDateTime = LocalDateTime.parse(raw, TEXT_LOCAL_DATE_TIME);
+            return ZonedDateTime.of(localDateTime, TimeZone.getDefault().toZoneId());
+          }
+
           LocalDateTime localDateTime =
               LocalDateTime.parse(raw, TEXT_LOCAL_DATE_TIME.withZone(timeZone.toZoneId()));
           return ZonedDateTime.of(localDateTime, timeZone.toZoneId());
@@ -960,7 +1123,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return OffsetTime value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public OffsetTime getInternalOffsetTime(ColumnInformation columnInfo, TimeZone timeZone)
+  public OffsetTime getInternalOffsetTime(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -970,7 +1133,10 @@ public class TextRowProtocol extends RowProtocol {
       return null;
     }
 
-    ZoneId zoneId = timeZone.toZoneId().normalized();
+    ZoneId zoneId =
+        timeZone != null
+            ? timeZone.toZoneId().normalized()
+            : TimeZone.getDefault().toZoneId().normalized();
     if (zoneId instanceof ZoneOffset) {
       ZoneOffset zoneOffset = (ZoneOffset) zoneId;
       String raw = new String(buf, pos, length, StandardCharsets.UTF_8);
@@ -1048,7 +1214,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return LocalTime value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public LocalTime getInternalLocalTime(ColumnInformation columnInfo, TimeZone timeZone)
+  public LocalTime getInternalLocalTime(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -1066,6 +1232,9 @@ public class TextRowProtocol extends RowProtocol {
       case Types.LONGVARCHAR:
       case Types.CHAR:
         try {
+          if (timeZone == null) {
+            return LocalTime.parse(raw, DateTimeFormatter.ISO_LOCAL_TIME);
+          }
           return LocalTime.parse(
               raw, DateTimeFormatter.ISO_LOCAL_TIME.withZone(timeZone.toZoneId()));
         } catch (DateTimeParseException dateParserEx) {
@@ -1099,7 +1268,7 @@ public class TextRowProtocol extends RowProtocol {
    * @return LocalDate value
    * @throws SQLException if column type doesn't permit conversion
    */
-  public LocalDate getInternalLocalDate(ColumnInformation columnInfo, TimeZone timeZone)
+  public LocalDate getInternalLocalDate(ColumnDefinition columnInfo, TimeZone timeZone)
       throws SQLException {
     if (lastValueWasNull()) {
       return null;
@@ -1120,6 +1289,9 @@ public class TextRowProtocol extends RowProtocol {
           return null;
         }
         try {
+          if (timeZone == null) {
+            return LocalDate.parse(raw, DateTimeFormatter.ISO_LOCAL_DATE);
+          }
           return LocalDate.parse(
               raw, DateTimeFormatter.ISO_LOCAL_DATE.withZone(timeZone.toZoneId()));
         } catch (DateTimeParseException dateParserEx) {
