@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,10 +52,12 @@
 
 package org.mariadb.jdbc.internal.com.send.parameters;
 
-import org.mariadb.jdbc.internal.*;
-import org.mariadb.jdbc.internal.io.output.*;
-
-import java.io.*;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.Reader;
+import java.nio.charset.StandardCharsets;
+import org.mariadb.jdbc.internal.ColumnType;
+import org.mariadb.jdbc.internal.io.output.PacketOutputStream;
 
 public class ReaderParameter implements Cloneable, ParameterHolder {
 
@@ -101,7 +103,7 @@ public class ReaderParameter implements Cloneable, ParameterHolder {
    *
    * @return approximated data length.
    */
-  public long getApproximateTextProtocolLength() {
+  public int getApproximateTextProtocolLength() {
     return -1;
   }
 
@@ -112,6 +114,29 @@ public class ReaderParameter implements Cloneable, ParameterHolder {
    * @throws IOException if socket error occur
    */
   public void writeBinary(final PacketOutputStream pos) throws IOException {
+    ByteArrayOutputStream bb = new ByteArrayOutputStream();
+    char[] buf = new char[4096];
+    int len;
+    if (length == Long.MAX_VALUE) {
+      while ((len = reader.read(buf)) >= 0) {
+        byte[] data = new String(buf, 0, len).getBytes(StandardCharsets.UTF_8);
+        bb.write(data, 0, data.length);
+      }
+    } else {
+      long maxLen = length;
+      while ((len = reader.read(buf)) >= 0 && maxLen > 0) {
+        byte[] data =
+            new String(buf, 0, Math.min(len, (int) maxLen)).getBytes(StandardCharsets.UTF_8);
+        maxLen -= len;
+        bb.write(data, 0, data.length);
+      }
+    }
+    byte[] val = bb.toByteArray();
+    pos.writeFieldLength(val.length);
+    pos.write(val);
+  }
+
+  public void writeLongData(final PacketOutputStream pos) throws IOException {
     if (length == Long.MAX_VALUE) {
       pos.write(reader, false, noBackslashEscapes);
     } else {
@@ -132,7 +157,7 @@ public class ReaderParameter implements Cloneable, ParameterHolder {
     return false;
   }
 
-  public boolean isLongData() {
+  public boolean canBeLongData() {
     return true;
   }
 }
