@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -52,39 +52,63 @@
 
 package org.mariadb.jdbc.internal.protocol;
 
-import org.mariadb.jdbc.*;
-import org.mariadb.jdbc.internal.*;
-import org.mariadb.jdbc.internal.com.read.*;
-import org.mariadb.jdbc.internal.com.read.dao.*;
-import org.mariadb.jdbc.internal.com.read.resultset.*;
-import org.mariadb.jdbc.internal.com.send.*;
-import org.mariadb.jdbc.internal.com.send.parameters.*;
-import org.mariadb.jdbc.internal.io.output.*;
-import org.mariadb.jdbc.internal.logging.*;
-import org.mariadb.jdbc.internal.util.*;
-import org.mariadb.jdbc.internal.util.constant.*;
-import org.mariadb.jdbc.internal.util.dao.*;
-import org.mariadb.jdbc.internal.util.exceptions.*;
-import org.mariadb.jdbc.internal.util.pool.*;
-import org.mariadb.jdbc.internal.util.scheduler.*;
-
-import java.io.*;
-import java.net.*;
-import java.nio.charset.*;
-import java.sql.*;
-import java.util.*;
-import java.util.concurrent.*;
-import java.util.concurrent.locks.*;
-
 import static org.mariadb.jdbc.internal.com.Packet.*;
 import static org.mariadb.jdbc.internal.util.SqlStates.*;
 
+import java.io.*;
+import java.net.SocketException;
+import java.net.SocketTimeoutException;
+import java.net.URL;
+import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
+import java.sql.*;
+import java.util.*;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.FutureTask;
+import java.util.concurrent.ThreadPoolExecutor;
+import java.util.concurrent.locks.ReentrantLock;
+import javax.sql.rowset.serial.SerialException;
+import org.mariadb.jdbc.LocalInfileInterceptor;
+import org.mariadb.jdbc.MariaDbConnection;
+import org.mariadb.jdbc.MariaDbStatement;
+import org.mariadb.jdbc.UrlParser;
+import org.mariadb.jdbc.internal.MariaDbServerCapabilities;
+import org.mariadb.jdbc.internal.com.read.Buffer;
+import org.mariadb.jdbc.internal.com.read.ErrorPacket;
+import org.mariadb.jdbc.internal.com.read.dao.Results;
+import org.mariadb.jdbc.internal.com.read.resultset.ColumnDefinition;
+import org.mariadb.jdbc.internal.com.read.resultset.SelectResultSet;
+import org.mariadb.jdbc.internal.com.read.resultset.UpdatableResultSet;
+import org.mariadb.jdbc.internal.com.send.ComQuery;
+import org.mariadb.jdbc.internal.com.send.ComStmtExecute;
+import org.mariadb.jdbc.internal.com.send.ComStmtPrepare;
+import org.mariadb.jdbc.internal.com.send.SendChangeDbPacket;
+import org.mariadb.jdbc.internal.com.send.parameters.ParameterHolder;
+import org.mariadb.jdbc.internal.io.LruTraceCache;
+import org.mariadb.jdbc.internal.io.output.PacketOutputStream;
+import org.mariadb.jdbc.internal.logging.Logger;
+import org.mariadb.jdbc.internal.logging.LoggerFactory;
+import org.mariadb.jdbc.internal.util.BulkStatus;
+import org.mariadb.jdbc.internal.util.LogQueryTool;
+import org.mariadb.jdbc.internal.util.SqlStates;
+import org.mariadb.jdbc.internal.util.Utils;
+import org.mariadb.jdbc.internal.util.constant.ServerStatus;
+import org.mariadb.jdbc.internal.util.constant.StateChange;
+import org.mariadb.jdbc.internal.util.dao.ClientPrepareResult;
+import org.mariadb.jdbc.internal.util.dao.PrepareResult;
+import org.mariadb.jdbc.internal.util.dao.ServerPrepareResult;
+import org.mariadb.jdbc.internal.util.exceptions.ExceptionFactory;
+import org.mariadb.jdbc.internal.util.exceptions.MariaDbSqlException;
+import org.mariadb.jdbc.internal.util.exceptions.MaxAllowedPacketException;
+import org.mariadb.jdbc.internal.util.pool.GlobalStateInfo;
+import org.mariadb.jdbc.internal.util.scheduler.SchedulerServiceProviderHolder;
+
 public class AbstractQueryProtocol extends AbstractConnectProtocol implements Protocol {
 
   private static final Logger logger = LoggerFactory.getLogger(AbstractQueryProtocol.class);
-  private static final String CHECK_GALERA_STATE_QUERY = "show status like 'wsrep_local_state'";
-  private final LogQueryTool logQuery;
-  private final List<String> galeraAllowedStates;
+  private static final Set<Integer> LOCK_DEADLOCK_ERROR_CODES =
+      new HashSet<>(Arrays.asList(1205, 1213, 1614));
+
   private ThreadPoolExecutor readScheduler = null;
   private int transactionIsolationLevel = 0;
   private InputStream localInfileInputStream;
@@ -98,15 +122,14 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    *
    * @param urlParser connection URL information's
    * @param lock the lock for thread synchronisation
+   * @param traceCache trace cache
    */
   AbstractQueryProtocol(
-      final UrlParser urlParser, final GlobalStateInfo globalInfo, final ReentrantLock lock) {
-    super(urlParser, globalInfo, lock);
-    logQuery = new LogQueryTool(options);
-    galeraAllowedStates =
-        urlParser.getOptions().galeraAllowedState == null
-            ? Collections.emptyList()
-            : Arrays.asList(urlParser.getOptions().galeraAllowedState.split(","));
+      final UrlParser urlParser,
+      final GlobalStateInfo globalInfo,
+      final ReentrantLock lock,
+      LruTraceCache traceCache) {
+    super(urlParser, globalInfo, lock, traceCache);
   }
 
   /**
@@ -137,13 +160,66 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       }
 
     } catch (SQLException sqlException) {
-      throw logQuery.exceptionWithQuery(
-          "COM_RESET_CONNECTION failed.", sqlException, explicitClosed);
+      throw exceptionWithQuery("COM_RESET_CONNECTION failed.", sqlException, explicitClosed);
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(
+          "COM_RESET_CONNECTION failed.", handleIoException(e), explicitClosed);
     }
   }
 
+  private MariaDbSqlException exceptionWithQuery(
+      ParameterHolder[] parameters,
+      PrepareResult serverPrepareResult,
+      SQLException sqlException,
+      boolean explicitClosed) {
+    return exceptionWithQuery(
+        LogQueryTool.queryWithParams(serverPrepareResult, parameters, options),
+        sqlException,
+        explicitClosed);
+  }
+
+  private MariaDbSqlException exceptionWithQuery(
+      String sql, SQLException sqlException, boolean explicitClosed) {
+    MariaDbSqlException ex;
+    if (explicitClosed) {
+      ex =
+          new MariaDbSqlException(
+              "Connection has explicitly been closed/aborted.", sql, sqlException);
+    } else {
+      if (sqlException.getCause() instanceof SocketTimeoutException) {
+        ex = new MariaDbSqlException("Connection timed out", sql, "08000", sqlException);
+      } else {
+        ex = MariaDbSqlException.of(sqlException, sql);
+      }
+    }
+
+    if (options.includeThreadDumpInDeadlockExceptions || sqlException.getErrorCode() == 1064) {
+      ex.withThreadName(Thread.currentThread().getName());
+    }
+
+    // Add innoDB status if asked
+    if (options.includeInnodbStatusInDeadlockExceptions
+        && sqlException.getSQLState() != null
+        && LOCK_DEADLOCK_ERROR_CODES.contains(sqlException.getErrorCode())) {
+      try {
+        lock.lock();
+        cmdPrologue();
+        Results results = new Results();
+        executeQuery(isMasterConnection(), results, "SHOW ENGINE INNODB STATUS");
+        results.commandEnd();
+        ResultSet rs = results.getResultSet();
+        if (rs.next()) {
+          return ex.withDeadLockInfo(rs.getString(3));
+        }
+      } catch (SQLException sqle) {
+        // eat
+      } finally {
+        lock.unlock();
+      }
+    }
+    return ex;
+  }
+
   /**
    * Execute internal query.
    *
@@ -181,9 +257,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       if ("70100".equals(sqlException.getSQLState()) && 1927 == sqlException.getErrorCode()) {
         throw handleIoException(sqlException);
       }
-      throw logQuery.exceptionWithQuery(sql, sqlException, explicitClosed);
+      throw exceptionWithQuery(sql, sqlException, explicitClosed);
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(sql, handleIoException(e), explicitClosed);
     }
   }
 
@@ -201,9 +277,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       getResult(results);
 
     } catch (SQLException sqlException) {
-      throw logQuery.exceptionWithQuery(sql, sqlException, explicitClosed);
+      throw exceptionWithQuery(sql, sqlException, explicitClosed);
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(sql, handleIoException(e), explicitClosed);
     }
   }
 
@@ -240,9 +316,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       getResult(results);
 
     } catch (SQLException queryException) {
-      throw logQuery.exceptionWithQuery(parameters, queryException, clientPrepareResult);
+      throw exceptionWithQuery(parameters, clientPrepareResult, queryException, false);
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(parameters, clientPrepareResult, handleIoException(e), false);
     }
   }
 
@@ -281,9 +357,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       getResult(results);
 
     } catch (SQLException queryException) {
-      throw logQuery.exceptionWithQuery(parameters, queryException, clientPrepareResult);
+      throw exceptionWithQuery(parameters, clientPrepareResult, queryException, false);
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(parameters, clientPrepareResult, handleIoException(e), false);
     }
   }
 
@@ -294,15 +370,13 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    * @param results results
    * @param prepareResult ClientPrepareResult
    * @param parametersList List of parameters
-   * @param hasLongData has parameter with long data (stream)
    * @throws SQLException exception
    */
   public boolean executeBatchClient(
       boolean mustExecuteOnMaster,
       Results results,
       final ClientPrepareResult prepareResult,
-      final List<ParameterHolder[]> parametersList,
-      boolean hasLongData)
+      final List<ParameterHolder[]> parametersList)
       throws SQLException {
 
     // ***********************************************************************************************************
@@ -326,10 +400,8 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       } else if (prepareResult.isQueryMultipleRewritable()) {
 
         if (options.useBulkStmts
-            && !hasLongData
             && prepareResult.isQueryMultipleRewritable() // INSERT FROM SELECT not allowed
             && results.getAutoGeneratedKeys() == Statement.NO_GENERATED_KEYS
-            && versionGreaterOrEqual(10, 2, 7)
             && executeBulkBatch(results, prepareResult.getSql(), null, parametersList)) {
           return true;
         }
@@ -342,9 +414,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     }
 
     if (options.useBulkStmts
-        && !hasLongData
         && results.getAutoGeneratedKeys() == Statement.NO_GENERATED_KEYS
-        && versionGreaterOrEqual(10, 2, 7)
         && executeBulkBatch(results, prepareResult.getSql(), null, parametersList)) {
       return true;
     }
@@ -377,21 +447,20 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
     // **************************************************************************************
     // Ensure BULK can be use :
-    // - server version >= 10.2.7
-    // - no stream
+    // - server support bulk
     // - parameter type doesn't change
     // - avoid INSERT FROM SELECT
     // **************************************************************************************
+    if ((serverCapabilities & MariaDbServerCapabilities.MARIADB_CLIENT_STMT_BULK_OPERATIONS) == 0)
+      return false;
 
-    // ensure that there is no long data and type doesn't change
+    // ensure that type doesn't change
     ParameterHolder[] initParameters = parametersList.get(0);
     int parameterCount = initParameters.length;
     short[] types = new short[parameterCount];
     for (int i = 0; i < parameterCount; i++) {
       types[i] = initParameters[i].getColumnType().getType();
     }
-
-    // must ensure that data type doesn't change
     for (ParameterHolder[] parameters : parametersList) {
       for (int i = 0; i < parameterCount; i++) {
         if (parameters[i].getColumnType().getType() != types[i]) {
@@ -406,7 +475,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     }
 
     cmdPrologue();
-
+    ParameterHolder[] parameters = null;
     ServerPrepareResult tmpServerPrepareResult = serverPrepareResult;
     try {
       SQLException exception = null;
@@ -447,7 +516,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
         }
 
         for (; index < parametersList.size(); index++) {
-          ParameterHolder[] parameters = parametersList.get(index);
+          parameters = parametersList.get(index);
           for (int i = 0; i < parameterCount; i++) {
             ParameterHolder holder = parameters[i];
             if (holder.isNullData()) {
@@ -491,7 +560,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
             return false;
           }
           if (exception == null) {
-            exception = logQuery.exceptionWithQuery(sql, sqle, explicitClosed);
+            exception = exceptionWithQuery(sql, sqle, explicitClosed);
             if (!options.continueBatchOnError) {
               throw exception;
             }
@@ -519,7 +588,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
             return false;
           }
           if (exception == null) {
-            exception = logQuery.exceptionWithQuery(sql, sqle, explicitClosed);
+            exception = exceptionWithQuery(sql, sqle, explicitClosed);
             if (!options.continueBatchOnError) {
               throw exception;
             }
@@ -534,7 +603,8 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       return true;
 
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(
+          parameters, tmpServerPrepareResult, handleIoException(e), explicitClosed);
     } finally {
       if (serverPrepareResult == null && tmpServerPrepareResult != null) {
         releasePrepareStatement(tmpServerPrepareResult);
@@ -599,18 +669,30 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
         int counter = results.getCurrentStatNumber() - 1;
         ParameterHolder[] parameters = parametersList.get(counter);
         List<byte[]> queryParts = clientPrepareResult.getQueryParts();
-        StringBuilder sql = new StringBuilder(new String(queryParts.get(0)));
 
-        for (int i = 0; i < paramCount; i++) {
-          sql.append(parameters[i].toString()).append(new String(queryParts.get(i + 1)));
+        StringBuilder sql;
+        if (clientPrepareResult.isRewriteType()) {
+          // build sql from rewrite parsing
+          sql = new StringBuilder(new String(queryParts.get(0)));
+          sql.append(new String(queryParts.get(1)));
+          for (int i = 0; i < paramCount; i++) {
+            sql.append(parameters[i].toString()).append(new String(queryParts.get(i + 2)));
+          }
+          sql.append(new String(queryParts.get(paramCount + 2)));
+        } else {
+          // build sql from basic parsing
+          sql = new StringBuilder(new String(queryParts.get(0)));
+          for (int i = 0; i < paramCount; i++) {
+            sql.append(parameters[i].toString()).append(new String(queryParts.get(i + 1)));
+          }
         }
 
-        return logQuery.exceptionWithQuery(sql.toString(), qex, explicitClosed);
+        return exceptionWithQuery(sql.toString(), qex, explicitClosed);
       }
 
       @Override
       public int getParamCount() {
-        return clientPrepareResult.getQueryParts().size() - 1;
+        return clientPrepareResult.getParamCount();
       }
 
       @Override
@@ -686,14 +768,14 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
         } catch (SQLException sqlException) {
           if (exception == null) {
-            exception = logQuery.exceptionWithQuery(sql, sqlException, explicitClosed);
+            exception = exceptionWithQuery(sql, sqlException, explicitClosed);
             if (!options.continueBatchOnError) {
               throw exception;
             }
           }
         } catch (IOException e) {
           if (exception == null) {
-            exception = handleIoException(e);
+            exception = exceptionWithQuery(sql, handleIoException(e), explicitClosed);
             if (!options.continueBatchOnError) {
               throw exception;
             }
@@ -740,7 +822,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
           PrepareResult prepareResult) {
 
         String sql = queries.get(currentCounter + sendCmdCounter);
-        return logQuery.exceptionWithQuery(sql, qex, explicitClosed);
+        return exceptionWithQuery(sql, qex, explicitClosed);
       }
 
       @Override
@@ -762,9 +844,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    * <p>For failover, two additional information are in the result-set object : - current connection
    * : Since server maintain a state of this prepare statement, all query will be executed on this
    * particular connection. - executeOnMaster : state of current connection when creating this
-   * prepareStatement (if was on master, will only be executed on master. If was on a slave, can be
-   * execute temporary on master, but we keep this flag, so when a slave is connected back to
-   * relaunch this query on slave)
+   * prepareStatement (if was on master, will only be executed on master. If was on a replica, can
+   * be execute temporary on master, but we keep this flag, so when a replica is connected back to
+   * relaunch this query on replica)
    *
    * @param sql the query
    * @param executeOnMaster state of current connection when creating this prepareStatement
@@ -774,26 +856,22 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
   @Override
   public ServerPrepareResult prepare(String sql, boolean executeOnMaster) throws SQLException {
 
-    cmdPrologue();
     lock.lock();
     try {
+      // search in cache first
       if (options.cachePrepStmts && options.useServerPrepStmts) {
-
         ServerPrepareResult pr = serverPrepareStatementCache.get(database + "-" + sql);
-
         if (pr != null && pr.incrementShareCounter()) {
           return pr;
         }
       }
-      writer.startPacket(0);
-      writer.write(COM_STMT_PREPARE);
-      writer.write(sql);
-      writer.flush();
 
-      ComStmtPrepare comStmtPrepare = new ComStmtPrepare(this, sql);
-      return comStmtPrepare.read(reader, eofDeprecated);
+      cmdPrologue();
+
+      return new ComStmtPrepare(this, sql).send(writer).read(reader, eofDeprecated);
+
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(sql, handleIoException(e), explicitClosed);
     } finally {
       lock.unlock();
     }
@@ -835,13 +913,13 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
       } catch (SQLException sqlException) {
         if (exception == null) {
-          exception = logQuery.exceptionWithQuery(firstSql, sqlException, explicitClosed);
+          exception = exceptionWithQuery(firstSql, sqlException, explicitClosed);
           if (!options.continueBatchOnError) {
             throw exception;
           }
         }
       } catch (IOException e) {
-        throw handleIoException(e);
+        throw exceptionWithQuery(firstSql, handleIoException(e), explicitClosed);
       }
       stopIfInterrupted();
 
@@ -870,7 +948,6 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
     cmdPrologue();
 
-    ParameterHolder[] parameters;
     int currentIndex = 0;
     int totalParameterList = parameterList.size();
 
@@ -896,9 +973,10 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       } while (currentIndex < totalParameterList);
 
     } catch (SQLException sqlEx) {
-      throw logQuery.exceptionWithQuery(sqlEx, prepareResult);
+      throw MariaDbSqlException.of(sqlEx, prepareResult.getSql());
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(
+          parameterList.get(currentIndex), prepareResult, handleIoException(e), explicitClosed);
     } finally {
       results.setRewritten(rewriteValues);
     }
@@ -912,7 +990,6 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    * @param results execution results
    * @param sql sql query if needed to be prepared
    * @param parametersList parameter list
-   * @param hasLongData has long data (stream)
    * @return executed
    * @throws SQLException if parameter error or connection error occur.
    */
@@ -921,16 +998,13 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       ServerPrepareResult serverPrepareResult,
       Results results,
       String sql,
-      final List<ParameterHolder[]> parametersList,
-      boolean hasLongData)
+      final List<ParameterHolder[]> parametersList)
       throws SQLException {
 
     cmdPrologue();
 
     if (options.useBulkStmts
-        && !hasLongData
         && results.getAutoGeneratedKeys() == Statement.NO_GENERATED_KEYS
-        && versionGreaterOrEqual(10, 2, 7)
         && executeBulkBatch(results, sql, serverPrepareResult, parametersList)) {
       return true;
     }
@@ -940,7 +1014,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     }
     initializeBatchReader();
     new AbstractMultiSend(
-        this, writer, results, serverPrepareResult, parametersList, true, sql, readScheduler) {
+        this, writer, results, serverPrepareResult, parametersList, sql, readScheduler) {
       @Override
       public void sendCmd(
           PacketOutputStream writer,
@@ -962,12 +1036,12 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
         // send binary data in a separate stream
         for (int i = 0; i < paramCount; i++) {
-          if (parameters[i].isLongData()) {
+          if (parameters[i].canBeLongData()) {
             writer.startPacket(0);
             writer.write(COM_STMT_SEND_LONG_DATA);
             writer.writeInt(statementId);
             writer.writeShort((short) i);
-            parameters[i].writeBinary(writer);
+            parameters[i].writeLongData(writer);
             writer.flush();
           }
         }
@@ -993,14 +1067,12 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
           int sendCmdCounter,
           int paramCount,
           PrepareResult prepareResult) {
-        return logQuery.exceptionWithQuery(qex, prepareResult);
+        return MariaDbSqlException.of(qex, prepareResult.getSql());
       }
 
       @Override
       public int getParamCount() {
-        return getPrepareResult() == null
-            ? parametersList.get(0).length
-            : ((ServerPrepareResult) getPrepareResult()).getParameters().length;
+        return serverPrepareResult.getParameters().length;
       }
 
       @Override
@@ -1036,12 +1108,12 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
       // send binary data in a separate stream
       for (int i = 0; i < parameterCount; i++) {
-        if (parameters[i].isLongData()) {
+        if (parameters[i].canBeLongData()) {
           writer.startPacket(0);
           writer.write(COM_STMT_SEND_LONG_DATA);
           writer.writeInt(serverPrepareResult.getStatementId());
           writer.writeShort((short) i);
-          parameters[i].writeBinary(writer);
+          parameters[i].writeLongData(writer);
           writer.flush();
         }
       }
@@ -1057,9 +1129,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       getResult(results);
 
     } catch (SQLException qex) {
-      throw logQuery.exceptionWithQuery(parameters, qex, serverPrepareResult);
+      throw exceptionWithQuery(parameters, serverPrepareResult, qex, false);
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery(parameters, serverPrepareResult, handleIoException(e), false);
     }
   }
 
@@ -1106,10 +1178,8 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
           return true;
         } catch (IOException e) {
           connected = false;
-          throw new SQLException(
-              "Could not deallocate query: " + e.getMessage(),
-              CONNECTION_EXCEPTION.getSqlState(),
-              e);
+          throw new SQLNonTransientConnectionException(
+              "Could not deallocate query: " + e.getMessage(), "08000", e);
         }
 
       } finally {
@@ -1152,8 +1222,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
     } catch (IOException e) {
       connected = false;
-      throw new SQLException(
-          "Could not ping: " + e.getMessage(), CONNECTION_EXCEPTION.getSqlState(), e);
+      throw new SQLNonTransientConnectionException("Could not ping: " + e.getMessage(), "08000", e);
     } finally {
       lock.unlock();
     }
@@ -1252,13 +1321,13 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
         throw new SQLException(
             "Could not select database '" + database + "' : " + ep.getMessage(),
             ep.getSqlState(),
-            ep.getErrorNumber());
+            ep.getErrorCode());
       }
 
       this.database = database;
 
     } catch (IOException e) {
-      throw handleIoException(e);
+      throw exceptionWithQuery("COM_INIT_DB", handleIoException(e), false);
     } finally {
       lock.unlock();
     }
@@ -1280,7 +1349,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
   @Override
   public void cancelCurrentQuery() throws SQLException {
     try (MasterProtocol copiedProtocol =
-        new MasterProtocol(urlParser, new GlobalStateInfo(), new ReentrantLock())) {
+        new MasterProtocol(urlParser, new GlobalStateInfo(), new ReentrantLock(), traceCache)) {
       copiedProtocol.setHostAddress(getHostAddress());
       copiedProtocol.connect();
       // no lock, because there is already a query running that possessed the lock.
@@ -1374,6 +1443,19 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     }
   }
 
+  /**
+   * Puts this connection in read-only / read-write mode
+   *
+   * @param readOnly true enables read-only mode; false disables it
+   * @throws SQLException If socket error.
+   */
+  public void setReadonly(final boolean readOnly) throws SQLException {
+    if (options.assureReadOnly && this.readOnly != readOnly && versionGreaterOrEqual(5, 6, 5)) {
+      executeQuery("SET SESSION TRANSACTION " + (readOnly ? "READ ONLY" : "READ WRITE"));
+    }
+    this.readOnly = readOnly;
+  }
+
   /**
    * Set transaction isolation.
    *
@@ -1721,9 +1803,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     try {
 
       // read columns information's
-      ColumnInformation[] ci = new ColumnInformation[(int) fieldCount];
+      ColumnDefinition[] ci = new ColumnDefinition[(int) fieldCount];
       for (int i = 0; i < fieldCount; i++) {
-        ci[i] = new ColumnInformation(reader.getPacket(false));
+        ci[i] = new ColumnDefinition(reader.getPacket(false));
       }
 
       boolean callableResult = false;
@@ -1791,28 +1873,28 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       long maxRows, boolean hasProxy, MariaDbConnection connection, MariaDbStatement statement)
       throws SQLException {
     if (explicitClosed) {
-      throw new SQLNonTransientConnectionException("execute() is called on closed connection");
+      throw new SQLNonTransientConnectionException(
+          "execute() is called on closed connection", "08000");
     }
     // old failover handling
     if (!hasProxy && shouldReconnectWithoutProxy()) {
       try {
         connectWithoutProxy();
       } catch (SQLException qe) {
-        ExceptionMapper.throwException(qe, connection, statement);
+        throw ExceptionFactory.of((int) serverThreadId, options).create(qe);
       }
     }
 
     try {
       setMaxRows(maxRows);
     } catch (SQLException qe) {
-      ExceptionMapper.throwException(qe, connection, statement);
+      throw ExceptionFactory.of((int) serverThreadId, options).create(qe);
     }
 
     connection.reenableWarnings();
   }
 
-  public ServerPrepareResult addPrepareInCache(
-      String key, ServerPrepareResult serverPrepareResult) {
+  public ServerPrepareResult putInCache(String key, ServerPrepareResult serverPrepareResult) {
     return serverPrepareStatementCache.put(key, serverPrepareResult);
   }
 
@@ -1847,9 +1929,41 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     }
 
     if (!this.connected) {
-      throw new SQLException("Connection is closed", "08000", 1220);
+      throw exceptionFactory.create("Connection is closed", "08000", 1220);
     }
     interrupted = false;
+    if (this.options.ensureSocketState) {
+      // ensure that the socket buffer is empty before issuing new command.
+      // (this doesn't concern pipelining commands).
+      // If data is present in socket, an error will be raised, throwing the content of socket data
+      // to permit
+      // identification of error.
+      try {
+        int avail = this.reader.getInputStream().available();
+        if (avail > 0) {
+          // unexpected data in socket buffer
+
+          // reading socket buffer to add content to error.
+          byte[] data = new byte[Math.min(avail, 16000)];
+          int remaining = avail;
+          int off = 0;
+          do {
+            int count = this.reader.getInputStream().read(data, off, remaining);
+            if (count < 0) {
+              break;
+            }
+            remaining -= count;
+            off += count;
+          } while (remaining > 0);
+
+          throw exceptionFactory.create(
+              "Unexpected data in socket buffer:" + Utils.hexdump(data), "HY000");
+        }
+
+      } catch (IOException ioe) {
+        throw exceptionFactory.create("Unexpected socket error", "08000", ioe);
+      }
+    }
   }
 
   /**
@@ -1911,6 +2025,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
             UNDEFINED_SQLSTATE.getSqlState(),
             initialException);
       }
+    } else if (initialException instanceof NotSerializableException) {
+      return new SerialException(
+          "Parameter cannot be serialized: " + initialException.getMessage());
     } else {
       maxSizeError = writer.exceedMaxLength();
       if (maxSizeError) {
@@ -1919,9 +2036,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     }
 
     if (mustReconnect && !explicitClosed) {
+      String traces = getTraces();
       try {
         connect();
-
         try {
           resetStateAfterFailover(
               getMaxRows(), getTransactionIsolationLevel(), getDatabase(), getAutocommit());
@@ -1931,36 +2048,32 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
                 "Could not send query: query size is >= to max_allowed_packet ("
                     + writer.getMaxAllowedPacket()
                     + ")"
-                    + getTraces(),
-                UNDEFINED_SQLSTATE.getSqlState(),
+                    + traces,
+                "HY000",
                 initialException);
           }
 
-          return new SQLNonTransientConnectionException(
-              initialException.getMessage() + getTraces(),
-              UNDEFINED_SQLSTATE.getSqlState(),
-              initialException);
+          return new SQLTransientConnectionException(
+              initialException.getMessage() + traces, "HY000", initialException);
 
         } catch (SQLException queryException) {
-          return new SQLNonTransientConnectionException(
-              "reconnection succeed, but resetting previous state failed",
-              UNDEFINED_SQLSTATE.getSqlState() + getTraces(),
+          return new SQLTransientConnectionException(
+              "reconnection succeed, but resetting previous state failed" + traces,
+              "HY000",
               initialException);
         }
 
       } catch (SQLException queryException) {
         connected = false;
         return new SQLNonTransientConnectionException(
-            initialException.getMessage() + "\nError during reconnection" + getTraces(),
-            CONNECTION_EXCEPTION.getSqlState(),
-            initialException);
+            initialException.getMessage() + "\nError during reconnection" + traces,
+            "08000",
+            queryException);
       }
     }
     connected = false;
     return new SQLNonTransientConnectionException(
-        initialException.getMessage() + getTraces(),
-        CONNECTION_EXCEPTION.getSqlState(),
-        initialException);
+        initialException.getMessage() + getTraces(), "08000", initialException);
   }
 
   public void setActiveFutureTask(FutureTask activeFutureTask) {
