@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -22,12 +22,11 @@
 
 package org.mariadb.jdbc.util;
 
-import org.mariadb.jdbc.credential.*;
-import org.mariadb.jdbc.internal.util.*;
-import org.mariadb.jdbc.internal.util.constant.*;
-
-import java.lang.reflect.*;
-import java.util.*;
+import java.lang.reflect.Field;
+import java.util.Properties;
+import org.mariadb.jdbc.credential.CredentialPlugin;
+import org.mariadb.jdbc.internal.util.OptionUtils;
+import org.mariadb.jdbc.internal.util.constant.HaMode;
 
 public enum DefaultOptions {
   USER("user", "1.0.0", "Database user name", false),
@@ -68,12 +67,12 @@ public enum DefaultOptions {
       "tcpAbortiveClose",
       Boolean.FALSE,
       "1.1.1",
-      "Sets corresponding option on the connection " + "socket.",
+      "Sets corresponding option on the connection socket.",
       false),
   LOCAL_SOCKET_ADDRESS(
       "localSocketAddress",
       "1.1.8",
-      "Hostname or IP address to bind the connection socket to a " + "local (UNIX domain) socket.",
+      "Hostname or IP address to bind the connection socket to a local (UNIX domain) socket.",
       false),
   SOCKET_TIMEOUT(
       "socketTimeout",
@@ -121,7 +120,7 @@ public enum DefaultOptions {
       "createDatabaseIfNotExist",
       Boolean.FALSE,
       "1.1.8",
-      "the specified database in the " + "url will be created if non-existent.",
+      "the specified database in the url will be created if non-existent.",
       false),
   SERVER_TIMEZONE(
       "serverTimezone",
@@ -134,7 +133,7 @@ public enum DefaultOptions {
       "nullCatalogMeansCurrent",
       Boolean.TRUE,
       "1.1.8",
-      "DatabaseMetaData use current catalog" + " if null.",
+      "DatabaseMetaData use current catalog if null.",
       false),
   TINY_INT_IS_BIT(
       "tinyInt1isBit",
@@ -213,7 +212,7 @@ public enum DefaultOptions {
       "trustServerCertificate",
       Boolean.FALSE,
       "1.1.1",
-      "When using SSL, do not check server's" + " certificate.",
+      "When using SSL, do not check server's certificate.",
       false),
   SERVER_SSL_CERT(
       "serverSslCert",
@@ -279,7 +278,7 @@ public enum DefaultOptions {
       50,
       0,
       "1.2.0",
-      "time in second a server is" + " blacklisted after a connection failure.",
+      "time in second a server is blacklisted after a connection failure.",
       false),
   CACHE_PREP_STMTS(
       "cachePrepStmts",
@@ -307,8 +306,8 @@ public enum DefaultOptions {
       "assureReadOnly",
       Boolean.FALSE,
       "1.3.0",
-      "If true, in high availability, and switching to a "
-          + "read-only host, assure that this host is in read-only mode by setting the session to read-only.",
+      "Ensure that when Connection.setReadOnly(true) is called, host is in read-only mode by "
+          + "setting the session transaction to read-only.",
       false),
   USE_LEGACY_DATETIME_CODE(
       "useLegacyDatetimeCode",
@@ -389,7 +388,7 @@ public enum DefaultOptions {
       "continueBatchOnError",
       Boolean.TRUE,
       "1.4.0",
-      "When executing batch queries, must batch " + "continue on error.",
+      "When executing batch queries, must batch continue on error.",
       false),
   JDBC_COMPLIANT_TRUNCATION(
       "jdbcCompliantTruncation",
@@ -400,9 +399,9 @@ public enum DefaultOptions {
       false),
   CACHE_CALLABLE_STMTS(
       "cacheCallableStmts",
-      Boolean.TRUE,
+      Boolean.FALSE,
       "1.4.0",
-      "enable/disable callable Statement cache, default" + " true.",
+      "enable/disable callable Statement cache, default false.",
       false),
   CALLABLE_STMT_CACHE_SIZE(
       "callableStmtCacheSize",
@@ -459,12 +458,12 @@ public enum DefaultOptions {
       null,
       0L,
       "1.5.0",
-      "Will log query with execution time superior" + " to this value (if defined )",
+      "Will log query with execution time superior to this value (if defined )",
       false),
   PASSWORD_CHARACTER_ENCODING(
       "passwordCharacterEncoding",
       "1.5.9",
-      "Indicate password encoding charset. If not set," + " driver use platform's default charset.",
+      "Indicate password encoding charset. If not set, driver use platform's default charset.",
       false),
   PIPELINE_AUTH(
       "usePipelineAuth",
@@ -526,7 +525,7 @@ public enum DefaultOptions {
       8,
       1,
       "2.2.0",
-      "The maximum number of physical connections that the pool should " + "contain.",
+      "The maximum number of physical connections that the pool should contain.",
       false),
   MIN_POOL_SIZE(
       "minPoolSize",
@@ -582,8 +581,8 @@ public enum DefaultOptions {
       "allowMasterDownConnection",
       Boolean.FALSE,
       "2.2.0",
-      "When using master/slave configuration, "
-          + "permit to create connection when master is down. If no master is up, default connection is then a slave "
+      "When using master/replica configuration, "
+          + "permit to create connection when master is down. If no master is up, default connection is then a replica "
           + "and Connection.isReadOnly() will then return true.",
       false),
   GALERA_ALLOWED_STATE(
@@ -680,8 +679,19 @@ public enum DefaultOptions {
       "Permit to get MySQL server key retrieval",
       false),
   TLS_SOCKET_TYPE(
-      "tlsSocketType", (String) null, "2.5.0", "Indicate TLS socket type implementation", false);
-
+      "tlsSocketType", (String) null, "2.5.0", "Indicate TLS socket type implementation", false),
+  TRACK_SCHEMA(
+      "trackSchema",
+      Boolean.TRUE,
+      "2.6.0",
+      "manage session_track_schema setting when server has CLIENT_SESSION_TRACK capability",
+      false),
+  ENSURE_SOCKET_STATE(
+      "ensureSocketState",
+      Boolean.FALSE,
+      "2.7.0",
+      "ensure socket state before issuing a new command",
+      false);
   private final String optionName;
   private final String description;
   private final boolean required;
