@@ -3,7 +3,7 @@
  * MariaDB Client for Java
  *
  * Copyright (c) 2012-2014 Monty Program Ab.
- * Copyright (c) 2015-2019 MariaDB Ab.
+ * Copyright (c) 2015-2020 MariaDB Corporation Ab.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
@@ -22,9 +22,10 @@
 
 package org.mariadb.jdbc.util;
 
-import java.lang.reflect.*;
-import java.sql.*;
-import java.util.*;
+import java.lang.reflect.Field;
+import java.sql.DriverManager;
+import java.util.Objects;
+import java.util.Properties;
 
 @SuppressWarnings("ConstantConditions")
 public class Options implements Cloneable {
@@ -62,6 +63,7 @@ public class Options implements Cloneable {
   public String localSocketAddress;
   public Integer socketTimeout;
   public boolean allowMultiQueries;
+  public boolean trackSchema = true;
   public boolean rewriteBatchedStatements;
   public boolean useCompression;
   public boolean interactiveClient;
@@ -89,7 +91,7 @@ public class Options implements Cloneable {
   public boolean useServerPrepStmts;
   public boolean continueBatchOnError = true;
   public boolean jdbcCompliantTruncation = true;
-  public boolean cacheCallableStmts = true;
+  public boolean cacheCallableStmts;
   public int callableStmtCacheSize = 150;
   public String connectionAttributes;
   public Boolean useBatchMultiSend;
@@ -138,6 +140,7 @@ public class Options implements Cloneable {
   // MySQL sha authentication
   public String serverRsaPublicKeyFile;
   public boolean allowPublicKeyRetrieval;
+  public boolean ensureSocketState;
 
   @Override
   public String toString() {
