@@ -65,14 +65,22 @@ import org.mariadb.jdbc.internal.util.scheduler.SchedulerServiceProviderHolder;
 
 public class ConnectionTest extends BaseTest {
 
-  /**
-   * Initialisation.
-   *
-   * @throws SQLException exception
-   */
   @BeforeClass()
   public static void initClass() throws SQLException {
-    createTable("dummy", "a BLOB");
+    afterClass();
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute("CREATE TABLE dummy(a BLOB)");
+      stmt.execute("CREATE DATABASE gogogo");
+      stmt.execute("FLUSH TABLES");
+    }
+  }
+
+  @AfterClass
+  public static void afterClass() throws SQLException {
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute("DROP TABLE IF EXISTS dummy");
+      stmt.execute("DROP DATABASE IF EXISTS gogogo");
+    }
   }
 
   /** Conj-166. Connection error code must be thrown */
@@ -311,7 +319,11 @@ public class ConnectionTest extends BaseTest {
    */
   @Test
   public void isValidConnectionThatTimesOutByServer() throws SQLException, InterruptedException {
-    Assume.assumeFalse(sharedIsAurora());
+    Assume.assumeTrue(
+        !sharedIsAurora()
+            && System.getenv("SKYSQL") == null
+            && System.getenv("SKYSQL_HA") == null
+            && System.getenv("MAXSCALE_TEST_DISABLE") == null);
     try (Connection connection = setConnection()) {
       try (Statement statement = connection.createStatement()) {
         statement.execute("set session wait_timeout=1");
@@ -429,7 +441,6 @@ public class ConnectionTest extends BaseTest {
     final String db = sharedConnection.getCatalog();
     Statement stmt = sharedConnection.createStatement();
 
-    stmt.execute("CREATE DATABASE gogogo");
     stmt.execute("USE gogogo");
     String db2 = sharedConnection.getCatalog();
     assertEquals("gogogo", db2);
@@ -582,7 +593,10 @@ public class ConnectionTest extends BaseTest {
 
   @Test
   public void verificationEd25519AuthPlugin() throws Throwable {
-    Assume.assumeTrue(System.getenv("MAXSCALE_VERSION") == null && System.getenv("SKYSQL") == null);
+    Assume.assumeTrue(
+        System.getenv("MAXSCALE_TEST_DISABLE") == null
+            && System.getenv("SKYSQL") == null
+            && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(isMariadbServer() && minVersion(10, 2));
     Statement stmt = sharedConnection.createStatement();
 
@@ -669,15 +683,14 @@ public class ConnectionTest extends BaseTest {
       // excepted exception
       // since retriesAllDown is = 20 , that means 10 entire loop with 250ms sleep
       // first loop has not sleep, last too, so 8 * 250 = 2s
-      System.out.println(System.currentTimeMillis() - start);
       assertTrue(System.currentTimeMillis() - start > min);
       assertTrue(System.currentTimeMillis() - start < max);
     }
   }
 
   @Test
-  public void slaveDownConnection() throws SQLException {
-    Assume.assumeTrue(System.getenv("SKYSQL") == null);
+  public void replicaDownConnection() throws SQLException {
+    Assume.assumeTrue(System.getenv("SKYSQL") == null && System.getenv("SKYSQL_HA") == null);
     String url =
         "jdbc:mariadb:replication://"
             + hostname
@@ -708,7 +721,10 @@ public class ConnectionTest extends BaseTest {
 
   @Test
   public void multiAuthPlugin() throws Throwable {
-    Assume.assumeTrue(System.getenv("MAXSCALE_VERSION") == null && System.getenv("SKYSQL") == null);
+    Assume.assumeTrue(
+        System.getenv("MAXSCALE_TEST_DISABLE") == null
+            && System.getenv("SKYSQL") == null
+            && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(isMariadbServer() && minVersion(10, 4, 2));
     Statement stmt = sharedConnection.createStatement();
     try {
@@ -766,7 +782,7 @@ public class ConnectionTest extends BaseTest {
 
   @Test
   public void connectionUnexpectedClose() throws SQLException {
-    Assume.assumeTrue(System.getenv("MAXSCALE_VERSION") == null && System.getenv("SKYSQL") == null);
+    Assume.assumeTrue(System.getenv("SKYSQL") == null && System.getenv("SKYSQL_HA") == null);
     try (Connection connection =
         DriverManager.getConnection(
             "jdbc:mariadb:failover//"
@@ -821,7 +837,7 @@ public class ConnectionTest extends BaseTest {
 
   @Test
   public void setReadonlyError() throws SQLException {
-    Assume.assumeTrue(System.getenv("MAXSCALE_VERSION") == null && System.getenv("SKYSQL") == null);
+    Assume.assumeTrue(System.getenv("SKYSQL") == null && System.getenv("SKYSQL_HA") == null);
     try (Connection connection =
         DriverManager.getConnection(
             "jdbc:mariadb:replication://"
@@ -917,7 +933,10 @@ public class ConnectionTest extends BaseTest {
 
   @Test
   public void setClientNotConnectError() throws SQLException {
-    Assume.assumeTrue(System.getenv("MAXSCALE_VERSION") == null && System.getenv("SKYSQL") == null);
+    Assume.assumeTrue(
+        System.getenv("MAXSCALE_TEST_DISABLE") == null
+            && System.getenv("SKYSQL") == null
+            && System.getenv("SKYSQL_HA") == null);
     // only mariadb return a specific error when connection has explicitly been killed
     Assume.assumeTrue(isMariadbServer());
 
@@ -986,6 +1005,7 @@ public class ConnectionTest extends BaseTest {
   public void readOnly() throws SQLException {
     Statement stmt = sharedConnection.createStatement();
     stmt.execute("CREATE TABLE testReadOnly(id int)");
+    stmt.execute("FLUSH TABLES");
     sharedConnection.setAutoCommit(false);
 
     sharedConnection.setReadOnly(true);
