@@ -22,10 +22,7 @@
 package org.mariadb.jdbc;
 
 import java.sql.*;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.*;
 
 public class CredentialPluginTest extends BaseTest {
 
@@ -35,7 +32,7 @@ public class CredentialPluginTest extends BaseTest {
    * @throws SQLException if any
    */
   @Before
-  public void before() throws SQLException {
+  public void beforeTest() throws SQLException {
     boolean useOldNotation = true;
     if ((isMariadbServer() && minVersion(10, 2, 0))
         || (!isMariadbServer() && minVersion(8, 0, 0))) {
@@ -43,15 +40,23 @@ public class CredentialPluginTest extends BaseTest {
     }
     Statement stmt = sharedConnection.createStatement();
     if (useOldNotation) {
+      stmt.execute("CREATE USER 'identityUser'@'localhost'");
+      stmt.execute(
+          "GRANT SELECT ON "
+              + database
+              + ".* TO 'identityUser'@'localhost' IDENTIFIED BY '!Passw0rd3Works'");
       stmt.execute("CREATE USER 'identityUser'@'%'");
       stmt.execute(
           "GRANT SELECT ON "
               + database
               + ".* TO 'identityUser'@'%' IDENTIFIED BY '!Passw0rd3Works'");
     } else {
+      stmt.execute("CREATE USER 'identityUser'@'localhost' IDENTIFIED BY '!Passw0rd3Works'");
+      stmt.execute("GRANT SELECT ON " + database + ".* TO 'identityUser'@'localhost'");
       stmt.execute("CREATE USER 'identityUser'@'%' IDENTIFIED BY '!Passw0rd3Works'");
       stmt.execute("GRANT SELECT ON " + database + ".* TO 'identityUser'@'%'");
     }
+    stmt.execute("FLUSH PRIVILEGES");
   }
 
   /**
@@ -60,9 +65,10 @@ public class CredentialPluginTest extends BaseTest {
    * @throws SQLException if any
    */
   @After
-  public void after() throws SQLException {
+  public void afterTest() throws SQLException {
     Statement stmt = sharedConnection.createStatement();
     stmt.execute("DROP USER 'identityUser'@'%'");
+    stmt.execute("DROP USER 'identityUser'@'localhost'");
   }
 
   @Test
@@ -93,6 +99,7 @@ public class CredentialPluginTest extends BaseTest {
 
   @Test
   public void specificPropertiesIdentityTest() throws SQLException {
+
     System.setProperty("myUserKey", "identityUser");
     System.setProperty("myPwdKey", "!Passw0rd3Works");
 
@@ -105,7 +112,11 @@ public class CredentialPluginTest extends BaseTest {
                 + "/"
                 + ((database == null) ? "" : database)
                 + "?credentialType=PROPERTY"
-                + "&userKey=myUserKey&pwdKey=myPwdKey")) {
+                + "&userKey=myUserKey&pwdKey=myPwdKey"
+                + ((options.useSsl != null) ? "&useSsl=" + options.useSsl : "")
+                + ((options.serverSslCert != null)
+                    ? "&serverSslCert=" + options.serverSslCert
+                    : ""))) {
       Statement stmt = conn.createStatement();
 
       ResultSet rs = stmt.executeQuery("SELECT '5'");
