@@ -55,64 +55,160 @@ package org.mariadb.jdbc;
 import static org.junit.Assert.*;
 
 import java.sql.*;
-import org.junit.Assume;
-import org.junit.Before;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.*;
 
 public class DatabaseMetadataTest extends BaseTest {
 
-  /**
-   * Initialisation.
-   *
-   * @throws SQLException exception
-   */
   @BeforeClass()
   public static void initClass() throws SQLException {
-    createTable(
-        "dbpk_test",
-        "val varchar(20), id1 int not null, id2 int not null,primary key(id1, id2)",
-        "engine=innodb");
-    createTable("datetime_test", "dt datetime");
-    createTable(
-        "`manycols`",
-        "  `tiny` tinyint(4) DEFAULT NULL,"
-            + "  `tiny_uns` tinyint(3) unsigned DEFAULT NULL,"
-            + "  `small` smallint(6) DEFAULT NULL,"
-            + "  `small_uns` smallint(5) unsigned DEFAULT NULL,"
-            + "  `medium` mediumint(9) DEFAULT NULL,"
-            + "  `medium_uns` mediumint(8) unsigned DEFAULT NULL,"
-            + "  `int_col` int(11) DEFAULT NULL,"
-            + "  `int_col_uns` int(10) unsigned DEFAULT NULL,"
-            + "  `big` bigint(20) DEFAULT NULL,"
-            + "  `big_uns` bigint(20) unsigned DEFAULT NULL,"
-            + "  `decimal_col` decimal(10,5) DEFAULT NULL,"
-            + "  `fcol` float DEFAULT NULL,"
-            + "  `fcol_uns` float unsigned DEFAULT NULL,"
-            + "  `dcol` double DEFAULT NULL,"
-            + "  `dcol_uns` double unsigned DEFAULT NULL,"
-            + "  `date_col` date DEFAULT NULL,"
-            + "  `time_col` time DEFAULT NULL,"
-            + "  `timestamp_col` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,"
-            + "  `year_col` year(4) DEFAULT NULL,"
-            + "  `bit_col` bit(5) DEFAULT NULL,"
-            + "  `char_col` char(5) DEFAULT NULL,"
-            + "  `varchar_col` varchar(10) DEFAULT NULL,"
-            + "  `binary_col` binary(10) DEFAULT NULL,"
-            + "  `varbinary_col` varbinary(10) DEFAULT NULL,"
-            + "  `tinyblob_col` tinyblob,"
-            + "  `blob_col` blob,"
-            + "  `mediumblob_col` mediumblob,"
-            + "  `longblob_col` longblob,"
-            + "  `tinytext_col` tinytext,"
-            + "  `text_col` text,"
-            + "  `mediumtext_col` mediumtext,"
-            + "  `longtext_col` longtext");
-    createTable("ytab", "y year");
-    createTable("maxcharlength", "maxcharlength char(1)", "character set utf8");
-    createTable("conj72", "t tinyint(1)");
-    if (isMariadbServer() && minVersion(10, 3, 4)) {
-      createTable("versionTable", "x INT", "WITH SYSTEM VERSIONING");
+    afterClass();
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute(
+          "CREATE TABLE dbpk_test(val varchar(20), id1 int not null, id2 int not null,primary key(id1, id2)) engine=innodb");
+      stmt.execute("CREATE TABLE datetime_test(dt datetime)");
+      stmt.execute(
+          "CREATE TABLE `manycols`("
+              + "  `tiny` tinyint(4) DEFAULT NULL,"
+              + "  `tiny_uns` tinyint(3) unsigned DEFAULT NULL,"
+              + "  `small` smallint(6) DEFAULT NULL,"
+              + "  `small_uns` smallint(5) unsigned DEFAULT NULL,"
+              + "  `medium` mediumint(9) DEFAULT NULL,"
+              + "  `medium_uns` mediumint(8) unsigned DEFAULT NULL,"
+              + "  `int_col` int(11) DEFAULT NULL,"
+              + "  `int_col_uns` int(10) unsigned DEFAULT NULL,"
+              + "  `big` bigint(20) DEFAULT NULL,"
+              + "  `big_uns` bigint(20) unsigned DEFAULT NULL,"
+              + "  `decimal_col` decimal(10,5) DEFAULT NULL,"
+              + "  `fcol` float DEFAULT NULL,"
+              + "  `fcol_uns` float unsigned DEFAULT NULL,"
+              + "  `dcol` double DEFAULT NULL,"
+              + "  `dcol_uns` double unsigned DEFAULT NULL,"
+              + "  `date_col` date DEFAULT NULL,"
+              + "  `time_col` time DEFAULT NULL,"
+              + "  `timestamp_col` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,"
+              + "  `year_col` year(4) DEFAULT NULL,"
+              + "  `bit_col` bit(5) DEFAULT NULL,"
+              + "  `char_col` char(5) DEFAULT NULL,"
+              + "  `varchar_col` varchar(10) DEFAULT NULL,"
+              + "  `binary_col` binary(10) DEFAULT NULL,"
+              + "  `varbinary_col` varbinary(10) DEFAULT NULL,"
+              + "  `tinyblob_col` tinyblob,"
+              + "  `blob_col` blob,"
+              + "  `mediumblob_col` mediumblob,"
+              + "  `longblob_col` longblob,"
+              + "  `tinytext_col` tinytext,"
+              + "  `text_col` text,"
+              + "  `mediumtext_col` mediumtext,"
+              + "  `longtext_col` longtext)");
+      stmt.execute("CREATE TABLE ytab(y year)");
+      stmt.execute("CREATE TABLE maxcharlength(maxcharlength char(1)) character set utf8");
+      stmt.execute("CREATE TABLE conj72(t tinyint(1))");
+      if (isMariadbServer() && minVersion(10, 3, 4)) {
+        stmt.execute("CREATE TABLE versionTable(x INT) WITH SYSTEM VERSIONING");
+      }
+      stmt.execute("create table cross1 (id int not null primary key, val varchar(20))");
+      stmt.execute(
+          "create table cross2 (id int not null, id2 int not null,  "
+              + "id_ref0 int, foreign key (id_ref0) references cross1(id), UNIQUE unik_name (id, id2))");
+      stmt.execute(
+          "create table cross3 (id int not null primary key, "
+              + "id_ref1 int, id_ref2 int, foreign key fk_my_name (id_ref1, id_ref2) references cross2(id, id2) on "
+              + "update cascade)");
+      stmt.execute(
+          "create table getBestRowIdentifier1(i int not null primary key auto_increment, id int, "
+              + "id_ref1 int, id_ref2 int, foreign key fk_my_name_1 (id_ref1, id_ref2) references cross2(id, id2) on "
+              + "update cascade, UNIQUE getBestRowIdentifier_unik (id))");
+      stmt.execute(
+          "create table getBestRowIdentifier2(id_ref0 int not null, "
+              + "id_ref1 int, id_ref2 int not null, UNIQUE (id_ref1, id_ref2) , UNIQUE (id_ref0, id_ref2))");
+      stmt.execute(
+          "CREATE TABLE getPrecision("
+              + "num1 NUMERIC(9,4), "
+              + "num2 NUMERIC (9,0),"
+              + "num3 NUMERIC (9,4) UNSIGNED,"
+              + "num4 NUMERIC (9,0) UNSIGNED,"
+              + "num5 FLOAT(9,4),"
+              + "num6 FLOAT(9,4) UNSIGNED,"
+              + "num7 DOUBLE(9,4),"
+              + "num8 DOUBLE(9,4) UNSIGNED)");
+      stmt.execute(
+          "CREATE TABLE getTimePrecision("
+              + "d date, "
+              + "t1 datetime(0),"
+              + "t2 datetime(6),"
+              + "t3 timestamp(0) DEFAULT '2000-01-01 00:00:00',"
+              + "t4 timestamp(6) DEFAULT '2000-01-01 00:00:00',"
+              + "t5 time(0),"
+              + "t6 time(6))");
+      stmt.execute(
+          "CREATE TABLE getStringPrecision("
+              + "col1 CHAR(100) CHARSET utf8mb4, "
+              + "col2 CHAR(100) CHARSET utf8,"
+              + "col3 CHAR(100) CHARSET latin2)");
+      stmt.execute(
+          "CREATE TABLE getTimePrecision2("
+              + "d date, "
+              + "t1 datetime(0),"
+              + "t2 datetime(6),"
+              + "t3 timestamp(0) DEFAULT '2000-01-01 00:00:00',"
+              + "t4 timestamp(6) DEFAULT '2000-01-01 00:00:00',"
+              + "t5 time(0),"
+              + "t6 time(6))");
+
+      stmt.execute(
+          "CREATE PROCEDURE getProcTimePrecision2(IN  I date, "
+              + "IN t1 DATETIME,"
+              + "IN t3 timestamp,"
+              + "IN t5 time) BEGIN SELECT I; END");
+      stmt.execute(
+          "CREATE PROCEDURE getProcTimePrecision(IN  I date, "
+              + "IN t1 DATETIME(0),"
+              + "IN t2 DATETIME(6),"
+              + "IN t3 timestamp(0),"
+              + "IN t4 timestamp(6),"
+              + "IN t5 time ,"
+              + "IN t6 time(6)) BEGIN SELECT I; END");
+      stmt.execute(
+          "create table prim_key (id int not null primary key, val varchar(20)) engine=innodb");
+      stmt.execute(
+          "create table fore_key0 (id int not null primary key, "
+              + "id_ref0 int, foreign key (id_ref0) references prim_key(id)) engine=innodb");
+      stmt.execute(
+          "create table fore_key1 (id int not null primary key, "
+              + "id_ref1 int, foreign key (id_ref1) references prim_key(id) on update cascade) engine=innodb");
+      stmt.execute(
+          "create table table_type_test (id int not null primary key, "
+              + "val varchar(20)) engine=innodb");
+      stmt.execute("FLUSH TABLES");
+    }
+  }
+
+  @AfterClass
+  public static void afterClass() throws SQLException {
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute("DROP TABLE IF EXISTS dbpk_test");
+      stmt.execute("DROP TABLE IF EXISTS datetime_test");
+      stmt.execute("DROP TABLE IF EXISTS `manycols`");
+      stmt.execute("DROP TABLE IF EXISTS ytab");
+      stmt.execute("DROP TABLE IF EXISTS maxcharlength");
+      stmt.execute("DROP TABLE IF EXISTS conj72");
+      stmt.execute("DROP TABLE IF EXISTS versionTable");
+      stmt.execute("drop table if exists getBestRowIdentifier1");
+      stmt.execute("drop table if exists getBestRowIdentifier2");
+      stmt.execute("drop table if exists cross3");
+      stmt.execute("drop table if exists cross2");
+      stmt.execute("drop table if exists cross1");
+      stmt.execute("drop table if exists getPrecision");
+      stmt.execute("drop table if exists tablegetcolumns");
+      stmt.execute("drop table if exists getTimePrecision");
+      stmt.execute("drop table if exists getTimePrecision2");
+      stmt.execute("drop procedure if exists getProcTimePrecision");
+      stmt.execute("drop procedure if exists getProcTimePrecision2");
+      stmt.execute("drop table if exists fore_key0");
+      stmt.execute("drop table if exists fore_key1");
+      stmt.execute("drop table if exists prim_key");
+      stmt.execute("drop table if exists table_type_test");
+      stmt.execute("drop table if exists getStringPrecision");
     }
   }
 
@@ -154,7 +250,7 @@ public class DatabaseMetadataTest extends BaseTest {
         "CREATE TABLE t2 (id2a integer, id2b integer, constraint pk primary key(id2a, id2b), "
             + "constraint fk1 foreign key(id2a) references t1(id1),  constraint fk2 foreign key(id2b) "
             + "references t1(id1))");
-
+    stmt.execute("FLUSH TABLES");
     DatabaseMetaData dbmd = sharedConnection.getMetaData();
     ResultSet rs = dbmd.getPrimaryKeys("testj", null, "t2");
     int counter = 0;
@@ -235,7 +331,7 @@ public class DatabaseMetadataTest extends BaseTest {
 
     st.execute(
         "CREATE TABLE t1.product ( category INT NOT NULL, id INT NOT NULL, price DECIMAL,"
-            + " PRIMARY KEY(category, id) )   ENGINE=INNODB");
+            + " UNIQUE unik_name (category, id) )   ENGINE=INNODB");
 
     st.execute("CREATE TABLE `cus``tomer` (id INT NOT NULL, PRIMARY KEY (id))   ENGINE=INNODB");
 
@@ -254,33 +350,71 @@ public class DatabaseMetadataTest extends BaseTest {
             + "    FOREIGN KEY (customer_id)\n"
             + "      REFERENCES `cus``tomer`(id)\n"
             + ")   ENGINE=INNODB;");
-
+    st.execute("FLUSH TABLES");
     /*
     Test that I_S implementation is equivalent to parsing "show create table" .
      Get result sets using either method and compare (ignore minor differences INT vs SMALLINT
     */
     ResultSet rs1 =
         ((MariaDbDatabaseMetaData) sharedConnection.getMetaData())
-            .getImportedKeysUsingShowCreateTable("testj", "product_order");
+            .getImportedKeysUsingShowCreateTable(database, "product_order");
     ResultSet rs2 =
         ((MariaDbDatabaseMetaData) sharedConnection.getMetaData())
-            .getImportedKeysUsingInformationSchema("testj", "product_order");
+            .getImportedKeysUsingInformationSchema(database, "product_order");
     assertEquals(rs1.getMetaData().getColumnCount(), rs2.getMetaData().getColumnCount());
+    for (int i = 0; i < 2; i++) {
+      ResultSet rs = i == 0 ? rs1 : rs2;
+      assertTrue(rs.next());
 
-    while (rs1.next()) {
-      assertTrue(rs2.next());
-      for (int i = 1; i <= rs1.getMetaData().getColumnCount(); i++) {
-        Object s1 = rs1.getObject(i);
-        Object s2 = rs2.getObject(i);
-        if (s1 instanceof Number && s2 instanceof Number) {
-          assertEquals(((Number) s1).intValue(), ((Number) s2).intValue());
-        } else {
-          if (s1 != null && s2 != null && !s1.equals(s2)) {
-            fail();
-          }
-          assertEquals(s1, s2);
-        }
-      }
+      assertEquals("t1", rs.getString("PKTABLE_CAT"));
+      assertEquals(null, rs.getString("PKTABLE_SCHEM"));
+      assertEquals("product", rs.getString("PKTABLE_NAME"));
+      assertEquals("category", rs.getString("PKCOLUMN_NAME"));
+      assertEquals(database, rs.getString("FKTABLE_CAT"));
+      assertEquals(null, rs.getString("FKTABLE_SCHEM"));
+      assertEquals("product_order", rs.getString("FKTABLE_NAME"));
+      assertEquals("product_category", rs.getString("FKCOLUMN_NAME"));
+      assertEquals(1, rs.getInt("KEY_SEQ"));
+      assertEquals(DatabaseMetaData.importedKeyCascade, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+      assertEquals("product_order_ibfk_1", rs.getString("FK_NAME"));
+      // with show, meta don't know contraint name
+      assertEquals((i == 0) ? null : "unik_name", rs.getString("PK_NAME"));
+      assertEquals(DatabaseMetaData.importedKeyNotDeferrable, rs.getInt("DEFERRABILITY"));
+
+      assertTrue(rs.next());
+      assertEquals("t1", rs.getString("PKTABLE_CAT"));
+      assertEquals(null, rs.getString("PKTABLE_SCHEM"));
+      assertEquals("product", rs.getString("PKTABLE_NAME"));
+      assertEquals("id", rs.getString("PKCOLUMN_NAME"));
+      assertEquals(database, rs.getString("FKTABLE_CAT"));
+      assertEquals(null, rs.getString("FKTABLE_SCHEM"));
+      assertEquals("product_order", rs.getString("FKTABLE_NAME"));
+      assertEquals("product_id", rs.getString("FKCOLUMN_NAME"));
+      assertEquals(2, rs.getInt("KEY_SEQ"));
+      assertEquals(DatabaseMetaData.importedKeyCascade, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+      assertEquals("product_order_ibfk_1", rs.getString("FK_NAME"));
+      // with show, meta don't know contraint name
+      assertEquals((i == 0) ? null : "unik_name", rs.getString("PK_NAME"));
+      assertEquals(DatabaseMetaData.importedKeyNotDeferrable, rs.getInt("DEFERRABILITY"));
+
+      assertTrue(rs.next());
+      assertEquals(database, rs.getString("PKTABLE_CAT"));
+      assertEquals(null, rs.getString("PKTABLE_SCHEM"));
+      assertEquals("cus`tomer", rs.getString("PKTABLE_NAME"));
+      assertEquals("id", rs.getString("PKCOLUMN_NAME"));
+      assertEquals(database, rs.getString("FKTABLE_CAT"));
+      assertEquals(null, rs.getString("FKTABLE_SCHEM"));
+      assertEquals("product_order", rs.getString("FKTABLE_NAME"));
+      assertEquals("customer_id", rs.getString("FKCOLUMN_NAME"));
+      assertEquals(1, rs.getInt("KEY_SEQ"));
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+      assertEquals("product_order_ibfk_2", rs.getString("FK_NAME"));
+      // with show, meta don't know contraint name
+      assertEquals((i == 0) ? null : "PRIMARY", rs.getString("PK_NAME"));
+      assertEquals(DatabaseMetaData.importedKeyNotDeferrable, rs.getInt("DEFERRABILITY"));
     }
 
     /* Also compare metadata */
@@ -297,86 +431,82 @@ public class DatabaseMetadataTest extends BaseTest {
 
   @Test
   public void exportedKeysTest() throws SQLException {
-    Statement stmt = sharedConnection.createStatement();
-    stmt.execute("drop table if exists fore_key0");
-    stmt.execute("drop table if exists fore_key1");
-    stmt.execute("drop table if exists prim_key");
-    stmt.execute("drop table if exists fore_key3");
-    stmt.execute("drop table if exists prim2_key");
-
-    stmt.execute(
-        "create table prim_key (id int not null primary key, val varchar(20)) engine=innodb");
-    stmt.execute(
-        "create table prim2_key (id int not null primary key, val varchar(20)) engine=innodb");
-    stmt.execute(
-        "create table fore_key0 (id int not null primary key, "
-            + "id_ref0 int, foreign key (id_ref0) references prim_key(id)) engine=innodb");
-    stmt.execute(
-        "create table fore_key1 (id int not null primary key, "
-            + "id_ref1 int, foreign key (id_ref1) references prim_key(id) on update cascade) engine=innodb");
-    stmt.execute(
-        "create table fore_key3 (id int not null primary key, "
-            + "id_ref0 int, foreign key (id_ref0) references prim2_key(id)) engine=innodb");
 
     DatabaseMetaData dbmd = sharedConnection.getMetaData();
-    ResultSet rs = dbmd.getExportedKeys("testj", null, "prim_key");
-    int counter = 0;
-    while (rs.next()) {
-      assertEquals("id", rs.getString("pkcolumn_name"));
-      assertEquals("fore_key" + counter, rs.getString("fktable_name"));
-      assertEquals("id_ref" + counter, rs.getString("fkcolumn_name"));
-      assertEquals("PRIMARY", rs.getString("pk_name"));
-      counter++;
+    ResultSet rs = dbmd.getExportedKeys(database, null, "cross%");
+    assertTrue(rs.next());
+    assertEquals(database, rs.getString("PKTABLE_CAT"));
+    assertEquals(null, rs.getString("PKTABLE_SCHEM"));
+    assertEquals("cross1", rs.getString("PKTABLE_NAME"));
+    assertEquals("id", rs.getString("PKCOLUMN_NAME"));
+    assertEquals(database, rs.getString("FKTABLE_CAT"));
+    assertEquals(null, rs.getString("FKTABLE_SCHEM"));
+    assertEquals("cross2", rs.getString("FKTABLE_NAME"));
+    assertEquals("id_ref0", rs.getString("FKCOLUMN_NAME"));
+    assertEquals(1, rs.getInt("KEY_SEQ"));
+    if (!isMariadbServer() && minVersion(8, 0)) {
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("DELETE_RULE"));
+    } else {
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
     }
-    assertEquals(2, counter);
+    assertEquals("cross2_ibfk_1", rs.getString("FK_NAME"));
+    assertEquals("PRIMARY", rs.getString("PK_NAME"));
 
-    rs = dbmd.getExportedKeys("testj", null, "prim_k%");
-    counter = 0;
-    while (rs.next()) {
-      assertEquals("id", rs.getString("pkcolumn_name"));
-      assertEquals("fore_key" + counter, rs.getString("fktable_name"));
-      assertEquals("id_ref" + counter, rs.getString("fkcolumn_name"));
-      assertEquals("PRIMARY", rs.getString("pk_name"));
-      counter++;
+    assertTrue(rs.next());
+
+    assertEquals(database, rs.getString("PKTABLE_CAT"));
+    assertEquals(null, rs.getString("PKTABLE_SCHEM"));
+    assertEquals("cross2", rs.getString("PKTABLE_NAME"));
+    assertEquals("id", rs.getString("PKCOLUMN_NAME"));
+    assertEquals(database, rs.getString("FKTABLE_CAT"));
+    assertEquals(null, rs.getString("FKTABLE_SCHEM"));
+    assertEquals("cross3", rs.getString("FKTABLE_NAME"));
+    assertEquals("id_ref1", rs.getString("FKCOLUMN_NAME"));
+    assertEquals(1, rs.getInt("KEY_SEQ"));
+    assertEquals(DatabaseMetaData.importedKeyCascade, rs.getInt("UPDATE_RULE"));
+    if (!isMariadbServer() && minVersion(8, 0)) {
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("DELETE_RULE"));
+    } else {
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
     }
-    assertEquals(2, counter);
-
-    rs = dbmd.getExportedKeys("testj", null, null);
-    counter = 0;
-    int totalCounter = 0;
-    while (rs.next()) {
-      if ("prim_key".equals(rs.getString("pktable_name"))) {
-        assertEquals("id", rs.getString("pkcolumn_name"));
-        assertEquals("fore_key" + counter, rs.getString("fktable_name"));
-        assertEquals("id_ref" + counter, rs.getString("fkcolumn_name"));
-        assertEquals("PRIMARY", rs.getString("PK_NAME"));
-        counter++;
-      }
-      totalCounter++;
+    if (!isMariadbServer()) {
+      // index name not taken into account
+      assertEquals("cross3_ibfk_1", rs.getString("FK_NAME"));
+    } else {
+      assertEquals("fk_my_name", rs.getString("FK_NAME"));
     }
-    assertEquals(2, counter);
-    assertTrue(totalCounter > 2);
+    assertEquals("unik_name", rs.getString("PK_NAME"));
 
-    stmt.execute("drop table if exists fore_key0");
-    stmt.execute("drop table if exists fore_key1");
-    stmt.execute("drop table if exists prim_key");
+    assertTrue(rs.next());
+    assertEquals(database, rs.getString("PKTABLE_CAT"));
+    assertEquals(null, rs.getString("PKTABLE_SCHEM"));
+    assertEquals("cross2", rs.getString("PKTABLE_NAME"));
+    assertEquals("id2", rs.getString("PKCOLUMN_NAME"));
+    assertEquals(database, rs.getString("FKTABLE_CAT"));
+    assertEquals(null, rs.getString("FKTABLE_SCHEM"));
+    assertEquals("cross3", rs.getString("FKTABLE_NAME"));
+    assertEquals("id_ref2", rs.getString("FKCOLUMN_NAME"));
+    assertEquals(2, rs.getInt("KEY_SEQ"));
+    assertEquals(DatabaseMetaData.importedKeyCascade, rs.getInt("UPDATE_RULE"));
+    if (!isMariadbServer() && minVersion(8, 0)) {
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("DELETE_RULE"));
+    } else {
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+    }
+    if (!isMariadbServer()) {
+      // wrong index name
+      assertEquals("cross3_ibfk_1", rs.getString("FK_NAME"));
+    } else {
+      assertEquals("fk_my_name", rs.getString("FK_NAME"));
+    }
+    assertEquals("unik_name", rs.getString("PK_NAME"));
   }
 
   @Test
   public void importedKeysTest() throws SQLException {
     Statement stmt = sharedConnection.createStatement();
-    stmt.execute("drop table if exists fore_key0");
-    stmt.execute("drop table if exists fore_key1");
-    stmt.execute("drop table if exists prim_key");
-
-    stmt.execute(
-        "create table prim_key (id int not null primary key, " + "val varchar(20)) engine=innodb");
-    stmt.execute(
-        "create table fore_key0 (id int not null primary key, "
-            + "id_ref0 int, foreign key (id_ref0) references prim_key(id)) engine=innodb");
-    stmt.execute(
-        "create table fore_key1 (id int not null primary key, "
-            + "id_ref1 int, foreign key (id_ref1) references prim_key(id) on update cascade) engine=innodb");
 
     DatabaseMetaData dbmd = sharedConnection.getMetaData();
     ResultSet rs = dbmd.getImportedKeys(sharedConnection.getCatalog(), null, "fore_key0");
@@ -387,9 +517,6 @@ public class DatabaseMetadataTest extends BaseTest {
       counter++;
     }
     assertEquals(1, counter);
-    stmt.execute("drop table if exists fore_key0");
-    stmt.execute("drop table if exists fore_key1");
-    stmt.execute("drop table if exists prim_key");
   }
 
   @Test
@@ -415,19 +542,6 @@ public class DatabaseMetadataTest extends BaseTest {
   @Test
   public void testGetTables() throws SQLException {
     Statement stmt = sharedConnection.createStatement();
-    stmt.execute("drop table if exists fore_key0");
-    stmt.execute("drop table if exists fore_key1");
-    stmt.execute("drop table if exists prim_key");
-
-    stmt.execute(
-        "create table prim_key (id int not null primary key, " + "val varchar(20)) engine=innodb");
-    stmt.execute(
-        "create table fore_key0 (id int not null primary key, "
-            + "id_ref0 int, foreign key (id_ref0) references prim_key(id)) engine=innodb");
-    stmt.execute(
-        "create table fore_key1 (id int not null primary key, "
-            + "id_ref1 int, foreign key (id_ref1) references prim_key(id) on update cascade) engine=innodb");
-
     DatabaseMetaData dbmd = sharedConnection.getMetaData();
     ResultSet rs = dbmd.getTables(null, null, "prim_key", null);
 
@@ -465,12 +579,6 @@ public class DatabaseMetadataTest extends BaseTest {
   @Test
   public void testGetTables3() throws SQLException {
     Statement stmt = sharedConnection.createStatement();
-    stmt.execute("drop table if exists table_type_test");
-
-    stmt.execute(
-        "create table table_type_test (id int not null primary key, "
-            + "val varchar(20)) engine=innodb");
-
     DatabaseMetaData dbmd = sharedConnection.getMetaData();
     ResultSet tableSet = dbmd.getTables(null, null, "table_type_test", null);
 
@@ -489,19 +597,19 @@ public class DatabaseMetadataTest extends BaseTest {
   public void testGetColumns() throws SQLException {
     // mysql 5.6 doesn't permit VIRTUAL keyword
     Assume.assumeTrue(isMariadbServer() || !isMariadbServer() && minVersion(5, 7));
-
-    if (minVersion(10, 2) || !isMariadbServer()) {
-      createTable(
-          "tablegetcolumns",
-          "a INT NOT NULL primary key auto_increment, b VARCHAR(32), c INT AS (CHAR_LENGTH(b)) VIRTUAL, "
-              + "d VARCHAR(5) AS (left(b,5)) STORED",
-          "CHARACTER SET 'utf8mb4'");
-    } else {
-      createTable(
-          "tablegetcolumns",
-          "a INT NOT NULL primary key auto_increment, b VARCHAR(32), c INT AS (CHAR_LENGTH(b)) VIRTUAL, "
-              + "d VARCHAR(5) AS (left(b,5)) PERSISTENT",
-          "CHARACTER SET 'utf8mb4'");
+    try (Statement stmt = sharedConnection.createStatement()) {
+      if (minVersion(10, 2) || !isMariadbServer()) {
+        stmt.execute(
+            "CREATE TABLE tablegetcolumns("
+                + "a INT NOT NULL primary key auto_increment, b VARCHAR(32), c INT AS (CHAR_LENGTH(b)) VIRTUAL, "
+                + "d VARCHAR(5) AS (left(b,5)) STORED) CHARACTER SET 'utf8mb4'");
+      } else {
+        stmt.execute(
+            "CREATE TABLE tablegetcolumns("
+                + "a INT NOT NULL primary key auto_increment, b VARCHAR(32), c INT AS (CHAR_LENGTH(b)) VIRTUAL, "
+                + "d VARCHAR(5) AS (left(b,5)) PERSISTENT) CHARACTER SET 'utf8mb4'");
+      }
+      stmt.execute("FLUSH TABLES");
     }
 
     DatabaseMetaData dbmd = sharedConnection.getMetaData();
@@ -778,12 +886,103 @@ public class DatabaseMetadataTest extends BaseTest {
   }
 
   @Test
-  public void getBestRowIdentifierBasic() throws SQLException {
+  public void getBestRowIdentifier() throws SQLException {
     testResultSetColumns(
         sharedConnection.getMetaData().getBestRowIdentifier(null, null, "", 0, true),
         "SCOPE short,COLUMN_NAME String,DATA_TYPE int, TYPE_NAME String,"
             + "COLUMN_SIZE int,BUFFER_LENGTH int,"
             + "DECIMAL_DIGITS short,PSEUDO_COLUMN short");
+
+    ResultSet rs =
+        sharedConnection.getMetaData().getBestRowIdentifier(null, null, "cross1", 0, true);
+    assertTrue(rs.next());
+
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("id", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertFalse(rs.next());
+
+    rs = sharedConnection.getMetaData().getBestRowIdentifier(null, null, "cross2", 0, true);
+    assertTrue(rs.next());
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("id", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertTrue(rs.next());
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("id2", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertFalse(rs.next());
+
+    rs = sharedConnection.getMetaData().getBestRowIdentifier(null, null, "cross3", 0, true);
+    assertTrue(rs.next());
+
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("id", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertFalse(rs.next());
+
+    // CHECK using PRI even if exist UNI
+
+    rs =
+        sharedConnection
+            .getMetaData()
+            .getBestRowIdentifier(null, null, "getBestRowIdentifier1", 0, true);
+    assertTrue(rs.next());
+
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("i", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertFalse(rs.next());
+
+    rs =
+        sharedConnection
+            .getMetaData()
+            .getBestRowIdentifier(null, null, "getBestRowIdentifier2", 0, true);
+    assertTrue(rs.next());
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("id_ref0", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertTrue(rs.next());
+
+    assertEquals(DatabaseMetaData.bestRowSession, rs.getInt(1));
+    assertEquals("id_ref2", rs.getString(2));
+    assertEquals(Types.INTEGER, rs.getInt(3));
+    assertEquals("int", rs.getString(4));
+    assertEquals(10, rs.getInt(5));
+    assertEquals(0, rs.getInt(6));
+    assertEquals(0, rs.getInt(7));
+    assertEquals(DatabaseMetaData.bestRowNotPseudo, rs.getInt(8));
+    assertFalse(rs.next());
   }
 
   @Test
@@ -931,6 +1130,78 @@ public class DatabaseMetadataTest extends BaseTest {
             + "UPDATE_RULE short,DELETE_RULE short,FK_NAME String,PK_NAME String,DEFERRABILITY short");
   }
 
+  @Test
+  public void getCrossReferenceResults() throws SQLException {
+    DatabaseMetaData dbmd = sharedConnection.getMetaData();
+    ResultSet rs = dbmd.getCrossReference(null, null, "cross%", null, null, "cross%");
+
+    assertTrue(rs.next());
+    assertEquals(database, rs.getString(1));
+    assertEquals(null, rs.getString(2));
+    assertEquals("cross1", rs.getString(3));
+    assertEquals("id", rs.getString(4));
+    assertEquals(database, rs.getString(5));
+    assertEquals(null, rs.getString(6));
+    assertEquals("cross2", rs.getString(7));
+    assertEquals("id_ref0", rs.getString(8));
+    assertEquals(1, rs.getInt(9));
+    if (!isMariadbServer() && minVersion(8, 0)) {
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("DELETE_RULE"));
+    } else {
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("UPDATE_RULE"));
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+    }
+    assertEquals("cross2_ibfk_1", rs.getString(12));
+
+    assertTrue(rs.next());
+    assertEquals(database, rs.getString(1));
+    assertEquals(null, rs.getString(2));
+    assertEquals("cross2", rs.getString(3));
+    assertEquals("id", rs.getString(4));
+    assertEquals(database, rs.getString(5));
+    assertEquals(null, rs.getString(6));
+    assertEquals("cross3", rs.getString(7));
+    assertEquals("id_ref1", rs.getString(8));
+    assertEquals(1, rs.getInt(9));
+    assertEquals(DatabaseMetaData.importedKeyCascade, rs.getInt(10));
+    if (!isMariadbServer() && minVersion(8, 0)) {
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("DELETE_RULE"));
+    } else {
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+    }
+    if (!isMariadbServer()) {
+      // wrong index naming
+      assertEquals("cross3_ibfk_1", rs.getString(12));
+    } else {
+      assertEquals("fk_my_name", rs.getString("FK_NAME"));
+    }
+
+    assertTrue(rs.next());
+    assertEquals(database, rs.getString(1));
+    assertEquals(null, rs.getString(2));
+    assertEquals("cross2", rs.getString(3));
+    assertEquals("id2", rs.getString(4));
+    assertEquals(database, rs.getString(5));
+    assertEquals(null, rs.getString(6));
+    assertEquals("cross3", rs.getString(7));
+    assertEquals("id_ref2", rs.getString(8));
+    assertEquals(2, rs.getInt(9));
+    assertEquals(DatabaseMetaData.importedKeyCascade, rs.getInt(10));
+    if (!isMariadbServer() && minVersion(8, 0)) {
+      assertEquals(DatabaseMetaData.importedKeyNoAction, rs.getInt("DELETE_RULE"));
+    } else {
+      assertEquals(DatabaseMetaData.importedKeyRestrict, rs.getInt("DELETE_RULE"));
+    }
+    if (!isMariadbServer()) {
+      // wrong index naming
+      assertEquals("cross3_ibfk_1", rs.getString(12));
+    } else {
+      assertEquals("fk_my_name", rs.getString(12));
+    }
+    assertFalse(rs.next());
+  }
+
   @Test
   public void getUdtsBasic() throws SQLException {
     testResultSetColumns(
@@ -1133,16 +1404,6 @@ public class DatabaseMetadataTest extends BaseTest {
 
   @Test
   public void getPrecision() throws SQLException {
-    createTable(
-        "getPrecision",
-        "num1 NUMERIC(9,4), "
-            + "num2 NUMERIC (9,0),"
-            + "num3 NUMERIC (9,4) UNSIGNED,"
-            + "num4 NUMERIC (9,0) UNSIGNED,"
-            + "num5 FLOAT(9,4),"
-            + "num6 FLOAT(9,4) UNSIGNED,"
-            + "num7 DOUBLE(9,4),"
-            + "num8 DOUBLE(9,4) UNSIGNED");
     Statement stmt = sharedConnection.createStatement();
     ResultSet rs = stmt.executeQuery("SELECT * FROM getPrecision");
     ResultSetMetaData rsmd = rs.getMetaData();
@@ -1167,15 +1428,6 @@ public class DatabaseMetadataTest extends BaseTest {
   @Test
   public void getTimePrecision() throws SQLException {
     Assume.assumeTrue(doPrecisionTest);
-    createTable(
-        "getTimePrecision",
-        "d date, "
-            + "t1 datetime(0),"
-            + "t2 datetime(6),"
-            + "t3 timestamp(0) DEFAULT '2000-01-01 00:00:00',"
-            + "t4 timestamp(6) DEFAULT '2000-01-01 00:00:00',"
-            + "t5 time(0),"
-            + "t6 time(6)");
     Statement stmt = sharedConnection.createStatement();
     ResultSet rs = stmt.executeQuery("SELECT * FROM getTimePrecision");
     ResultSetMetaData rsmd = rs.getMetaData();
@@ -1202,23 +1454,31 @@ public class DatabaseMetadataTest extends BaseTest {
     assertEquals(6, rsmd.getScale(7));
   }
 
+  @Test
+  public void getStringPrecision() throws SQLException {
+    Assume.assumeTrue(doPrecisionTest);
+    Statement stmt = sharedConnection.createStatement();
+    ResultSet rs = stmt.executeQuery("SELECT * FROM getStringPrecision");
+    ResultSetMetaData rsmd = rs.getMetaData();
+    // CHAR(100) CHARSET=utf8mb4
+    assertEquals(100, rsmd.getPrecision(1));
+    assertEquals(0, rsmd.getScale(1));
+    // CHAR(100) CHARSET=utf8
+    assertEquals(100, rsmd.getPrecision(2));
+    assertEquals(0, rsmd.getScale(2));
+    // CHAR(100) CHARSET=latin2
+    assertEquals(100, rsmd.getPrecision(3));
+    assertEquals(0, rsmd.getScale(3));
+  }
+
   @Test
   public void metaTimeResultSet() throws SQLException {
     Assume.assumeTrue(doPrecisionTest);
-    createTable(
-        "getTimePrecision",
-        "d date, "
-            + "t1 datetime(0),"
-            + "t2 datetime(6),"
-            + "t3 timestamp(0) DEFAULT '2000-01-01 00:00:00',"
-            + "t4 timestamp(6) DEFAULT '2000-01-01 00:00:00',"
-            + "t5 time(0),"
-            + "t6 time(6)");
 
     final int columnSizeField = 7;
 
     DatabaseMetaData dmd = sharedConnection.getMetaData();
-    ResultSet rs = dmd.getColumns(null, null, "getTimePrecision", null);
+    ResultSet rs = dmd.getColumns(null, null, "getTimePrecision2", null);
     // date
     assertTrue(rs.next());
     assertEquals(10, rs.getInt(columnSizeField));
@@ -1251,12 +1511,6 @@ public class DatabaseMetadataTest extends BaseTest {
    */
   @Test
   public void metaTimeNoPrecisionProcedureResultSet() throws SQLException {
-    createProcedure(
-        "getProcTimePrecision2",
-        "(IN  I date, "
-            + "IN t1 DATETIME,"
-            + "IN t3 timestamp,"
-            + "IN t5 time) BEGIN SELECT I; END");
 
     final int precisionField = 8;
     final int lengthField = 9;
@@ -1297,15 +1551,6 @@ public class DatabaseMetadataTest extends BaseTest {
   @Test
   public void metaTimeProcedureResultSet() throws SQLException {
     Assume.assumeTrue(doPrecisionTest);
-    createProcedure(
-        "getProcTimePrecision",
-        "(IN  I date, "
-            + "IN t1 DATETIME(0),"
-            + "IN t2 DATETIME(6),"
-            + "IN t3 timestamp(0),"
-            + "IN t4 timestamp(6),"
-            + "IN t5 time ,"
-            + "IN t6 time(6)) BEGIN SELECT I; END");
 
     final int precisionField = 8;
     final int lengthField = 9;
@@ -1352,4 +1597,10 @@ public class DatabaseMetadataTest extends BaseTest {
 
     assertFalse(rs.next());
   }
+
+  @Test
+  public void various() throws SQLException {
+    DatabaseMetaData meta = sharedConnection.getMetaData();
+    assertEquals(64, meta.getMaxProcedureNameLength());
+  }
 }
