@@ -57,18 +57,36 @@ import static org.junit.Assert.*;
 import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.SQLException;
+import java.sql.Statement;
 import java.util.UUID;
 import javax.sql.XAConnection;
 import javax.transaction.xa.XAException;
 import javax.transaction.xa.XAResource;
 import javax.transaction.xa.Xid;
+import org.junit.AfterClass;
 import org.junit.Assume;
-import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
 public class DistributedTransactionTest extends BaseTest {
 
+  @BeforeClass()
+  public static void initClass() throws SQLException {
+    Assume.assumeTrue(System.getenv("SKYSQL") == null && System.getenv("SKYSQL_HA") == null);
+    afterClass();
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute("CREATE TABLE xatable(i int)");
+      stmt.execute("FLUSH TABLES");
+    }
+  }
+
+  @AfterClass
+  public static void afterClass() throws SQLException {
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute("DROP TABLE IF EXISTS xatable");
+    }
+  }
+
   private final MariaDbDataSource dataSource;
 
   /** Initialisation. */
@@ -81,17 +99,6 @@ public class DistributedTransactionTest extends BaseTest {
     dataSource.setPassword(password);
   }
 
-  @BeforeClass()
-  public static void initClass() throws SQLException {
-    Assume.assumeFalse(options.useSsl != null && options.useSsl);
-    createTable("xatable", "i int", "ENGINE=InnoDB");
-  }
-
-  @Before
-  public void checkSupported() throws SQLException {
-    requireMinimumVersion(5, 0);
-  }
-
   private Xid newXid() {
     return new MariaDbXid(
         1, UUID.randomUUID().toString().getBytes(), UUID.randomUUID().toString().getBytes());
@@ -110,7 +117,7 @@ public class DistributedTransactionTest extends BaseTest {
    */
   private int test2PhaseCommit(boolean doCommit) throws Exception {
 
-    int connectionNumber = 1;
+    int connectionNumber = 2;
 
     Xid parentXid = newXid();
     Connection[] connections = new Connection[connectionNumber];
@@ -242,6 +249,7 @@ public class DistributedTransactionTest extends BaseTest {
     ds.setUser(username);
     ds.setPassword(password);
     ds.setPort(port);
+
     XAConnection xaConn1 = null;
     Xid xid = newXid();
     try {
