@@ -84,7 +84,8 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
    */
   @BeforeClass()
   public static void initClass() throws SQLException {
-    if (testSingleHost || !"true".equals(System.getenv("AURORA"))) {
+    if (testSingleHost
+        || ((System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null))) {
       try (Statement st = sharedConnection.createStatement()) {
         ResultSet rs =
             st.executeQuery(
@@ -123,19 +124,53 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
       utcDateFormatSimple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
       utcDateFormatSimple.setTimeZone(utcTimeZone);
-      createTable("daylightMysql", " tt DATE");
-      if (doPrecisionTest) {
-        createTable("timeZoneTime", "id int, tt TIME(6)");
-        createTable(
-            "ttimeTest", "id int not null primary key auto_increment, dd TIME(3), dd2 TIME(3)");
+      try (Statement stmt = sharedConnection.createStatement()) {
+        stmt.execute("CREATE TABLE daylightMysql(tt DATE)");
+        stmt.execute(
+            "CREATE TABLE daylight(id int, t1 TIMESTAMP(6) NULL, t2 TIME(6), t3 DATETIME(6) , t4 DATE)");
+        stmt.execute("CREATE TABLE daylight2(id int, tt TIMESTAMP(6))");
+        stmt.execute(
+            "CREATE TABLE checkLocalDateTimefalsefalse(id int, tt DATETIME(6), tt2 TIME(6), tt3 varchar(100), tt4 varchar(100))");
+        stmt.execute(
+            "CREATE TABLE checkLocalDateTimefalsetrue(id int, tt DATETIME(6), tt2 TIME(6), tt3 varchar(100), tt4 varchar(100))");
+        stmt.execute(
+            "CREATE TABLE checkLocalDateTimetruetrue(id int, tt DATETIME(6), tt2 TIME(6), tt3 varchar(100), tt4 varchar(100))");
+        stmt.execute(
+            "CREATE TABLE checkLocalDateTimetruefalse(id int, tt DATETIME(6), tt2 TIME(6), tt3 varchar(100), tt4 varchar(100))");
+        stmt.execute("CREATE TABLE timeVerificationWithTimezone(time_field TIME)");
+        stmt.execute("CREATE TABLE daylightCanada(id int, tt TIMESTAMP(6))");
+
+        if (doPrecisionTest) {
+          stmt.execute("CREATE TABLE timeZoneTime(id int, tt TIME(6))");
+          stmt.execute(
+              "CREATE TABLE ttimeTest(id int not null primary key auto_increment, dd TIME(3), dd2 TIME(3))");
+        }
+        stmt.execute("FLUSH TABLES");
       }
     }
   }
 
+  @AfterClass
+  public static void afterClass() throws SQLException {
+    try (Statement stmt = sharedConnection.createStatement()) {
+      stmt.execute("DROP TABLE IF EXISTS daylightMysql");
+      stmt.execute("DROP TABLE IF EXISTS daylight");
+      stmt.execute("DROP TABLE IF EXISTS daylight2");
+      stmt.execute("DROP TABLE IF EXISTS checkLocalDateTimefalsefalse");
+      stmt.execute("DROP TABLE IF EXISTS checkLocalDateTimefalsetrue");
+      stmt.execute("DROP TABLE IF EXISTS checkLocalDateTimetruetrue");
+      stmt.execute("DROP TABLE IF EXISTS checkLocalDateTimetruefalse");
+      stmt.execute("DROP TABLE IF EXISTS timeVerificationWithTimezone");
+      stmt.execute("DROP TABLE IF EXISTS daylightCanada");
+      stmt.execute("DROP TABLE IF EXISTS timeZoneTime");
+      stmt.execute("DROP TABLE IF EXISTS ttimeTest");
+    }
+  }
   /** Put the TimeZone to previous state. */
   @AfterClass()
   public static void endClass() {
-    if (testSingleHost || !"true".equals(System.getenv("AURORA"))) {
+    if (testSingleHost
+        || ((System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null))) {
       TimeZone.setDefault(previousTimeZone);
       if (previousFormatLocale != null) {
         Locale.setDefault(previousFormatLocale);
@@ -182,7 +217,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testTimeStamp() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     TimeZone.setDefault(parisTimeZone);
     try (Connection connection =
         setConnection("&serverTimezone=Europe/Paris&useServerPrepStmts=true")) {
@@ -216,7 +251,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testTimeStampUtcNow() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     TimeZone.setDefault(parisTimeZone);
     try (Connection connection = setConnection("&serverTimezone=UTC&useServerPrepStmts=true")) {
       TimeZone.setDefault(parisTimeZone);
@@ -296,7 +331,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testTimeUtc() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(doPrecisionTest);
 
     TimeZone.setDefault(parisTimeZone);
@@ -329,7 +364,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testTimeUtcNow() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     TimeZone.setDefault(parisTimeZone);
     try (Connection connection = setConnection("&serverTimezone=UTC")) {
       setSessionTimeZone(connection, "+00:00");
@@ -347,7 +382,10 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testTimeOffsetNowUseServer() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(
+        System.getenv("AURORA") == null
+            && System.getenv("SKYSQL_HA") == null
+            && System.getenv("SKYSQL") == null);
     try (Connection connection =
         setConnection("&useLegacyDatetimeCode=false&serverTimezone=+5:00")) {
       setSessionTimeZone(connection, "+5:00");
@@ -358,13 +396,15 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
       assertTrue(rs.next());
       Timestamp nowServer = rs.getTimestamp(1);
       long timeDifference = currentTimeParis.getTime() - nowServer.getTime();
-      assertTrue(timeDifference < 1000); // must have less than one second difference
+      assertTrue(
+          "Error, timeDifference too important : " + timeDifference,
+          timeDifference < 1000); // must have less than one second difference
     }
   }
 
   @Test
   public void testDifferentTimeZoneServer() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     try (Connection connection = setConnection("&serverTimezone=UTC")) {
       setSessionTimeZone(sharedConnection, "+00:00");
       // timestamp timezone to parisTimeZone like server
@@ -380,7 +420,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testTimeStampOffsetNowUseServer() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     try (Connection connection = setConnection("&serverTimezone=Europe/Paris")) {
       // timestamp timezone to parisTimeZone like server
       Timestamp currentTimeParis = new Timestamp(System.currentTimeMillis());
@@ -404,7 +444,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
   }
 
   private void testDayLight(boolean legacy) throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(doPrecisionTest);
     Assume.assumeTrue(hasSuperPrivilege("testDayLight") && !sharedIsRewrite());
     TimeZone.setDefault(parisTimeZone);
@@ -413,9 +453,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
             "&useLegacyDatetimeCode="
                 + legacy
                 + "&serverTimezone=Canada/Atlantic&sessionVariables=time_zone='Canada/Atlantic'")) {
-
-      createTable("daylight", "id int, t1 TIMESTAMP(6) NULL, t2 TIME(6), t3 DATETIME(6) , t4 DATE");
-
+      sharedConnection.createStatement().execute("TRUNCATE daylight");
       Calendar quarterBeforeChangingHour = Calendar.getInstance(TimeZone.getTimeZone("utc"));
       quarterBeforeChangingHour.clear();
       quarterBeforeChangingHour.set(2015, Calendar.MARCH, 29, 0, 45, 0);
@@ -517,8 +555,8 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
       assertTrue(rs1.getBoolean(3));
       assertTrue(rs1.getBoolean(4));
 
-      checkResult(legacy, true, connection);
-      checkResult(legacy, false, connection);
+      checkResult(legacy, false);
+      checkResult(legacy, true);
     }
   }
 
@@ -527,241 +565,252 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
    *
    * @param legacy is in legacy mode
    * @param binaryProtocol binary protocol
-   * @param connection connection
    * @return current resultset
    * @throws SQLException if connection error occur.
    */
-  public ResultSet checkResult(boolean legacy, boolean binaryProtocol, Connection connection)
-      throws SQLException {
-    ResultSet rs;
-    PreparedStatement pst;
-    if (binaryProtocol) {
-      pst =
-          connection.prepareStatement(
-              "SELECT * from daylight where 1 = ?",
-              ResultSet.TYPE_SCROLL_INSENSITIVE,
-              ResultSet.CONCUR_READ_ONLY);
-    } else {
-      MariaDbConnection mariaDbConnection = (MariaDbConnection) connection;
-      ExceptionFactory exceptionFactory =
-          ExceptionFactory.of((int) mariaDbConnection.getServerThreadId(), new Options());
-      pst =
-          new ClientSidePreparedStatement(
-              mariaDbConnection,
-              "SELECT * from daylight where 1 = ?",
-              ResultSet.TYPE_SCROLL_INSENSITIVE,
-              ResultSet.CONCUR_READ_ONLY,
-              Statement.NO_GENERATED_KEYS,
-              exceptionFactory);
-    }
-    pst.setInt(1, 1);
-    rs = pst.executeQuery();
+  public ResultSet checkResult(boolean legacy, boolean binaryProtocol) throws SQLException {
+    try (Connection connection =
+        setConnection(
+            "&useServerPrepStmts="
+                + binaryProtocol
+                + "&useLegacyDatetimeCode="
+                + legacy
+                + "&serverTimezone=Canada/Atlantic&sessionVariables=time_zone='Canada/Atlantic'")) {
+      ResultSet rs;
+      PreparedStatement pst;
+      if (binaryProtocol) {
+        pst =
+            connection.prepareStatement(
+                "SELECT * from daylight where 1 = ?",
+                ResultSet.TYPE_SCROLL_INSENSITIVE,
+                ResultSet.CONCUR_READ_ONLY);
+      } else {
+        MariaDbConnection mariaDbConnection = (MariaDbConnection) connection;
+        ExceptionFactory exceptionFactory =
+            ExceptionFactory.of((int) mariaDbConnection.getServerThreadId(), new Options());
+        pst =
+            new ClientSidePreparedStatement(
+                mariaDbConnection,
+                "SELECT * from daylight where 1 = ?",
+                ResultSet.TYPE_SCROLL_INSENSITIVE,
+                ResultSet.CONCUR_READ_ONLY,
+                Statement.NO_GENERATED_KEYS,
+                exceptionFactory);
+      }
+      pst.setInt(1, 1);
+      rs = pst.executeQuery();
 
-    assertTrue(rs.next());
+      assertTrue(rs.next());
 
-    // test timestamp(6)
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getTimestamp(2)));
-    assertEquals(
-        "2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(2, Timestamp.class)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(2, Time.class)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(2, Date.class)));
-    assertEquals(
-        "2015-03-29T01:45:00.012+0100",
-        formatter.format(rs.getObject(2, Calendar.class).getTime()));
-    assertEquals(
-        "2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(2, java.util.Date.class)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getTime(2)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getDate(2)));
-
-    // test time(6)
-    assertEquals("1970-01-01T01:45:00.000+0100", formatter.format(rs.getTimestamp(3)));
-    assertEquals(
-        "1970-01-01T01:45:00.000+0100", formatter.format(rs.getObject(3, Timestamp.class)));
-    assertEquals("1970-01-01T01:45:00.000+0100", formatter.format(rs.getObject(3, Time.class)));
-    assertEquals(
-        "1970-01-01T01:45:00.000+0100",
-        formatter.format(rs.getObject(3, Calendar.class).getTime()));
-    assertEquals(
-        "1970-01-01T01:45:00.000+0100", formatter.format(rs.getObject(3, java.util.Date.class)));
-    try {
-      formatter.format(rs.getObject(3, Date.class));
-      fail();
-    } catch (SQLException e) {
-      // expected exception
-    }
-    assertEquals("1970-01-01T01:45:00.000+0100", formatter.format(rs.getTime(3)));
-    assertEquals(2700000, rs.getTime(3).getTime());
-    try {
-      formatter.format(rs.getDate(3));
-      fail();
-    } catch (SQLException e) {
-      // expected exception
-    }
-
-    // test datetime(6)
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getTimestamp(4)));
-    assertEquals(
-        "2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(4, Timestamp.class)));
-    assertEquals(
-        "2015-03-29T01:45:00.012+0100",
-        formatter.format(rs.getObject(4, Calendar.class).getTime()));
-    assertEquals(
-        "2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(4, java.util.Date.class)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(4, Time.class)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(4, Date.class)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getTime(4)));
-    assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getDate(4)));
-
-    // test date(6)
-    assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getTimestamp(5)));
-    assertEquals(
-        "2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Timestamp.class)));
-    try {
-      formatter.format(rs.getObject(5, Time.class));
-      fail();
-    } catch (SQLException e) {
-      // expected exception
-    }
-    assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Date.class)));
-    try {
-      formatter.format(rs.getTime(5));
-      fail();
-    } catch (SQLException e) {
-      // expected exception
-    }
-    assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(5)));
-    assertEquals(new Date(2015 - 1900, 2, 29), rs.getDate(5));
-    assertEquals(rs.getString(5), "2015-03-29");
-
-    assertTrue(rs.next());
-    // test timestamp(6)
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getTimestamp(2)));
-    assertEquals(
-        "2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(2, Timestamp.class)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(2, Time.class)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(2, Date.class)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getTime(2)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getDate(2)));
-
-    // test time(6)
-    assertEquals("1970-01-01T03:15:00.000+0100", formatter.format(rs.getTimestamp(3)));
-    assertEquals(
-        "1970-01-01T03:15:00.000+0100", formatter.format(rs.getObject(3, Timestamp.class)));
-    assertEquals("1970-01-01T03:15:00.000+0100", formatter.format(rs.getObject(3, Time.class)));
-    try {
-      formatter.format(rs.getObject(3, Date.class));
-      fail();
-    } catch (SQLException e) {
-      // expected exception
-    }
-    assertEquals("1970-01-01T03:15:00.000+0100", formatter.format(rs.getTime(3)));
-    assertEquals(8100000, rs.getTime(3).getTime());
-    try {
-      formatter.format(rs.getDate(3));
-      fail();
-    } catch (SQLException e) {
-      // expected exception
-    }
-
-    // test datetime(6)
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getTimestamp(4)));
-    assertEquals(
-        "2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(4, Timestamp.class)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(4, Time.class)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(4, Date.class)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getTime(4)));
-    assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getDate(4)));
-
-    // test date(6)
-    assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getTimestamp(5)));
-    assertEquals(
-        "2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Timestamp.class)));
-    try {
-      formatter.format(rs.getObject(5, Time.class));
-    } catch (SQLException e) {
-      // expected exception
-    }
-    assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Date.class)));
-    try {
-      formatter.format(rs.getTime(5));
-    } catch (SQLException e) {
-      // expected exception
-    }
-    assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(5)));
-    assertEquals(new Date(2015 - 1900, 2, 29), rs.getDate(5));
-    assertEquals(rs.getString(5), "2015-03-29");
-
-    assertTrue(rs.next());
-    // test timestamp(6)
-    for (int i = 2; i < 6; i++) {
-      assertNull(rs.getTimestamp(i));
-      assertNull(rs.getTime(i));
-      assertNull(rs.getDate(i));
-      assertNull(rs.getObject(i, Timestamp.class));
-      assertNull(rs.getObject(i, Time.class));
-      assertNull(rs.getObject(i, Date.class));
-      assertNull(rs.getObject(i, Calendar.class));
-      assertNull(rs.getObject(i, java.util.Date.class));
-    }
-
-    rs.first();
-
-    // additional tests for java 8 objects
-    assertEquals("2015-03-29T01:45:00.012340", rs.getTimestamp(2).toLocalDateTime().toString());
-    assertEquals("2015-03-29T01:45:00.012340", rs.getObject(2, LocalDateTime.class).toString());
-    if (legacy) {
+      // test timestamp(6)
+      assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getTimestamp(2)));
       assertEquals(
-          "2015-03-29T01:45:00.012340+01:00[Europe/Paris]",
-          rs.getObject(2, ZonedDateTime.class).toString());
+          "2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(2, Timestamp.class)));
+      assertEquals("1970-01-01T01:45:00.012+0100", formatter.format(rs.getObject(2, Time.class)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(2, Date.class)));
       assertEquals(
-          "2015-03-29T01:45:00.012340+01:00", rs.getObject(2, OffsetDateTime.class).toString());
-    } else {
+          "2015-03-29T01:45:00.012+0100",
+          formatter.format(rs.getObject(2, Calendar.class).getTime()));
       assertEquals(
-          "2015-03-28T21:45:00.012340-03:00[Canada/Atlantic]",
-          rs.getObject(2, ZonedDateTime.class).toString());
-      assertEquals(
-          "2015-03-28T21:45:00.012340-03:00", rs.getObject(2, OffsetDateTime.class).toString());
-    }
-    assertEquals("01:45:00.012340", rs.getObject(2, LocalTime.class).toString());
-    assertEquals("2015-03-29", rs.getObject(2, LocalDate.class).toString());
+          "2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(2, java.util.Date.class)));
+      assertEquals("1970-01-01T01:45:00.012+0100", formatter.format(rs.getTime(2)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(2)));
 
-    assertTrue(rs.next());
-    // additional tests for java 8 objects
-    assertEquals("2015-03-29T03:15:00.012340", rs.getTimestamp(2).toLocalDateTime().toString());
-    assertEquals("2015-03-29T03:15:00.012340", rs.getObject(2, LocalDateTime.class).toString());
-    if (legacy) {
+      // test time(6)
+      assertEquals("1970-01-01T01:45:00.000+0100", formatter.format(rs.getTimestamp(3)));
       assertEquals(
-          "2015-03-29T03:15:00.012340+02:00[Europe/Paris]",
-          rs.getObject(2, ZonedDateTime.class).toString());
+          "1970-01-01T01:45:00.000+0100", formatter.format(rs.getObject(3, Timestamp.class)));
+      assertEquals("1970-01-01T01:45:00.000+0100", formatter.format(rs.getObject(3, Time.class)));
       assertEquals(
-          "2015-03-29T03:15:00.012340+02:00", rs.getObject(2, OffsetDateTime.class).toString());
-    } else {
-      assertEquals(
-          "2015-03-28T22:15:00.012340-03:00[Canada/Atlantic]",
-          rs.getObject(2, ZonedDateTime.class).toString());
-      assertEquals(
-          "2015-03-28T22:15:00.012340-03:00", rs.getObject(2, OffsetDateTime.class).toString());
-    }
-    assertEquals("03:15:00.012340", rs.getObject(2, LocalTime.class).toString());
-    assertEquals("2015-03-29", rs.getObject(2, LocalDate.class).toString());
+          "1970-01-01T01:45:00.000+0100",
+          formatter.format(rs.getObject(3, Calendar.class).getTime()));
+      try {
+        formatter.format(rs.getObject(3, java.util.Date.class));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
 
-    assertTrue(rs.next());
-    // test timestamp(6)
-    for (int i = 2; i < 6; i++) {
-      assertNull(rs.getObject(i, LocalDateTime.class));
-      assertNull(rs.getObject(i, OffsetDateTime.class));
-      assertNull(rs.getObject(i, ZonedDateTime.class));
-      assertNull(rs.getObject(i, LocalDate.class));
-      assertNull(rs.getObject(i, LocalTime.class));
-      assertNull(rs.getObject(i, OffsetTime.class));
-    }
+      try {
+        formatter.format(rs.getObject(3, Date.class));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
+      assertEquals("1970-01-01T01:45:00.000+0100", formatter.format(rs.getTime(3)));
+      assertEquals(2700000, rs.getTime(3).getTime());
+      try {
+        formatter.format(rs.getDate(3));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
 
-    return rs;
+      // test datetime(6)
+      assertEquals("2015-03-29T01:45:00.012+0100", formatter.format(rs.getTimestamp(4)));
+      assertEquals(
+          "2015-03-29T01:45:00.012+0100", formatter.format(rs.getObject(4, Timestamp.class)));
+      assertEquals(
+          "2015-03-29T01:45:00.012+0100",
+          formatter.format(rs.getObject(4, Calendar.class).getTime()));
+      assertEquals(
+          "2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(4, java.util.Date.class)));
+      assertEquals("1970-01-01T01:45:00.012+0100", formatter.format(rs.getObject(4, Time.class)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(4, Date.class)));
+      assertEquals("1970-01-01T01:45:00.012+0100", formatter.format(rs.getTime(4)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(4)));
+
+      // test date(6)
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getTimestamp(5)));
+      assertEquals(
+          "2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Timestamp.class)));
+      try {
+        formatter.format(rs.getObject(5, Time.class));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Date.class)));
+      try {
+        formatter.format(rs.getTime(5));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(5)));
+      assertEquals(new Date(2015 - 1900, 2, 29), rs.getDate(5));
+      assertEquals(rs.getString(5), "2015-03-29");
+
+      assertTrue(rs.next());
+      // test timestamp(6)
+      assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getTimestamp(2)));
+      assertEquals(
+          "2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(2, Timestamp.class)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(2)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(2, Date.class)));
+      assertEquals("1970-01-01T03:15:00.012+0100", formatter.format(rs.getTime(2)));
+      assertEquals("1970-01-01T03:15:00.012+0100", formatter.format(rs.getObject(2, Time.class)));
+
+      // test time(6)
+      assertEquals("1970-01-01T03:15:00.000+0100", formatter.format(rs.getTimestamp(3)));
+      assertEquals(
+          "1970-01-01T03:15:00.000+0100", formatter.format(rs.getObject(3, Timestamp.class)));
+      assertEquals("1970-01-01T03:15:00.000+0100", formatter.format(rs.getObject(3, Time.class)));
+      try {
+        formatter.format(rs.getObject(3, Date.class));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
+      assertEquals("1970-01-01T03:15:00.000+0100", formatter.format(rs.getTime(3)));
+      assertEquals(8100000, rs.getTime(3).getTime());
+      try {
+        formatter.format(rs.getDate(3));
+        fail();
+      } catch (SQLException e) {
+        // expected exception
+      }
+
+      // test datetime(6)
+      assertEquals("2015-03-29T03:15:00.012+0200", formatter.format(rs.getTimestamp(4)));
+      assertEquals(
+          "2015-03-29T03:15:00.012+0200", formatter.format(rs.getObject(4, Timestamp.class)));
+      assertEquals("1970-01-01T03:15:00.012+0100", formatter.format(rs.getObject(4, Time.class)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(4, Date.class)));
+      assertEquals("1970-01-01T03:15:00.012+0100", formatter.format(rs.getTime(4)));
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(4)));
+
+      // test date(6)
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getTimestamp(5)));
+      assertEquals(
+          "2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Timestamp.class)));
+      try {
+        formatter.format(rs.getObject(5, Time.class));
+      } catch (SQLException e) {
+        // expected exception
+      }
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getObject(5, Date.class)));
+      try {
+        formatter.format(rs.getTime(5));
+      } catch (SQLException e) {
+        // expected exception
+      }
+      assertEquals("2015-03-29T00:00:00.000+0100", formatter.format(rs.getDate(5)));
+      assertEquals(new Date(2015 - 1900, 2, 29), rs.getDate(5));
+      assertEquals(rs.getString(5), "2015-03-29");
+
+      assertTrue(rs.next());
+      // test timestamp(6)
+      for (int i = 2; i < 6; i++) {
+        assertNull(rs.getTimestamp(i));
+        assertNull(rs.getTime(i));
+        assertNull(rs.getDate(i));
+        assertNull(rs.getObject(i, Timestamp.class));
+        assertNull(rs.getObject(i, Time.class));
+        assertNull(rs.getObject(i, Date.class));
+        assertNull(rs.getObject(i, Calendar.class));
+        assertNull(rs.getObject(i, java.util.Date.class));
+      }
+
+      rs.first();
+
+      // additional tests for java 8 objects
+      assertEquals("2015-03-29T01:45:00.012340", rs.getTimestamp(2).toLocalDateTime().toString());
+      assertEquals("2015-03-29T01:45:00.012340", rs.getObject(2, LocalDateTime.class).toString());
+      if (legacy) {
+        assertEquals(
+            "2015-03-29T01:45:00.012340+01:00[Europe/Paris]",
+            rs.getObject(2, ZonedDateTime.class).toString());
+        assertEquals(
+            "2015-03-29T01:45:00.012340+01:00", rs.getObject(2, OffsetDateTime.class).toString());
+      } else {
+        assertEquals(
+            "2015-03-28T21:45:00.012340-03:00[Canada/Atlantic]",
+            rs.getObject(2, ZonedDateTime.class).toString());
+        assertEquals(
+            "2015-03-28T21:45:00.012340-03:00", rs.getObject(2, OffsetDateTime.class).toString());
+      }
+      assertEquals("01:45:00.012340", rs.getObject(2, LocalTime.class).toString());
+      assertEquals("2015-03-29", rs.getObject(2, LocalDate.class).toString());
+
+      assertTrue(rs.next());
+      // additional tests for java 8 objects
+      assertEquals("2015-03-29T03:15:00.012340", rs.getTimestamp(2).toLocalDateTime().toString());
+      assertEquals("2015-03-29T03:15:00.012340", rs.getObject(2, LocalDateTime.class).toString());
+      if (legacy) {
+        assertEquals(
+            "2015-03-29T03:15:00.012340+02:00[Europe/Paris]",
+            rs.getObject(2, ZonedDateTime.class).toString());
+        assertEquals(
+            "2015-03-29T03:15:00.012340+02:00", rs.getObject(2, OffsetDateTime.class).toString());
+      } else {
+        assertEquals(
+            "2015-03-28T22:15:00.012340-03:00[Canada/Atlantic]",
+            rs.getObject(2, ZonedDateTime.class).toString());
+        assertEquals(
+            "2015-03-28T22:15:00.012340-03:00", rs.getObject(2, OffsetDateTime.class).toString());
+      }
+      assertEquals("03:15:00.012340", rs.getObject(2, LocalTime.class).toString());
+      assertEquals("2015-03-29", rs.getObject(2, LocalDate.class).toString());
+
+      assertTrue(rs.next());
+      // test timestamp(6)
+      for (int i = 2; i < 6; i++) {
+        assertNull(rs.getObject(i, LocalDateTime.class));
+        assertNull(rs.getObject(i, OffsetDateTime.class));
+        assertNull(rs.getObject(i, ZonedDateTime.class));
+        assertNull(rs.getObject(i, LocalDate.class));
+        assertNull(rs.getObject(i, LocalTime.class));
+        assertNull(rs.getObject(i, OffsetTime.class));
+      }
+
+      return rs;
+    }
   }
 
   @Test
   public void testDayLightNotUtC() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(doPrecisionTest && hasSuperPrivilege("testDayLight") && !sharedIsRewrite());
     TimeZone.setDefault(canadaTimeZone);
     try (Connection connection = setConnection("&serverTimezone=Europe/Paris")) {
@@ -775,7 +824,6 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
         st.executeQuery("SET GLOBAL time_zone = 'Europe/Paris'");
         rs = st.executeQuery("SHOW GLOBAL VARIABLES LIKE 'time_zone';");
         assertTrue(rs.next());
-        createTable("daylightCanada", "id int, tt TIMESTAMP(6)");
 
         Calendar quarterBeforeChangingHour =
             Calendar.getInstance(TimeZone.getTimeZone("Canada/Atlantic"));
@@ -832,7 +880,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testDayLightWithClientTimeZoneDifferent() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(doPrecisionTest && !sharedIsRewrite());
     TimeZone.setDefault(parisTimeZone);
     try (Connection connection = setConnection("&serverTimezone=UTC")) {
@@ -848,8 +896,6 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
       int offsetAfter = parisTimeZone.getOffset(quarterAfterChangingHour.getTimeInMillis());
       assertEquals(offsetAfter, 7200000);
 
-      createTable("daylight2", "id int, tt TIMESTAMP(6)");
-
       Timestamp tt = new Timestamp(quarterBeforeChangingHour.getTimeInMillis());
       tt.setNanos(123400000);
       PreparedStatement pst = connection.prepareStatement("INSERT INTO daylight2 VALUES (?, ?)");
@@ -887,7 +933,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void testNoMysqlDayLightCompatibility() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeTrue(hasSuperPrivilege("testMysqlDayLightCompatibility"));
     TimeZone.setDefault(parisTimeZone);
     try (Connection connection =
@@ -922,7 +968,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void checkSetLocalDateTimeNoOffset() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeFalse(!isMariadbServer() && strictBeforeVersion(5, 6));
     checkSetLocalDateTime(true, true, "Europe/Paris");
     checkSetLocalDateTime(true, false, "Europe/Paris");
@@ -932,7 +978,7 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
 
   @Test
   public void checkSetLocalDateTimeOffset() throws SQLException {
-    Assume.assumeFalse("true".equals(System.getenv("AURORA")));
+    Assume.assumeTrue(System.getenv("AURORA") == null && System.getenv("SKYSQL_HA") == null);
     Assume.assumeFalse(!isMariadbServer() && strictBeforeVersion(5, 6));
     checkSetLocalDateTime(true, true, "+2:00");
     checkSetLocalDateTime(true, false, "+2:00");
@@ -951,9 +997,6 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
    */
   public void checkSetLocalDateTime(boolean legacy, boolean useBinaryFormat, String timeZone)
       throws SQLException {
-    createTable(
-        "checkLocalDateTime" + legacy + useBinaryFormat,
-        " id int, tt DATETIME(6), tt2 TIME(6), tt3 varchar(100), tt4 varchar(100) ");
     TimeZone initialTimeZone = TimeZone.getDefault();
     boolean isOffset = timeZone.startsWith("+");
     if (isOffset) {
@@ -1142,7 +1185,6 @@ public class TimezoneDaylightSavingTimeTest extends BaseTest {
     TimeZone timeZone = TimeZone.getTimeZone("Asia/Kuala_Lumpur");
     TimeZone.setDefault(timeZone);
     SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss");
-    createTable("timeVerificationWithTimezone", "time_field TIME");
 
     try (Connection conn = setConnection()) {
 
